/*
    @(#) $Id: AxCommon.cpp,v 1.9 2002/10/30 19:13:20 svante Exp $

	AxCrypt - Compressing and Encrypting Wrapper and Application Launcher for Secure Local,
	Server or Web Storage of Document Files.

	Copyright (C) 2001 Svante Seleborg/Axon Data, All rights reserved.

	This program is free software; you can redistribute it and/or modify it under the terms
	of the GNU General Public License as published by the Free Software Foundation;
	either version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
	Boston, MA 02111-1307 USA

	The author may be reached at mailto:axcrypt@axondata.se and http://axcrypt.sourceforge.net
----
	AxCommon.cpp					Some constant definitions shared between server and extension

	E-mail							YYYY-MM-DD				Reason
	axcrypt@axondata.se 			2001					Initial
                                    2002-07-25              Added common code
                                    2002-08-02              Rel 1.2

*/
#include	"StdAfx.h"
#include    "CVersion.h"
#include    "commctrl.h"
#include    "CWinTrix.h"
#include    "CRegistry.h"

CPtrTo<TCHAR> gszRegKeyEventLog;

CPtrTo<TCHAR> gszAxCryptRegKey;
CPtrTo<TCHAR> gszAxCryptMessageDLL;
CPtrTo<TCHAR> gszAxCryptMutex;
CPtrTo<TCHAR> gszProgressWndName;
CPtrTo<TCHAR> gszAxCryptFileMap;
CPtrTo<TCHAR> gszAxCryptEventSend;
CPtrTo<TCHAR> gszAxCryptEventReceive;
CPtrTo<TCHAR> gszAxCryptProgID;
CPtrTo<TCHAR> gszAxCryptProgDesc;
CPtrTo<TCHAR> gszAxCryptFileExt;
CPtrTo<TCHAR> gszAxCryptCLSID;

const TCHAR *szAxCryptDefFileExt = _T(".axx");

// HKEY_CURRENT_USER
const TCHAR *szRegValKeyWrapIterations = _T("KeyWrapIterations");
const TCHAR *szRegValDefaultLanguageId = _T("DefaultLanguageId");
const TCHAR *szRegValSaveEncKey = _T("SaveEncKey");
const TCHAR *szRegValSaveDecKey = _T("SaveDecKey");
const TCHAR *szRegValNoUnsafeWipeWarn = _T("NoUnsafeWipeWarn");
const TCHAR *szRegValNoDecryptMode = _T("NoDecryptMode");
const TCHAR *szRegValServerMode = _T("ServerMode");
const TCHAR *szRegValServerErrorShell = _T("ServerErrorShellCmd");
const TCHAR *szRegValCompressLevel = _T("CompressThreshold");
const TCHAR *szRegValNoRenameMenu = _T("DisableRenameMenu");

// HKEY_LOCAL_MACHINE / gszAxCryptRegKey
const TCHAR *szRegValProductName = _T("ProductName");
const TCHAR *szRegValCLSID = _T("CLSID");
const TCHAR *szRegValFileExt = _T("FileExtension");

BOOL gfNoDecryptMode = FALSE;

//
//  Properly initialize and format all global strings, customizing
//  with product name when appropriate.
//
void
InitGlobalStrings(HINSTANCE hInstance) {
    CPtrTo<TCHAR> szIntProdName = _tcslwr(CopySz(CVersion(hInstance).IntProductName()));
    CPtrTo<TCHAR> szCompanyName = CopySz(CVersion(hInstance).CompanyName());

    gszRegKeyEventLog = FormatSz(_T("Software\\%1\\%2"), szCompanyName, szIntProdName);
    gszAxCryptRegKey = FormatSz(_T("Software\\%1\\%2"), szCompanyName, szIntProdName);
    gszAxCryptMessageDLL = FormatSz(_T("%1M.dll"), szIntProdName);
    gszAxCryptMutex = FormatSz(_T("%1Mutex"), szIntProdName);
    gszProgressWndName = FormatSz(_T("%1 Progress"), szIntProdName);
    gszAxCryptFileMap = FormatSz(_T("%1FileMapRequest"), szIntProdName);
	gszAxCryptEventSend = FormatSz(_T("%1EventSend"), szIntProdName);
	gszAxCryptEventReceive = FormatSz(_T("%1EventReceive"), szIntProdName);
    gszAxCryptProgID = FormatSz(_T("%1.File"), szIntProdName);
    gszAxCryptProgDesc = FormatSz(_T("%1 Privacy Wrapper File"), szIntProdName);

    gszAxCryptCLSID = CRegistry(HKEY_LOCAL_MACHINE, gszAxCryptRegKey, szRegValCLSID).GetSz();
    gszAxCryptFileExt = CRegistry(HKEY_LOCAL_MACHINE, gszAxCryptRegKey, szRegValFileExt).GetSz(szAxCryptDefFileExt);

    gfNoDecryptMode = CRegistry(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValNoDecryptMode).GetDword(FALSE);
}
//
//  This must be called with an atexit() if the crypto heap is active to
//  ensure they are deleted before the heap itself goes...
//
void
UnInitGlobalStrings(void) {
    gszRegKeyEventLog = NULL;
    gszAxCryptRegKey = NULL;
    gszAxCryptMessageDLL = NULL;
    gszAxCryptMutex = NULL;
    gszProgressWndName = NULL;
    gszAxCryptFileMap = NULL;
	gszAxCryptEventSend = NULL;
	gszAxCryptEventReceive = NULL;
    gszAxCryptProgID = NULL;
    gszAxCryptProgDesc = NULL;
    gszAxCryptCLSID = NULL;
    gszAxCryptFileExt = NULL;
}

//
//	Wait for event and process messages too.
//
DWORD
MessageWaitForSingleObject(HANDLE hObject, DWORD dwTimeout) {
	DWORD dwReturnCode;
	while (TRUE) {
		MSG msg ; 
		while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) { 
			TranslateMessage(&msg); 
			DispatchMessage(&msg); 
		}
		if ((dwReturnCode = MsgWaitForMultipleObjects(1,
			&hObject, 
			FALSE,
			dwTimeout,
			QS_ALLINPUT)) == (WAIT_OBJECT_0 + 1)) {
			continue; // Message in queue, no event...
		} else {
			return dwReturnCode;
		}
	}
}
//
//	Wait for event and process messages too, but also process a modeless dialog...
//
DWORD
DlgMessageWaitForSingleObject(HWND hDlg, HANDLE hObject, DWORD dwTimeout) {
	DWORD dwReturnCode;
	while (TRUE) {
		MSG msg ; 
		while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
            if (!IsWindow(hDlg) || !IsDialogMessage(hDlg, &msg)) {
			    TranslateMessage(&msg); 
			    DispatchMessage(&msg); 
            }
		}
		if ((dwReturnCode = MsgWaitForMultipleObjects(1,
			&hObject, 
			FALSE,
			dwTimeout,
			QS_ALLINPUT)) == (WAIT_OBJECT_0 + 1)) {
			continue; // Message in queue, no event...
		} else {
			return dwReturnCode;
		}
	}
}
//
//  Create and initialize a progress dialogue, returning the
//  window handle.
//
CProgressDialog::CProgressDialog() {
    m_hProgress = m_hDlg = NULL;
}

CProgressDialog::~CProgressDialog() {
    Destroy();
}

CProgressDialog::Destroy() {
    if (m_hDlg != NULL) {
        PostMessage(m_hDlg, WM_APP + 1, 0, 0);  // Destroy!
        m_hProgress = m_hDlg = NULL;
    }
}

BOOL CALLBACK
CProgressDialog::dlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch (uMsg) {
    case WM_INITDIALOG:
        SetWindowLong(hDlg, GWL_USERDATA, 2);
        SetFocus(GetDlgItem(hDlg, IDCANCEL));
        return FALSE;
    case WM_TIMER:
        if (wParam == 1 && GetWindowLong(hDlg, GWL_USERDATA) == 1) {
            KillTimer(hDlg, 1);
            ShowWindow(hDlg, SW_SHOW);
        }
        return TRUE;
        break;
    case WM_COMMAND:
        switch (wParam) {
        case IDCANCEL:
            SetWindowLong(hDlg, GWL_USERDATA, FALSE);
            return TRUE;
        default:
            break;
        }
    case WM_APP:
        SetWindowLong(hDlg, GWL_USERDATA, 2);
        KillTimer(hDlg, 1);
        ShowWindow(hDlg, SW_HIDE);
        return TRUE;
    case WM_APP + 1:
        DestroyWindow(hDlg);
        return TRUE;
    case WM_APP + 2:
        if (GetWindowLong(hDlg, GWL_USERDATA) == 2) {
            SetWindowLong(hDlg, GWL_USERDATA, 1);
            SetTimer(hDlg, 1, 1000, NULL);
        }
        return TRUE;

    case WM_DESTROY:
        KillTimer(hDlg, 1);
        return TRUE;
    default:
        break;
    }
    return FALSE;
}

HWND
CProgressDialog::Create(HINSTANCE hInstance, DWORD dwIDD, HWND hParent) {
    if (!CRegistry(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValServerMode).GetDword(FALSE)) {
        Destroy();
        m_hDlg = CreateDialog(hInstance, MAKEINTRESOURCE(dwIDD), hParent, dlgProc);
        CAssert(m_hDlg != NULL).Sys(MSG_SYSTEM_CALL, _T("CProgressDialog::Create [CreateDialog]")).Throw();

        SetWindowText(m_hDlg, CVersion(hInstance).String());

        SetWindowLong(GetDlgItem(m_hDlg, IDS_FILE), GWL_STYLE, GetWindowLong(GetDlgItem(m_hDlg, IDS_FILE), GWL_STYLE) | SS_PATHELLIPSIS);
        SetDlgItemText(m_hDlg, IDCANCEL, CMessage().AppMsg(INF_IDCANCEL).GetMsg());

        InitCommonControls();

        RECT rcClient;
        GetClientRect(m_hDlg, &rcClient); 
        int cyVScroll = GetSystemMetrics(SM_CYVSCROLL); 
        m_hProgress = CreateWindow(PROGRESS_CLASS,
                                         gszProgressWndName,
                                         WS_CHILD | WS_VISIBLE,
                                         rcClient.left,
                                         rcClient.bottom - cyVScroll,
                                         rcClient.right, cyVScroll,
                                         m_hDlg,
                                         NULL,
                                         hInstance,
                                         NULL);
        CAssert(m_hProgress != NULL).Sys(MSG_SYSTEM_CALL, _T("CProgressDialog::Create [CreateWindow]")).Throw();

        PostMessage(m_hProgress, PBM_SETRANGE, 0, MAKELPARAM(0, 100));
    }
    return m_hProgress;
}

HWND
CProgressDialog::Wnd() {
    return m_hProgress;
}