/*
    @(#) $Id: CAssert.cpp,v 1.17 2002/10/30 19:13:25 svante Exp $

	AxCrypt - Compressing and Encrypting Wrapper and Application Launcher for Secure Local,
	Server or Web Storage of Document Files.

	Copyright (C) 2001 Svante Seleborg/Axon Data, All rights reserved.

	This program is free software; you can redistribute it and/or modify it under the terms
	of the GNU General Public License as published by the Free Software Foundation;
	either version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
	Boston, MA 02111-1307 USA

	The author may be reached at mailto:axcrypt@axondata.se and http://axcrypt.sourceforge.net
----
	CAssert.cpp						Exception and Message handling

	E-mail							YYYY-MM-DD				Reason
	axcrypt@axondata.se 			2001					Initial
									2001-12-23				Added logging support
									2002-02-11				Use CHKey in LogLevel to fix handle leak
                                    2002-08-11              Rel 1.2
*/
#include	"StdAfx.h"
#include	"CFileName.h"
#include	"CVersion.h"
#include	"stdio.h"
#include	"CWinTrix.h"
#include    "CRegistry.h"
//
//  Depends on ghMsgModule and ghInstance to be initialized properly
//
const TCHAR *szRegValEventLogLevel = _T("EventLogLevel");

CMessage::CMessage(HWND hWnd) : m_wMaxWidth(50) {
	m_hWnd = hWnd;
}

//
//	Get a Msg from system or application message table,
//	with up to two arguments (or NULL).
//	The third argument is the previous message, if any.
//
//	Return the generated message.
//	
CStrPtr
CMessage::Msg(DWORD dwFlag, DWORD dwMsgID, CStrPtr utStr1, CStrPtr utStr2) {
	LPCTSTR aszArg[4];

    // We must ensure to delete the returned string...
    CVersion version(ghInstance);
    const TCHAR *szExtProductName = version.ExtProductName();

    aszArg[0] = szExtProductName;
    aszArg[1] = utStr1;
	aszArg[2] = utStr2;
	aszArg[3] = m_utMsg;

	LPTSTR szMsg;
	//
	//	Now, this should not really be that hard... But I can't make it work as I understand
	//	the documentation - someone please explain? I'm sure there is an explanation.
	//	Anyway - what I want is to have the following preference order:
	//	1 - Registry specified language
	//	2 - System Default Language
	//	3 - User selected language
	//	4 - Any
	//  Supposedly '0' as language ID in that parameter should most of the trick.
	//	But I can't get it to work. It always picks	US English. So I first try to force it
	//	using the System Default, then the thread locale, if that doesn't work,
	//	I let it pick up whatever it can find.
	//
    DWORD dwLanguageId = CRegistry(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValDefaultLanguageId).GetDword(GetSystemDefaultLangID());
    //
    //	Note to developers: Newer versions of the message compiler produces Unicode messages, there
    //	is no switch do disable this. This in turn, causes messages to be erroneously formatted.
    //	Currently AxCrypt is not Unicode-enabled, thus please use the old message compiler. The compiler
    //	may reside in for example Microsoft SDK\bin or Microsoft Visual Studio\VC98\bin. A good (for
    //	this purpose) version is dated 1998-04-20 and is named mc.exe.
    //
	if (!FormatMessage(
			FORMAT_MESSAGE_ALLOCATE_BUFFER | dwFlag | FORMAT_MESSAGE_ARGUMENT_ARRAY | m_wMaxWidth,	// Max width of text
			ghMsgModule, dwMsgID, dwLanguageId,
			(LPTSTR)&szMsg,	0, (va_list *)aszArg)) {
		if (!FormatMessage(
				FORMAT_MESSAGE_ALLOCATE_BUFFER | dwFlag | FORMAT_MESSAGE_ARGUMENT_ARRAY | m_wMaxWidth,	// Max width of text
				ghMsgModule, dwMsgID, (WORD)GetThreadLocale(),
				(LPTSTR)&szMsg,	0, (va_list *)aszArg)) {
			if (!FormatMessage(
					FORMAT_MESSAGE_ALLOCATE_BUFFER | dwFlag | FORMAT_MESSAGE_ARGUMENT_ARRAY | m_wMaxWidth,	// Max width of text
					ghMsgModule, dwMsgID, 0,
					(LPTSTR)&szMsg,	0, (va_list *)aszArg)) {
				// Pretty catastrophic - can't even read the proper error message.
				return _T("Error - cannot read message definitions.");
			}
		}
	}
	CStrPtr	utStrReturn = szMsg;						// Save the text
	(void)LocalFree(szMsg);						// Free system allocated memory.
	return utStrReturn;
}
//
//  Show a dialog, or if in server mode, log the message to the log-
//  file, and return the appropriate affirmative IDOK, IDYES, IDIGNORE, IDCANCEL etc.
//
int
CMessage::ShowDialog(UINT uiStyleMask) {
    if (!CRegistry(HKEY_CURRENT_USER, gszRegKeyEventLog, szRegValServerMode).GetDword(FALSE)) {
        if (m_hWnd == NULL) MySetForegroundWindow();

	    return MessageBox(
		    m_hWnd == NULL ? /*GetForegroundWindow()*/NULL/*<SS 2002-09-01 testing...>*/ : m_hWnd,
		    m_utMsg,
		    CVersion(ghInstance).String(),
		    MB_TOPMOST | MB_SETFOREGROUND | uiStyleMask);
    } else {
        LogEvent(0);
        switch (uiStyleMask & MB_TYPEMASK) {
        case MB_OK:
        case MB_OKCANCEL:
            return IDOK;

        case MB_ABORTRETRYIGNORE:
            return IDIGNORE;

        case MB_YESNOCANCEL:
        case MB_YESNO:
            return IDYES;

        case MB_RETRYCANCEL:
            return IDCANCEL;

        default:
            return IDOK;
        }
    }
}
//
//	Return the log level in the registry, or zero if the key
//	does not exist.
DWORD
CMessage::LogLevel() {
    return CRegistry(HKEY_CURRENT_USER, gszRegKeyEventLog, szRegValEventLogLevel).GetDword(0);
}

CMessage&
CMessage::AppMsg(DWORD dwMsgID, CStrPtr utArg1, CStrPtr utArg2) {
	m_utMsg = Msg(FORMAT_MESSAGE_FROM_HMODULE, dwMsgID, utArg1, utArg2);
	return *this;
}

CMessage&
CMessage::AppMsg(DWORD dwMsgID, int iArg1, CStrPtr szArg2) {
	return AppMsg(dwMsgID, *CPtrTo<CStrPtr>(FmtInt(_T("%d"), iArg1)), szArg2);
}

CMessage&
CMessage::AppMsg(DWORD dwMsgID, CStrPtr szArg1, int iArg2) {
	return AppMsg(dwMsgID, szArg1, *CPtrTo<CStrPtr>(FmtInt(_T("%d"), iArg2)));
}

CStrPtr&
CMessage::GetMsg() {
	return m_utMsg;
}

CMessage&
CMessage::SysMsg(DWORD dwMsgID, CStrPtr utArg1, CStrPtr utArg2) {
	m_utMsg = Msg(FORMAT_MESSAGE_FROM_SYSTEM, dwMsgID, utArg1, utArg2);
	return *this;
}

CMessage&
CMessage::Wrap(WORD wMaxWidth) {
    if (wMaxWidth) {
	    m_wMaxWidth = wMaxWidth;
    } else {
        m_wMaxWidth = FORMAT_MESSAGE_MAX_WIDTH_MASK;
    }
	return *this;
}

int
CMessage::ShowError(UINT uiStyleMask) {
	return ShowDialog(uiStyleMask | MB_ICONERROR);
}

int
CMessage::ShowInfo(UINT uiStyleMask) {
	return ShowDialog(uiStyleMask | MB_ICONINFORMATION);
}

int
CMessage::ShowWarning(UINT uiStyleMask) {
	return ShowDialog(uiStyleMask | MB_ICONWARNING);
}

int
CMessage::ShowQuestion(UINT uiStyleMask) {
	return ShowDialog(uiStyleMask | MB_ICONQUESTION);
}
void
CMessage::LogEvent(DWORD dwLogLevel) {
	if (LogLevel() >= dwLogLevel) {
		CFileName utLogFile;

        utLogFile.SetPath2SysTempDir();

		utLogFile.SetDir(CStrPtr(utLogFile.GetDir()) + CStrPtr(_T("\\")));
		utLogFile.SetName(CVersion(ghInstance).IntProductName());
		utLogFile.SetExt(_T(".log"));
		HANDLE hLogFile = CreateFile(utLogFile.Get(), GENERIC_WRITE, 0, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL , NULL);
        if (hLogFile == INVALID_HANDLE_VALUE) {
            // Here we should probably do something smart to hande an error, but
            // we can't just throw the exception, as we risk infinite recursion
            // and other bad things.
            return;
        }
		CAssert(SetFilePointer(hLogFile, 0, NULL, FILE_END) != 0xffffffff).Sys().Throw();

		DWORD dwBytesWritten;
		TCHAR szDateTime[100];
		SYSTEMTIME stLocalTime;
		GetLocalTime(&stLocalTime);
		_stprintf(szDateTime, "%04d-%02d-%02d %02d.%02d.%02d ", stLocalTime.wYear, stLocalTime.wMonth, stLocalTime.wDay, stLocalTime.wHour, stLocalTime.wMinute, stLocalTime.wSecond);
		(void)WriteFile(hLogFile, szDateTime, _tcslen(szDateTime) * sizeof TCHAR, &dwBytesWritten, NULL);

        // Quick & dirty remove '\r' and '\n' from the message.
        CPtrTo<TCHAR> szOneLine = CopySz(m_utMsg);
        int iSpaceCount = 0;
        TCHAR *pIn, *pOut;
        // It is guaranteed that the output is not longer than the input
        for (pIn = pOut = szOneLine; *pIn; pIn++) {
            switch (*pIn) {
            case _T('\r'):
            case _T('\n'):
            case _T('\t'):
                if (!iSpaceCount) iSpaceCount = 1;
                break;
            case _T(' '):
                iSpaceCount++;
                break;
            default:
                if (iSpaceCount) {
                    do {
                        *pOut++ = _T(' ');
                    } while (--iSpaceCount);
                }
                *pOut++ = *pIn;
            }
        }
        *pOut = _T('\0');

		(void)WriteFile(hLogFile, szOneLine, _tcslen(szOneLine) * sizeof TCHAR, &dwBytesWritten, NULL);
        (void)WriteFile(hLogFile, _T("\r\n"), _tcslen(_T("\r\n")) * sizeof TCHAR, &dwBytesWritten, NULL);
		(void)CloseHandle(hLogFile);
	}
}

// Must be deleted by caller...
CStrPtr*
CMessage::FmtInt(LPTSTR szFmt, int iInt) {
	CStrPtr *pszStr = new CStrPtr(40);			// Should be enough...
	(void)_stprintf(*pszStr, szFmt, iInt);
	return pszStr;
}

//
//	Default constructor
//
CAssert::CAssert() {
}
//
//	Copy constructor
//
CAssert::CAssert(CAssert& utError) {
	m_bOk = utError.m_bOk;
	m_dwLastError = utError.m_dwLastError;
	m_utMsg = utError.m_utMsg;
}
//
//	The normally called constructor.
//
CAssert::CAssert(BOOL bOk) {
	m_bOk = bOk;
	m_dwLastError = GetLastError();
}
//
//  Alternate for other error code sources than GetLastError()
//
CAssert::CAssert(BOOL bOk, DWORD dwError) {
    m_bOk = bOk;
    m_dwLastError = dwError;
}
//
//	Display a proper message box for a ready formatted message.
//
CAssert&
CAssert::Show() {
	m_utMsg.ShowError();
	return *this;
}
//
//	Get the actual message so far.
//
CStrPtr&
CAssert::GetMsg() {
	return m_utMsg.GetMsg();
}
//
//	Get the last error code to the caller.
//
DWORD
CAssert::LastError() {
	return m_dwLastError;
}
//
//	Throw an exception...
//
void
CAssert::Throw() {
    if (!m_bOk) {
        // Now this is tricky... When we throw an exception, the CAssert
        // object contains dynamically allocated memory, thus we must
        // adjust for a leak here.
        HEAP_CHECK_BEGIN(_T("CAssert::Throw()"), TRUE);
        throw *this;
        HEAP_CHECK_END
    }
}
//
//	Generate application error msg, MsgID
//	%2 = N/A
//	%3 = szCtx
//	%4 = Previous message, if any
//
CAssert&
CAssert::App(DWORD dwMsgID, LPTSTR szCtx) {
	if (!m_bOk) {
		m_utMsg.AppMsg(dwMsgID, NULL, szCtx);
		m_dwLastError = dwMsgID;						// Remember the Alamo
	}
	return *this;
}
//
//	Generate system error directly from GetLastError()
//	%2 = N/A
//	%3 = N/A
//	%4 = Previous message, if any
//
CAssert&
CAssert::Sys() {
	if (!m_bOk) m_utMsg.SysMsg(m_dwLastError);
	return *this;
}
//
// Generate system error msg from MsgID
//	%2 = LastError = %2
//	%3 = szCtx = %3
//	%4 = Previous message, if any
//
CAssert&
CAssert::Sys(DWORD dwMsgID, LPTSTR szCtx) {
	if (!m_bOk) {
		m_utMsg.AppMsg(
			dwMsgID,
			CMessage().SysMsg(m_dwLastError).GetMsg(),
			szCtx);
		m_dwLastError = dwMsgID;						// Remember the Alamo
	}
	return *this;
}
//
//	Generate application file error, using msgID
//	%2 = LastError
//	%3 = szFile
//	%4 = Previous message, if any
//
CAssert&
CAssert::File(DWORD dwMsgID, LPTSTR szFile) {
	if (!m_bOk) {
		m_utMsg.AppMsg(
			dwMsgID,
			CMessage().SysMsg(m_dwLastError).GetMsg(),
			CFileName(szFile).GetTitle());
		m_dwLastError = dwMsgID;						// Remember the Alamo
	}
	return *this;
}

CAssert&
CAssert::File(LPTSTR szFile) {
	if (!m_bOk) {
		m_utMsg.SysMsg(m_dwLastError, CFileName(szFile).GetTitle());
	}
	return *this;
}
