/*
    @(#) $Id: CChildProc.cpp,v 1.5 2002/08/20 20:01:26 svante Exp $

	AxCrypt - Compressing and Encrypting Wrapper and Application Launcher for Secure Local,
	Server or Web Storage of Document Files.

	Copyright (C) 2001 Svante Seleborg/Axon Data, All rights reserved.

	This program is free software; you can redistribute it and/or modify it under the terms
	of the GNU General Public License as published by the Free Software Foundation;
	either version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
	Boston, MA 02111-1307 USA

	The author may be reached at mailto:axcrypt@axondata.se and http://axcrypt.sourceforge.net
----
	CChildProc.cpp					Utility functions of a more esoteric nature, doing the
									things that really should just require some simple API
									calls, but that for unknown reasons the designers of
									win32 still has not realized, even now when they are
									launching the fourth version of it! I mean, really,
									should you need to implement a process handle-to process
									id mapping by using side effects of the performance
									library?
									Unfortunately there is some OS-dependent code here, and
									even a few sort-of undocumented calls, at least calls
									that are not part of the Win32 API as such.

	E-mail							YYYY-MM-DD				Reason
	axcrypt@axondata.se 			2001					Initial

*/
#include	"StdAfx.h"
#include	"CChildProc.h"

const int CChildProc::ciNumQuanta = 10;

DWORD *
CChildProc::NtEnumProcesses(DWORD *pdwNrProc) {
	*pdwNrProc = 0;
	HINSTANCE hPSAPI = LoadLibrary(_T("psapi.dll"));
	if (hPSAPI == NULL) return NULL;

	ENUMPROCESSES pfEnumProcesses =
		(ENUMPROCESSES)GetProcAddress(
			hPSAPI,
			"EnumProcesses");
	if (pfEnumProcesses) {
		DWORD cbTableSize;
		*pdwNrProc = 512;
		DWORD *pdwProcTable = NULL;
		do {
			*pdwNrProc += *pdwNrProc;

			if (pdwProcTable != NULL) delete pdwProcTable;
			pdwProcTable = new DWORD[*pdwNrProc];
			CAssert(pdwProcTable != NULL).App(MSG_MEMORY_ALLOC, "CChildProc::NtEnumProcesses").Throw();

			if (!pfEnumProcesses(pdwProcTable, *pdwNrProc * sizeof DWORD, &cbTableSize)) {
				delete pdwProcTable;
				*pdwNrProc = 0;
				FreeLibrary(hPSAPI);
				return NULL;
			}
		} while (cbTableSize == (*pdwNrProc * sizeof DWORD));
		*pdwNrProc = cbTableSize / sizeof DWORD;
		FreeLibrary(hPSAPI);
		return pdwProcTable;
	} else {
		FreeLibrary(hPSAPI);
		return NULL;
	}
}
//
//	Create the object with a snapshot of the current kids
//	of the given process.
//
CChildProc::CChildProc(DWORD dwProcId) {
	Init();
	TakeSnapshot(dwProcId);
}

CChildProc::CChildProc() {
	Init();
}

CChildProc::~CChildProc() {
	if (m_pdwOldChildren != NULL) delete m_pdwOldChildren;
	if (m_pdwProcTable != NULL) delete m_pdwProcTable;
}

DWORD CChildProc::GetNewChild() {
	return m_dwLastNewChild;
}

void
CChildProc::Init() {
	m_pdwProcTable = m_pdwOldChildren = NULL;
	m_iSizOldChildren = 0;
	m_iNumOldChildren = 0;
	m_dwLastNewChild = 0;
}

void CChildProc::AddChild(DWORD dwProcId) {
	// First see if we already know this kid
	for (int i=0; i < m_iNumOldChildren; i++) {
		if (m_pdwOldChildren[i] == dwProcId) {
			return;
		}
	}

	// Yet Another Kid
	// First see if there is room, or we need to expand.
	if (m_iNumOldChildren == m_iSizOldChildren) {
		DWORD *pdwNewList = new DWORD[m_iSizOldChildren += ciNumQuanta];
		CAssert(pdwNewList != NULL).App(MSG_MEMORY_ALLOC, "CChildProc::AddChild").Throw();

		CopyMemory(pdwNewList, m_pdwOldChildren, m_iNumOldChildren * sizeof *m_pdwOldChildren);
		if (m_pdwOldChildren != NULL) delete m_pdwOldChildren;
		m_pdwOldChildren = pdwNewList;
	}
	// Add YAK
	m_dwLastNewChild = m_pdwOldChildren[m_iNumOldChildren++] = dwProcId;
}
//
//	Take an incremental snapshop, saving the ID of the last new entry.
//
void CChildProc::TakeSnapshot(DWORD dwProcId) {
	// OS-dependent code here, but the strategy is not to try to look at OS-versions,
	// but instead try different possibilities, thus handling unkonwn os-version reports
	// more gracefully and in general being more general - if such a word can be used
	// for this kind of stupid stuff.

	// In general - we fail silently and gracefully in all cases below without visible
	// signs of distress - memory allocation excepted.

	// Try to get the addresses of the Toolhelp32 kit
	HMODULE hKernel32 = GetModuleHandle(_T("kernel32.dll"));
	if (hKernel32 != NULL) {
		CREATETOOLHELP32SNAPSHOT pfCreateToolhelp32Snapshot =
			(CREATETOOLHELP32SNAPSHOT)GetProcAddress(
				hKernel32,
				"CreateToolhelp32Snapshot");
		PROCESS32FIRST pfProcess32First =
			(PROCESS32FIRST)GetProcAddress(
				hKernel32,
				"Process32First");
		PROCESS32NEXT pfProcess32Next =
			(PROCESS32NEXT)GetProcAddress(
				hKernel32,
				"Process32Next");

		// We like to use the toolhelp first - mostly because it is newer and more
		// frequently available.
		if (pfCreateToolhelp32Snapshot != NULL &&
			pfProcess32First != NULL &&
			pfProcess32Next != NULL) {
			
			HANDLE hSnapshot = pfCreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
			PROCESSENTRY32 stProcessEntry;
			stProcessEntry.dwSize = sizeof stProcessEntry;
		
			if (pfProcess32First(hSnapshot, &stProcessEntry)) {
				do {
					// if this has the given id as it's parent we found one!
					if (stProcessEntry.th32ParentProcessID == dwProcId) {
						AddChild(stProcessEntry.th32ProcessID);
					}
				} while (pfProcess32Next(hSnapshot, &stProcessEntry));
			}
			return;
		}
	}
	
	// Next let's try the NT-stuff - dependent on PSAPI.DLL, which the install
	// may, or may not, have installed on the system.
	// Try to get the address of the native API-function
	PROCNTQSIP pfNtQueryInformationProcess = (PROCNTQSIP)GetProcAddress(
		GetModuleHandle("ntdll.dll"),
		"NtQueryInformationProcess");

	if (pfNtQueryInformationProcess != NULL) {
		DWORD dwNrOfProcs;
		if ((m_pdwProcTable = NtEnumProcesses(&dwNrOfProcs)) != NULL) {
			for (DWORD i = 0; i < dwNrOfProcs; i++) {
				HANDLE hProc = OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, m_pdwProcTable[i]);
				if (hProc) {
					PROCESS_BASIC_INFORMATION pbi;
					ZeroMemory(&pbi, sizeof pbi);

					LONG lStatus = pfNtQueryInformationProcess(
						hProc,
						ProcessBasicInformation,
						(PVOID)&pbi,
						sizeof PROCESS_BASIC_INFORMATION,
						NULL);
					CloseHandle(hProc);
					if (lStatus == 0) {
						// this process has the give id as its parent, lets add it!
						if (pbi.InheritedFromUniqueProcessId == dwProcId) {
							AddChild(pbi.UniqueProcessId);
						}
					}
				}
			}
		}
	}
}
//
//	Do a variety of tests to return an indication if we think that installing
//	psapi.dll will imporove our chances of sucessfully identifying child
//	processes.
//
//	Basically, we look at the platform version, and then if the entry is missing
//	we recommend installation by returning non-zero value.
//
//	If we get unexpected errors below, we do not recommend installing psapi.dll,
//	as we are probably to far out on a limb anyway, either in unexpected new os-
//	versions, or stuff simply to old to imagine.
//
DWORD
CChildProc::NeedPsapi() {
	// Check if the toolhelp library seems to be implemented, if so we don't care
	// about os-versions etc!
	CREATETOOLHELP32SNAPSHOT pfCreateToolhelp32Snapshot =
		(CREATETOOLHELP32SNAPSHOT)GetProcAddress(
			GetModuleHandle(_T("kernel32.dll")),
			"CreateToolhelp32Snapshot");
	if (pfCreateToolhelp32Snapshot != NULL) return 0;
		
	OSVERSIONINFO stOsVersion;
	stOsVersion.dwOSVersionInfoSize = sizeof stOsVersion;
	// check if we are running Windows NT < 5. The call will probably fail (i can't test this)
	// on NT 3.1.
	if (GetVersionEx(&stOsVersion)) {
		if (stOsVersion.dwPlatformId == VER_PLATFORM_WIN32_NT) {
			if (stOsVersion.dwMajorVersion < 5) {
				HINSTANCE hPSAPI = LoadLibrary(_T("psapi.dll"));
				if (hPSAPI != NULL) FreeLibrary(hPSAPI);
				if (hPSAPI == NULL) {
					return 1;				// Couldn't load the library, need it!
				} else {
					return 0;
				}
			}
		}
	}
	return 0;
}
