/*
    @(#) $Id: CFile.cpp,v 1.11 2002/12/09 21:14:09 svante Exp $

	AxCrypt - Compressing and Encrypting Wrapper and Application Launcher for Secure Local,
	Server or Web Storage of Document Files.

	Copyright (C) 2001 Svante Seleborg/Axon Data, All rights reserved.

	This program is free software; you can redistribute it and/or modify it under the terms
	of the GNU General Public License as published by the Free Software Foundation;
	either version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
	Boston, MA 02111-1307 USA

	The author may be reached at mailto:axcrypt@axondata.se and http://axcrypt.sourceforge.net
----
	CFile.cpp						Basic open/read/write/memory map operations on files.

	E-mail							YYYY-MM-DD				Reason
	axcrypt@axondata.se 			2001					Initial
									2001-11-29				Moved temp-files to the temp dir.
                                    2002-08-11              Rel 1.2

*/
#include	"StdAfx.h"
#include	"CFileName.h"
#include	"CFile.h"
#include	"CXform.h"
#include    <winioctl.h>
//
//	File-operations, the source and destination of a transformation.
//
CFileIO::CFileIO() {
	m_hFile = INVALID_HANDLE_VALUE;
	m_hMapping = NULL;
	m_poView = NULL;
    m_cbView = 0;
	m_qwFileSize = m_qwFileOffset = 0;
	m_szFileName = NULL;
	m_fForceWriteThru = m_fDeleteOnClose = m_fIsTmp = FALSE;
}
//
//	Unmap view, close mapping and close the file itself.
//
CFileIO::~CFileIO() {
	Close();
	if (m_szFileName != NULL) delete m_szFileName;
}
//
//	dwReadWrite would normally be GENERIC_READ or GENERIC_READ|GENERIC_WRITE
//	dwOpenMode would normally be OPEN_EXISTING, CREATE_NEW or CREATE_ALWAYS
//
BOOL
CFileIO::CreateEx(LPTSTR szFileName, DWORD dwReadWrite, DWORD dwOpenMode, DWORD dwShareMode, DWORD dwFlags) {
	if (m_szFileName != NULL) delete m_szFileName;
	m_szFileName = _tcscpy(new _TCHAR[_tcslen(szFileName)+1], szFileName);
    //
    //  As we sometimes have difficult-to-handle problems with the shell locking
    //  files for us, we simply brute-force for now by insisting if we get a
    //  sharing violation. This seems to be really OS-dependent as well as
    //  situation dependent (it get's worse with network drives for example).
    //  This should really, really, be solved in the Shell Extension once and for
    //  all in a way that makes it always let go of files!!!
    //
    for (int i = 0; i < 50; i++) {
  	    m_hFile = CreateFile(
		    m_szFileName,
		    dwReadWrite,
		    dwShareMode,
		    NULL,
		    dwOpenMode,
		    dwFlags,
		    NULL);
        if (m_hFile != INVALID_HANDLE_VALUE) {
            break;
        } else {
            if (GetLastError() != ERROR_SHARING_VIOLATION) {
                break;
            }
        }
        Sleep(20);
    }
	if (m_hFile != INVALID_HANDLE_VALUE) {
		LARGE_INTEGER li;
		li.LowPart = GetFileSize(m_hFile, (DWORD*)&li.HighPart);
		CAssert(li.LowPart != 0xFFFFFFFF || GetLastError() == NO_ERROR).File(szFileName).Throw();
		//
		//	We now handle full 64-bit values for filesizes.
		//
		m_qwFileSize = li.QuadPart;
	}
	return m_hFile != INVALID_HANDLE_VALUE;
}
//
//	dwReadWrite would normally be GENERIC_READ or GENERIC_READ|GENERIC_WRITE
//	dwOpenMode would normally be OPEN_EXISTING, CREATE_NEW or CREATE_ALWAYS
//
void
CFileIO::Open(LPTSTR szFileName, BOOL fForceWriteThru, DWORD dwReadWrite, DWORD dwShareMode) {
	m_fDeleteOnClose = FALSE;
    DWORD dwFlagsAndAttributes = FILE_FLAG_SEQUENTIAL_SCAN;
    if (fForceWriteThru) {
        dwFlagsAndAttributes |= FILE_FLAG_WRITE_THROUGH;
    }
	CAssert(CreateEx(szFileName, dwReadWrite, OPEN_EXISTING, dwShareMode, dwFlagsAndAttributes)).Sys().Throw();
}
//
//  Open directory.
//
void
CFileIO::OpenDir(LPTSTR szFileName, DWORD dwReadWrite, DWORD dwShareMode) {
    m_fDeleteOnClose = FALSE;
	CAssert(CreateEx(szFileName, dwReadWrite, OPEN_EXISTING, dwShareMode, FILE_FLAG_BACKUP_SEMANTICS)).Sys().Throw();
}

//
//	dwReadWrite would normally be GENERIC_READ or GENERIC_READ|GENERIC_WRITE
//	dwOpenMode would normally be OPEN_EXISTING, CREATE_NEW or CREATE_ALWAYS
//
void
CFileIO::Create(LPTSTR szFileName, BOOL fForceWriteThru, DWORD dwReadWrite, DWORD dwOpenMode) {
	m_fDeleteOnClose = TRUE;
    DWORD dwFlagsAndAttributes = FILE_FLAG_SEQUENTIAL_SCAN;
    if (fForceWriteThru) {
        dwFlagsAndAttributes |= FILE_FLAG_WRITE_THROUGH;
    }
	CAssert(CreateEx(szFileName, dwReadWrite, dwOpenMode, 0, dwFlagsAndAttributes)).Sys().Throw();
}
//
//	Create a writeable temp-file. Throw an TAssert exception on error.
//
void
CFileIO::MakeTmp(LPTSTR szTempFile, BOOL fForceWriteThru) {
	if (m_szFileName != NULL) delete m_szFileName;
	m_szFileName = _tcscpy(new _TCHAR[(1+_tcslen(szTempFile)) * sizeof TCHAR], szTempFile);
	
    DWORD dwFlagsAndAttributes = FILE_ATTRIBUTE_TEMPORARY |
		FILE_ATTRIBUTE_HIDDEN |
		FILE_FLAG_RANDOM_ACCESS |
		FILE_FLAG_DELETE_ON_CLOSE;
    if (fForceWriteThru) {
        dwFlagsAndAttributes |= FILE_FLAG_WRITE_THROUGH;
    }
    m_hFile = CreateFile(
		m_szFileName,
		GENERIC_READ | GENERIC_WRITE,
		0,
		NULL,
		CREATE_ALWAYS,
        dwFlagsAndAttributes,
		NULL);
	CAssert(m_hFile != INVALID_HANDLE_VALUE).File(MSG_MAKE_TMP, m_szFileName).Throw();
	m_fIsTmp = TRUE;
	m_fDeleteOnClose = FALSE;			// Actually yes, but this the system handles.
}
//
//  Close, and if necessary, flush view of file.
//
void
CFileIO::CloseView() {
    if (m_poView != NULL) {
		if (m_fForceWriteThru) {
			CAssert(FlushViewOfFile(m_poView, m_cbView)).Sys().Throw();
		}
        CAssert(UnmapViewOfFile(m_poView)).Sys(MSG_MAP_VIEW).Throw();
        m_poView = NULL;
    }
}
//
//	If there is an open view, unmap it.
//	if there is an open mapping, close it.
//
void
CFileIO::CloseMap() {
    CloseView();
	if (m_hMapping != NULL) CAssert(CloseHandle(m_hMapping)).Sys().Throw();
	m_hMapping = NULL;
}
//
//	If the handle is open, close it
//
void
CFileIO::Close(BOOL fForceKeepOnClose) {
    CloseMap();
	if (m_hFile != INVALID_HANDLE_VALUE) {
		CAssert(CloseHandle(m_hFile)).Sys().Throw();
		if (m_fDeleteOnClose && !fForceKeepOnClose) {
			CAssert(DeleteFile(m_szFileName)).Sys().Throw();
		}
	}
	m_hFile = INVALID_HANDLE_VALUE;
	m_fForceWriteThru = m_fDeleteOnClose = m_fIsTmp = FALSE;
}
//
//	Wipe this file, and mark it for deletion on close
//
void
CFileIO::WipeTemp(HWND hProgressWnd) {
	// Just a placeholder for the transformer logic used by the wiper.
	CFileDummy utDummyIn;
	utDummyIn.m_qwFileSize = m_qwFileSize;
	
	// Trash ourselves.
    CloseView();            // Close any open view, using the current write thru setting
    SetWriteThru(TRUE);     // Ensure over-write data actually get's written.
	CWipeTemp(hProgressWnd).XformData(utDummyIn, *this);
	if (!m_fIsTmp) m_fDeleteOnClose = TRUE;
}
//
//	Wipe this file, and mark it for deletion on close
//
void
CFileIO::WipeData(HWND hProgressWnd) {
	// Just a placeholder for the transformer logic used by the wiper.
	CFileDummy utDummyIn;
	utDummyIn.m_qwFileSize = m_qwFileSize;
	
	// Trash ourselves.
    CloseView();            // Close any open view, using the current write thru setting
    SetWriteThru(TRUE);     // Ensure over-write data actually get's written.
	CWipeData(hProgressWnd).XformData(utDummyIn, *this);
	if (!m_fIsTmp) m_fDeleteOnClose = TRUE;
}
//
//	Just wipe the first 1024 bytes, or less, of the file.
//
void
CFileIO::WipeShort() {
	// Just a placeholder for the transformer logic used by the wiper.
	CFileDummy utDummyIn;
	utDummyIn.m_qwFileSize = Min(m_qwFileSize, 1024);
	
	// Trash ourselves.
    CloseView();            // Close any open view, using the current write thru setting
    SetWriteThru(TRUE);     // Ensure over-write data actually get's written.
	CWipeData(NULL).XformData(utDummyIn, *this);
	if (!m_fIsTmp) m_fDeleteOnClose = TRUE;
}
//
//	SetFilePointer-wrapper
//
void
CFileIO::SetFilePointer(LONGLONG llPos) {
	CAssert(::SetFilePointer(m_hFile, (DWORD)llPos, &((PLARGE_INTEGER)&llPos)->HighPart, FILE_BEGIN) != 0xFFFFFFFF).Sys().Throw();
}
//
//	SetEndOfFile-wrapper
//
void
CFileIO::SetEndOfFile() {
	CAssert(::SetEndOfFile(m_hFile)).Sys().Throw();
}
//
//
void
CFileIO::SetFileTimes(SFileTimes *pFileTimes) {
	CAssert(::SetFileTime(
		m_hFile,
		&pFileTimes->CreationTime,
		&pFileTimes->LastAccessTime,
		&pFileTimes->LastWriteTime)).Sys().Throw();
}

SFileTimes *
CFileIO::GetFileTimes() {
	CAssert(::GetFileTime(
		m_hFile,
		&m_FileTimes.CreationTime,
		&m_FileTimes.LastAccessTime,
		&m_FileTimes.LastWriteTime)).Sys().Throw();
	return &m_FileTimes;
}
//
//	Return the file-name of the associated file.
//
LPTSTR
CFileIO::FileName() {
	return m_szFileName;
}
//
//	Map a view of a file, handling alignment and size requirements etc.
//
//	dwMode is either FILE_MAP_READ or FILE_MAP_WRITE
//	dwOffset is the offset into the current operation, i.e. relative to start of operation
//		which is held in m_dwFileOffset.
//	puiSize is input the attempted/wanted size of buffer, output the actual received.
//
BYTE *CFileIO::MapView(DWORD dwMode, QWORD qwOffset, unsigned int* puiSize) {
	DWORD dwAllocationGranularity;
	// Get system allocation granularity to use with the memory mapping functions
	{
		SYSTEM_INFO SystemInfo;
		GetSystemInfo(&SystemInfo);	// No error return!
		dwAllocationGranularity = SystemInfo.dwAllocationGranularity;
		CAssert((MAX_VIEW_SIZE - MAX_VIEW_SIZE % dwAllocationGranularity) > 0).App(MSG_VIEW_SIZE).Throw();
	}

	QWORD qwTotalOffset = m_qwFileOffset + qwOffset;
	DWORD dwViewOffset = (DWORD)(qwTotalOffset % dwAllocationGranularity);
	qwTotalOffset -= dwViewOffset;
	// If we are attempting a map beyond known file-size,
	// close the mapping as we need a new!
	if ((qwTotalOffset + dwViewOffset + *puiSize) > m_qwFileSize) {
		CloseMap();
	}
	// If we are still trying for larger than maximum set View size, reduce it.
	if ((dwViewOffset + *puiSize) > (MAX_VIEW_SIZE - MAX_VIEW_SIZE % dwAllocationGranularity)) {
		*puiSize = (MAX_VIEW_SIZE - MAX_VIEW_SIZE % dwAllocationGranularity) - dwViewOffset;
	}

	if ((qwTotalOffset + dwViewOffset + *puiSize) > m_qwFileSize) {
		m_qwFileSize = qwTotalOffset + dwViewOffset + *puiSize;
	}

	// If we don't have a mapping, create one for the entire file, if necessary it will grow.
	if (m_hMapping == NULL) {
		m_hMapping = CreateFileMapping(
			m_hFile,
			NULL,
			dwMode == FILE_MAP_READ ? PAGE_READONLY : PAGE_READWRITE,
			((PULARGE_INTEGER)&m_qwFileSize)->HighPart,
			(DWORD)m_qwFileSize,
			NULL);
        // To enable debugging of this
        if (m_hMapping == NULL) {
		    CAssert(m_hMapping != NULL).Sys(MSG_MAP_VIEW).Throw();
        }
	}
	// If necessary flush old view, and unmap it.
    CloseView();
	m_poView = (BYTE *)MapViewOfFile(
		m_hMapping,
		dwMode,
		((PULARGE_INTEGER)&qwTotalOffset)->HighPart,
		(DWORD)qwTotalOffset,
		m_cbView = dwViewOffset + *puiSize);
    // To enable debugging of this
    if (m_poView == NULL) {
	    CAssert(m_poView != NULL).Sys(MSG_MAP_VIEW).Throw();
    }
	return m_poView + dwViewOffset;
}

LPTSTR
CFileIO::GetFileName() {
	return m_szFileName;
}
//
//  Set the file or directory compression and encryption state to 'off'.
//
//  This is primarily intended for temp directories, where we want to
//  ensure that they are wipeable.
//
void
CFileIO::SetNotCompressed() {
    USHORT usFormat;
    DWORD dwBytesRet;

    // We fail silently here, as the attribute is not supported on Win95
    if (DeviceIoControl(m_hFile, FSCTL_GET_COMPRESSION, NULL, 0, &usFormat, sizeof usFormat, &dwBytesRet, NULL)) {
        // If the directory or file is compressed, let's change it.
        if (usFormat != COMPRESSION_FORMAT_NONE) {
            usFormat = COMPRESSION_FORMAT_NONE;
            CAssert(DeviceIoControl(m_hFile, FSCTL_SET_COMPRESSION, &usFormat, sizeof usFormat, NULL, 0, &dwBytesRet, NULL)).Sys(MSG_SYSTEM_CALL, _T("CTempDir::New() [DeviceIoControl(FSCTL_SET_COMPRESSION)]")).Throw();
        }
    }
}
//
//  Modify the state of the Write Thru flag, to make some optimizations possible.
//  It should still be set to true on open or create etc, to ensure that the file
//  is opened with the proper file flag.
//
void
CFileIO::SetWriteThru(BOOL fForceWriteThru) {
    m_fForceWriteThru = fForceWriteThru;
}
