#ifndef	_CFILE
#define	_CFILE
/*
    @(#) $Id: CFile.h,v 1.8 2002/12/01 12:52:50 svante Exp $

	AxCrypt - Compressing and Encrypting Wrapper and Application Launcher for Secure Local,
	Server or Web Storage of Document Files.

	Copyright (C) 2001 Svante Seleborg/Axon Data, All rights reserved.

	This program is free software; you can redistribute it and/or modify it under the terms
	of the GNU General Public License as published by the Free Software Foundation;
	either version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
	Boston, MA 02111-1307 USA

	The author may be reached at mailto:axcrypt@axondata.se and http://axcrypt.sourceforge.net
----
	CFile.h							Basic open/read/write/memory map operations on files.

	E-mail							YYYY-MM-DD				Reason
	axcrypt@axondata.se 			2001					Initial

*/
#include    "Types.h"
//
//	Used to keep track and restore all file-times.
//
struct SFileTimes {
	FILETIME CreationTime;
	FILETIME LastAccessTime;
	FILETIME LastWriteTime;
};
//
// Support operations for input and output to (file) streams,
// both through memory maps and some normal IO.
//
// This is also a base-class for some special classes of 'dummy'
// files used for cryptographic wiping, hashing etc.
//
// The reason for virtualness is that the transformer class must
// operate properly regardless of which derived class is used
// in the transformation.
//
class CFileIO {
public:
	CFileIO();
	~CFileIO();
	// Open/Create the file with given modes. FALSE if fail.
	virtual void Open(LPTSTR szFileName, BOOL fForceWriteThru, DWORD dwReadWrite, DWORD dwShareMode = 0);
    void OpenDir(LPTSTR szFileName, DWORD dwReadWrite = GENERIC_READ|GENERIC_WRITE, DWORD dwShareMode = FILE_SHARE_READ|FILE_SHARE_WRITE);
	virtual void Create(LPTSTR szFileName, BOOL fForceWriteThru, DWORD dwReadWrite, DWORD dwOpenMode = CREATE_NEW);
	// Build a temporary file name, and open it. FALSE if fail.
	virtual void MakeTmp(LPTSTR szFileName, BOOL fForceWriteThru);
    virtual void CloseView();
    virtual void CloseMap();
	virtual void Close(BOOL fForceKeepOnClose = FALSE);
	virtual void WipeTemp(HWND hProgressWnd);
    virtual void WipeData(HWND hProgressWnd);
	virtual void WipeShort();	// Just wipe the first 512 bytes of the file.
	// Helpers for regular file I/O.
	virtual void SetFilePointer(LONGLONG llPos);
	virtual void SetEndOfFile();
	virtual void SetFileTimes(SFileTimes *pFileTimes);
	virtual SFileTimes *GetFileTimes();
	// Map a view with given mode at a given offset with a given size.
	virtual BYTE *MapView(DWORD dwMode, QWORD qwOffset, unsigned int *puiSize);
	virtual LPTSTR GetFileName();
	LPTSTR FileName();
    void SetNotCompressed();
    void SetWriteThru(BOOL fForceWriteThru);
public:
	HANDLE m_hFile;				// The file...
	QWORD m_qwFileOffset;		// Where to start in the file for the current operation.
	QWORD m_qwFileSize;			// This is the maximum known/expected size of the file. It may be 0 to begin with for output files.
private:
	BOOL CreateEx(LPTSTR szFileName, DWORD dwReadWrite, DWORD dwOpenMode, DWORD dwShareMode, DWORD dwFlags);
	LPTSTR m_szFileName;		// Remember filename for use with error messages, wipings etc.
	HANDLE m_hMapping;			// The mapping, which always maps the entire file
	BYTE *m_poView;				// If non-NULL, the View of maximum MAX_VIEW_SIZE * Allocation Granularity of the system.
    SIZE_T m_cbView;            // Size of view, when m_poView is valid, i.e. non-NULL.
	BOOL m_fDeleteOnClose;		// Set to have the Close() also delete the file.
	BOOL m_fIsTmp;
    BOOL m_fForceWriteThru;     // Set when wiping data to ensure that data is physically written to disk.
	SFileTimes m_FileTimes;
};
//
//	A placeholder CFileIO class. It will never receive or consume any data,
//	it is only there so that the tranformation class may run.
//
class CFileDummy : public CFileIO {
public:
	CFileDummy() {}
	~CFileDummy() {}
	// Dummy functions - do NOTHING!
    virtual void Open(LPTSTR szFileName, DWORD dwReadWrite) {}
	virtual void Create(LPTSTR szFileName, DWORD dwReadWrite) {}
	virtual void MakeTmp(LPTSTR szFileName) {}
    virtual void CloseView() {}
	virtual void CloseMap() {}
	virtual void Close() {}
	virtual void WipeTemp() {}
	virtual void WipeData() {}
	virtual void SetFilePointer(LONGLONG llPos) {}
	virtual void SetEndOfFile() {}
	virtual void SetFileTimes(SFileTimes *pFileTimes) {}
	virtual SFileTimes *GetFileTimes() {return NULL;}
	
	// Map a view with given mode at a given offset with a given size.
	virtual BYTE *MapView(DWORD dwMode, QWORD dwOffset, unsigned int *puiSize) { return NULL; }
	virtual LPTSTR GetFileName() {return NULL;}
};
#endif	_CFILE
