/*
    @(#) $Id: CFileName.cpp,v 1.14 2002/10/30 19:13:27 svante Exp $

	AxCrypt - Compressing and Encrypting Wrapper and Application Launcher for Secure Local,
	Server or Web Storage of Document Files.

	Copyright (C) 2001 Svante Seleborg/Axon Data, All rights reserved.

	This program is free software; you can redistribute it and/or modify it under the terms
	of the GNU General Public License as published by the Free Software Foundation;
	either version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
	Boston, MA 02111-1307 USA

	The author may be reached at mailto:axcrypt@axondata.se and http://axcrypt.sourceforge.net
----
	CFileName.cpp					File name related utility operations.

	E-mail							YYYY-MM-DD				Reason
	axcrypt@axondata.se 			2001					Initial
                                    2002-08-11              Rel 1.2

*/
#include	"StdAfx.h"
#include	"CFileName.h"
#include	"CVersion.h"
#ifndef	_SHELLEXTENSION
#include	"CCryptoRand.h"
#endif
#include	"Utility.h"
#include	"stdio.h"
#include    "CFile.h"

CFileName::CFileName() {
	Split(_T(""));
}

CFileName::CFileName(LPTSTR szFileName) {
	Split(szFileName);
}


CFileName&
CFileName::SetPath2ExeName(HINSTANCE hInstance) {
	CAssert(GetModuleFileName(hInstance, m_szWorkName, sizeof m_szWorkName / sizeof TCHAR) != 0).Sys().Throw();
	Split(m_szWorkName);
	return *this;
}

CFileName&
CFileName::SetPath2TempDir() {
    SetPath2SysTempDir();

    SetDir(CStrPtr(GetDir()) + CStrPtr(_T("\\")) + CStrPtr(CVersion().IntProductName()) + CStrPtr(_T("\\")));
	if (!CreateDirectory(GetDir(), NULL)) {
		CAssert(GetLastError() == ERROR_ALREADY_EXISTS).Sys().Throw();
	}

	return *this;
}
//
//  Set the path to just the system temp directory.
//
CFileName&
CFileName::SetPath2SysTempDir() {
	DWORD dwTempPathLen = GetTempPath(0, NULL);
	CStrPtr szTempPath(dwTempPathLen);
	CAssert(GetTempPath(dwTempPathLen, szTempPath) == (dwTempPathLen - 1)).Sys(MSG_SYSTEM_CALL, _T("GetTempPath() [CFileIO::MakeTemp()]")).Throw();

	SetDir(szTempPath);
	// Not all os's guarantees the existance of the directory...
	if (!CreateDirectory(GetDir(), NULL)) {
		CAssert(GetLastError() == ERROR_ALREADY_EXISTS).Sys().Throw();
	}
    return *this;
}
//
//	Set name only
//
CFileName&
CFileName::SetName(LPCTSTR szFileName) {
	_tsplitpath(szFileName, NULL, NULL, m_szName, NULL);
	return *this;
}
//
//	Set extension only
//
CFileName&
CFileName::SetExt(LPTSTR szExt) {
	_tsplitpath(szExt, NULL, NULL, NULL, m_szExt);
	return *this;
}
//
//	Set title (filename + extension) only, keep the rest.
//
CFileName&
CFileName::SetTitle(LPTSTR szTitle) {
	_tsplitpath(szTitle, NULL, NULL, m_szName, m_szExt);
	return *this;
}
//
//	Set directory (and drive) only, keep the rest.
//
CFileName&
CFileName::SetDir(LPTSTR szDir) {
	_tsplitpath(szDir, m_szDrive, m_szDir, NULL, NULL);
	return *this;
}
//
//  Set the full name, all components.
//
CFileName&
CFileName::Set(LPTSTR szFullName) {
    Split(szFullName);
    return *this;
}
//
//  Apply a current directory to the name. Do this
//  only if no drive or directory is set for the
//  name we already have.
//
CFileName&
CFileName::SetCurDir(LPTSTR szCurDir) {
    // Do nothing if there is a drive or directory specified!
    if (!m_szDrive[0] && !m_szDir[0]) {
        return SetDir(szCurDir);
    }
    return *this;
}
//
//	Override the components that exist in the given path
//
CFileName& 
CFileName::Override(LPTSTR szPath) {
	CFileName Path;
	Path.Set(szPath);

	_tsplitpath(szPath,
				*Path.m_szDrive ? m_szDrive : NULL,
				*Path.m_szDir ? m_szDir : NULL,
				*Path.m_szName ? m_szName : NULL,
				*Path.m_szExt ? m_szExt : NULL);
	return *this;
}

//
//	Delete the extension, keep the rest
//
CFileName&
CFileName::DelExt() {
	m_szExt[0] = TCHAR(0);
	return *this;
}
//
//	Move existing extension to filename, add the new extension
//	keep the rest
//
CFileName&
CFileName::AddExt(LPTSTR szExt) {
	_tcscat(m_szName, m_szExt);
	_tcscpy(m_szExt, szExt);
	Split(Get());
	return *this;
}
//
//	Get a fully qualified file name
//
LPTSTR
CFileName::Get() {
	_tmakepath(m_szWorkName, m_szDrive, m_szDir, m_szName, m_szExt);
	return m_szWorkName;
}
//
//	Get the title (filename + extension)
//
LPTSTR
CFileName::GetTitle() {
	_tmakepath(m_szWorkName, NULL, NULL, m_szName, m_szExt);
	return m_szWorkName;
}
//
//	Get the Drive + Directory
//
LPTSTR
CFileName::GetDir() {
	_tmakepath(m_szWorkName, m_szDrive, m_szDir, NULL, NULL);
	return m_szWorkName;
}
//
//  Get the name
//
LPTSTR
CFileName::GetName() {
	_tmakepath(m_szWorkName, NULL, NULL, m_szName, NULL);
	return m_szWorkName;
}
//
//  Get the extension
//
LPTSTR
CFileName::GetExt() {
	_tmakepath(m_szWorkName, NULL, NULL, NULL, m_szExt);
	return m_szWorkName;
}
//
//	Get the fully qualified name, and quote it.
//
LPTSTR
CFileName::GetQuoted() {
	Get();
	MoveMemory(&m_szWorkName[1], m_szWorkName, (_tcslen(m_szWorkName)+1) * sizeof TCHAR);
	m_szWorkName[0] = _T('"');
	_tcscat(m_szWorkName, _T("\""));
	return m_szWorkName;
}
//
//	for internal use - split into all components.
//
void
CFileName::Split(LPTSTR szFileName) {
	_tsplitpath(szFileName, m_szDrive, m_szDir, m_szName, m_szExt);
}
