/*
    @(#) $Id: CRegistry.cpp,v 1.3 2002/08/23 16:42:03 svante Exp $

	AxCrypt - Compressing and Encrypting Wrapper and Application Launcher for Secure Local,
	Server or Web Storage of Document Files.

	Copyright (C) 2001 Svante Seleborg/Axon Data, All rights reserved.

	This program is free software; you can redistribute it and/or modify it under the terms
	of the GNU General Public License as published by the Free Software Foundation;
	either version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
	Boston, MA 02111-1307 USA

	The author may be reached at mailto:axcrypt@axondata.se and http://axcrypt.sourceforge.net
----
	CRegistry.cpp					Registry manipulating class
	
	E-mail							YYYY-MM-DD				Reason
	axcrypt@axondata.se 			2002-08-17				Rel 1.2.1	Initial
*/
#include	"StdAfx.h"
#include    "CRegistry.h"
#include    "Utility.h"

CRegistry::CRegistry(HKEY hRootKey, LPCTSTR szKey, LPCTSTR szValueName) {
    m_hRootKey = hRootKey;
    m_hRegKey = NULL;
    m_szValueName = NULL;
    if (szKey != NULL) Key(szKey);
    if (szValueName != NULL) m_szValueName = CopySz(szValueName);
}

CRegistry::~CRegistry() {
    if (m_hRegKey != NULL) RegCloseKey(m_hRegKey);
    if (m_szValueName != NULL) delete m_szValueName;
}

CRegistry& CRegistry::Root(HKEY hRootKey) {
    if (m_hRegKey != NULL) {
        RegCloseKey(m_hRegKey);
        m_hRegKey = NULL;
    }
    m_hRootKey = hRootKey;
    return *this;
}

CRegistry& CRegistry::HKey(HKEY hKey) {
    if (m_hRegKey != NULL) {
        RegCloseKey(m_hRegKey);
        m_hRegKey = NULL;
    }
    m_hRegKey = hKey;
    return *this;
}

CRegistry& CRegistry::Key(LPCTSTR szKey) {
    if (m_hRegKey != NULL) {
        RegCloseKey(m_hRegKey);
        m_hRegKey = NULL;
    }
    
    LONG lRes = RegOpenKeyEx(m_hRootKey, 
                             szKey,
                             0,
                             KEY_READ | KEY_WRITE,
                             &m_hRegKey);
    // We allow silent failure if the key is no found. This is ok if we only want to
    // read it...
    if (lRes != ERROR_FILE_NOT_FOUND) {
        CAssertEq(lRes, ERROR_SUCCESS).Sys(MSG_SYSTEM_CALL, _T("CRegistry::Key() [RegOpenKeyEx]")).Throw();
    }
    return *this;
}
//
//  Create and open a handle to a registry key, using FormatMessage pattern
//  to fill in.
//
//  Use %1 to fill in the pattern.
//
CRegistry& CRegistry::CreateKey(LPCTSTR szKeyFormat, LPCTSTR szFillIn) {
    LPTSTR szFormattedKey;
    CAssert(FormatMessage(FORMAT_MESSAGE_FROM_STRING | FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_ARGUMENT_ARRAY,
                          szKeyFormat,
                          0,
                          0,
                          (LPTSTR)&szFormattedKey,
                          0,
                          (va_list *)&szFillIn)).Sys(MSG_SYSTEM_CALL, _T("CRegistry::CreateKey() [FormatMessage()]")).Throw();
    if (m_hRegKey != NULL) {
        RegCloseKey(m_hRegKey);
        m_hRegKey = NULL;
    }
    ULONG lDisp;
    LONG lRes = RegCreateKeyEx(m_hRootKey,
                               szFormattedKey,
                               0,
                               NULL,
                               0,
                               KEY_ALL_ACCESS,
                               NULL,
                               &m_hRegKey,
                               &lDisp);
    LocalFree(szFormattedKey);
    CAssertEq(lRes, ERROR_SUCCESS).Sys(MSG_SYSTEM_CALL, _T("CRegistry:CreateKey() [RegCreateKeyEx()]")).Throw();
    return *this;
}

CRegistry& CRegistry::Value(LPCTSTR szValueName) {
    if (m_szValueName != NULL) {
        delete m_szValueName;
        m_szValueName = NULL;
    }
    m_szValueName = CopySz(szValueName);
    return *this;
}

DWORD CRegistry::GetDword(DWORD dwDefault) {
    // We interpret a NULL hKey as 'not found', and thus return
    // the default value.
    if (m_hRegKey == NULL) {
        return dwDefault;
    }
    DWORD dwData, dwLen = sizeof dwData, dwType;
    LONG lRes = RegQueryValueEx(m_hRegKey, m_szValueName, NULL, &dwType, (BYTE *)&dwData, &dwLen);
    if (lRes == ERROR_FILE_NOT_FOUND) {
        dwData = dwDefault;
    } else {
        CAssertEq(lRes, ERROR_SUCCESS).Sys(MSG_SYSTEM_CALL, _T("CRegistry::GetDword() [ReqQueryValueEx]")).Throw();
        CAssert(dwLen == 4).App(ERR_UNSPECIFIED, _T("CRegistry::GetDword() [dwLen != 4]")).Throw();
    }
    return dwData;
}

void CRegistry::SetDword(DWORD dwValue) {
    CAssertEq(RegSetValueEx(m_hRegKey,
                            m_szValueName,
                            0,
                            REG_DWORD,
                            (BYTE *)&dwValue,
                            sizeof dwValue), ERROR_SUCCESS).Sys(MSG_SYSTEM_CALL, _T("CRegistry::SetDword() [RegSetValueEx()]")).Throw();
}

void *CRegistry::GetBinary(int *nLen) {
    return NULL;
}

void CRegistry::SetBinary(void *vData, int nLen) {
}
//
//  This returns a new'ed string, please remember to delete...
//
LPTSTR CRegistry::GetSz(LPCTSTR szDefault) {
    // If the registry does not exist - return default!
    if (m_hRegKey == NULL) {
        return CopySz(szDefault);
    }

    DWORD dwLen, dwType;
    // First get the length of the string
    LONG lRes = RegQueryValueEx(m_hRegKey, m_szValueName, NULL, &dwType, NULL, &dwLen);
    if (lRes == ERROR_FILE_NOT_FOUND) {
        return CopySz(szDefault);
    }
    CAssertEq(lRes, ERROR_SUCCESS).Sys(MSG_SYSTEM_CALL, _T("CRegistry::GetSz() [RegQueryValueEx()]")).Throw();
    // Must be reg_sz, otherwise default return.
    if (dwType != REG_SZ) {
        return CopySz(szDefault);
    }
    
    LPTSTR szValue = new char[dwLen]; // Yes, char, ReqQuery returns the size in bytes...

    lRes = RegQueryValueEx(m_hRegKey, m_szValueName, NULL, &dwType, (BYTE *)szValue, &dwLen);
    if (lRes != ERROR_SUCCESS) {
        delete szValue;
        CAssert(FALSE, lRes).Sys(MSG_SYSTEM_CALL, _T("CRegistry::GetSz() [RegQueryValueEx()]")).Throw();
    }
    return szValue;
}

void CRegistry::SetSz(LPCTSTR szValue) {
    CAssertEq(RegSetValueEx(m_hRegKey,
                            m_szValueName,
                            0,
                            REG_SZ,
                            (CONST BYTE *)szValue,
                            _tcslen(szValue)+1), ERROR_SUCCESS).Sys(MSG_SYSTEM_CALL, _T("CRegistry::SetSz() [RegSetValueEx()]")).Throw();
}

void CRegistry::SetSz(LPCTSTR szValueFormat, LPCTSTR szFillIn) {
    LPTSTR szFormattedValue;
    CAssert(FormatMessage(FORMAT_MESSAGE_FROM_STRING | FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_ARGUMENT_ARRAY,
                          szValueFormat,
                          0,
                          0,
                          (LPTSTR)&szFormattedValue,
                          0,
                          (va_list *)&szFillIn)).Sys(MSG_SYSTEM_CALL, _T("CRegistry::SetSz() [FormatMessage()]")).Throw();
    LONG lRes = RegSetValueEx(m_hRegKey,
                              m_szValueName,
                              0,
                              REG_SZ,
                              (CONST BYTE *)szFormattedValue,
                              _tcslen(szFormattedValue)+1);
    // First free the buffer, then assert...
    LocalFree(szFormattedValue);
    CAssertEq(lRes, ERROR_SUCCESS).Sys(MSG_SYSTEM_CALL, _T("CRegistry::SetSz() [RegSetValueEx()]")).Throw();
}

void
CRegistry::DelValue() {
    LONG lRes = RegDeleteValue(m_hRegKey, m_szValueName);
    CAssert(lRes == ERROR_FILE_NOT_FOUND || lRes == ERROR_SUCCESS, lRes).Sys(MSG_SYSTEM_CALL, _T("CRegistry::DelValue() [RegDeleteValue()]")).Throw();
}

void
CRegistry::DelSubKey(LPCTSTR szSubKey) {
    DelSubHKey(m_hRegKey, szSubKey);
}

void
CRegistry::DelSubHKey(HKEY hKey, LPCTSTR szSubKey) {
    LONG lRes = RegDeleteKey(hKey, szSubKey);
    CAssert(lRes == ERROR_FILE_NOT_FOUND || lRes == ERROR_SUCCESS, lRes).Sys(MSG_SYSTEM_CALL, _T("CRegistry::DelValue() [RegDeleteKey()]")).Throw();
}
