/*
    @(#) $Id: Dialog.cpp,v 1.10 2002/08/25 16:51:21 svante Exp $

	AxCrypt - Compressing and Encrypting Wrapper and Application Launcher for Secure Local,
	Server or Web Storage of Document Files.

	Copyright (C) 2001 Svante Seleborg/Axon Data, All rights reserved.

	This program is free software; you can redistribute it and/or modify it under the terms
	of the GNU General Public License as published by the Free Software Foundation;
	either version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
	Boston, MA 02111-1307 USA

	The author may be reached at mailto:axcrypt@axondata.se and http://axcrypt.sourceforge.net
----
	Dialog.cpp						Secure dialog procedures, handling password entry etc.

	E-mail							YYYY-MM-DD				Reason
	axcrypt@axondata.se 			2001					Initial
                                    2002-08-05              Rel 1.2

*/
//
//	Handle all user interaction
//
//	As far as is possible, we try to ensure that such things as passwords are
//	never stored in memory that is not locked in memory, to ensure that they
//	are not written to disk as virtual memory.
//
#include	"StdAfx.h"
#include	"CVersion.h"
#include    "CRegistry.h"

//
struct SSafeEdit {
	LPTSTR szPassphrase;
	unsigned uiLen;
	WNDPROC lpfnOldWndProc;
	BOOL fIsWindowUnicode;				// TRUE if chars received are in Unicode.
};
//
//	Arguments to dialog box.
//
struct SDlgInfo {
	LPTSTR szPassphrase;
    LPTSTR szFileName;
	int IDDMainPrompt;
    BOOL fSaveInCache;      // Set check-box if the user wants to save the key
};

//
//	The basic idea behind SafeEdit is to solve the problem of uncontrolled edit control
//	memory by simply not putting anything of interest there! We intercept the WM_CHAR
//	message and store it in our locally controlled buffer instead. That buffer is backed
//	by a memory mapped file, which thus never winds up in the swap file.
//
//	Still vulnerable to windows hooks and sniffers though...
//
LRESULT CALLBACK SafeEdit(
	HWND hwnd,      // handle to window
	UINT uMsg,      // message identifier
	WPARAM wParam,  // first message parameter
	LPARAM lParam   // second message parameter
	) {
	SSafeEdit* pSafeEdit = (SSafeEdit*)GetWindowLong(hwnd, GWL_USERDATA);
	WNDPROC lpfnOldWndProc = pSafeEdit->lpfnOldWndProc;
	switch (uMsg) {
	case WM_DESTROY:
		(void)SetWindowLong(hwnd, GWL_WNDPROC, (LONG)lpfnOldWndProc);
		delete pSafeEdit;
		break;
	case WM_CLEAR:
	case WM_COPY:
	case WM_CUT:
	case WM_PASTE:
	case WM_UNDO:
	case WM_LBUTTONDBLCLK:
	case WM_RBUTTONDOWN:
		return FALSE;
	case WM_LBUTTONDOWN:
		SetFocus(hwnd);
		return FALSE;
	case WM_KEYDOWN:
		switch ((int)wParam) {
		case VK_PRIOR:
		case VK_NEXT:
		case VK_HOME:
		case VK_LEFT:
		case VK_RIGHT:
		case VK_UP:
		case VK_DOWN:
		case VK_INSERT:
		case VK_DELETE:
			return FALSE;
		}
		break;
	case WM_CHAR:
		switch ((int)wParam) {
		// We only handle back-space of the non-printables.
		case 0x08:
			if (_tcsclen(pSafeEdit->szPassphrase) > 0) {
				pSafeEdit->szPassphrase[_tcsclen(pSafeEdit->szPassphrase)-1] = (_TCHAR)0;
			} else {
				(void)MessageBeep(MB_OK);
				return FALSE;
			}
			break;
		// Ignore other non-printables
		case 0x0a:
		case 0x0d:
		case 0x1b:
		case 0x09:
			break;
		// Now we have a printable character.
		default:
			unsigned char cChar[2];	// Yes, really a character + nul. Nothing else but...
			// If the char is Unicode, we first translate it to Ansi.
			if (pSafeEdit->fIsWindowUnicode) {
				BOOL fUsedDefault;
				if (!WideCharToMultiByte(CP_ACP,
									     WC_COMPOSITECHECK|WC_DEFAULTCHAR,
										 (LPCWSTR)&wParam,
										 1,
										 (LPTSTR)&cChar,
										 2,
										 NULL,
										 &fUsedDefault) || fUsedDefault) {
					(void)MessageBeep(MB_OK);
					return FALSE;
				}
			} else {
				cChar[0] = wParam;
			}
			if (strchr((const char *)szPassphraseChars, cChar[0]) == NULL || _tcsclen(pSafeEdit->szPassphrase) == pSafeEdit->uiLen) {
				(void)MessageBeep(MB_OK);
				return FALSE;
			}
			_tcsncat(pSafeEdit->szPassphrase, (const char *)&cChar, 1);
			wParam = *_T("*");
		}
		// Fall thru
	default:
		;
	}
	return CallWindowProc(lpfnOldWndProc, hwnd, uMsg, wParam, lParam);
}

//
//	Dialog procedure for the new passphrase dialog
//
BOOL CALLBACK NewPassphraseDlgProc(
	HWND hwndDlg,  // handle to dialog box
	UINT uMsg,     // message
	WPARAM wParam, // first message parameter
	LPARAM lParam  // second message parameter
) {
    SDlgInfo *pDlgInfo;
	SSafeEdit* pSafeEdit;
	switch (uMsg) {
	case WM_INITDIALOG:
        pDlgInfo = (SDlgInfo *)lParam;
        (void)SetWindowLong(hwndDlg, GWL_USERDATA, lParam);

        pSafeEdit = new SSafeEdit;
		pSafeEdit->uiLen = MAX_PASSPHRASE_LEN;
		(pSafeEdit->szPassphrase = new TCHAR[pSafeEdit->uiLen+1])[0] = (TCHAR)0;
		pSafeEdit->lpfnOldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndDlg, IDC_NEWPASSPHRASE1), GWL_WNDPROC, (LONG)SafeEdit);
		(void)SetWindowLong(GetDlgItem(hwndDlg, IDC_NEWPASSPHRASE1), GWL_USERDATA, (LONG)pSafeEdit);
		
		pSafeEdit = new SSafeEdit;
		pSafeEdit->uiLen = MAX_PASSPHRASE_LEN;
		(pSafeEdit->szPassphrase = new TCHAR[pSafeEdit->uiLen+1])[0] = (TCHAR)0;
		pSafeEdit->lpfnOldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndDlg, IDC_NEWPASSPHRASE2), GWL_WNDPROC, (LONG)SafeEdit);
		pSafeEdit->fIsWindowUnicode = IsWindowUnicode(hwndDlg);
		(void)SetWindowLong(GetDlgItem(hwndDlg, IDC_NEWPASSPHRASE2), GWL_USERDATA, (LONG)pSafeEdit);
		
		SetDlgItemText(hwndDlg, IDC_ENTER_PASS, CMessage().AppMsg(INF_ENTER_PASS).GetMsg());
		SetDlgItemText(hwndDlg, IDC_ENTER_VERIFY, CMessage().AppMsg(INF_ENTER_VERIFY).GetMsg());
        SetDlgItemText(hwndDlg, IDOK, CMessage().AppMsg(INF_IDOK).GetMsg());
		SetDlgItemText(hwndDlg, IDCANCEL, CMessage().AppMsg(INF_IDCANCEL).GetMsg());

        SetDlgItemText(hwndDlg, IDC_CHECKCACHE, CMessage().AppMsg(INF_SAVE_ENCKEY).GetMsg());
        CheckDlgButton(hwndDlg, IDC_CHECKCACHE, pDlgInfo->fSaveInCache ? BST_CHECKED : BST_UNCHECKED);
		
		SetWindowText(hwndDlg, CVersion().String());

        // Ensure that we are not obscured by parent, if possible.
        SetWindowPos(GetParent(hwndDlg), hwndDlg, 0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE);
		SetFocus(GetDlgItem(hwndDlg, IDC_NEWPASSPHRASE1));
        return FALSE;

	case WM_COMMAND:
	{
        pDlgInfo = (SDlgInfo *)GetWindowLong(hwndDlg, GWL_USERDATA);
		LPTSTR szPhrase1 = ((SSafeEdit*)GetWindowLong(GetDlgItem(hwndDlg, IDC_NEWPASSPHRASE1), GWL_USERDATA))->szPassphrase;
		LPTSTR szPhrase2 = ((SSafeEdit*)GetWindowLong(GetDlgItem(hwndDlg, IDC_NEWPASSPHRASE2), GWL_USERDATA))->szPassphrase;
		switch (wParam) {
        case IDC_CHECKCACHE:
            pDlgInfo->fSaveInCache = IsDlgButtonChecked(hwndDlg, IDC_CHECKCACHE);
            break;
		case IDOK:
			if (_tcscmp(szPhrase1, szPhrase2) != 0) {
				CMessage(hwndDlg).AppMsg(MSG_NOTSAME).ShowWarning();
				SendDlgItemMessage(hwndDlg, IDC_NEWPASSPHRASE1, WM_SETTEXT, 0, (LPARAM)_T(""));
				SendDlgItemMessage(hwndDlg, IDC_NEWPASSPHRASE2, WM_SETTEXT, 0, (LPARAM)_T(""));
				*szPhrase1 = *szPhrase2 = (_TCHAR)0;
				SetFocus(GetDlgItem(hwndDlg, IDC_NEWPASSPHRASE1));
				return FALSE;
			}
			pDlgInfo->szPassphrase = szPhrase1;
			delete szPhrase2;
			EndDialog(hwndDlg, TRUE);
			break;
		case IDCANCEL:
			delete szPhrase1;
			delete szPhrase2;
			EndDialog(hwndDlg, FALSE);
			break;
		}
		return TRUE;
	}
	default:
		return FALSE;
	}
}
//
//	Dialog procedure for simple passphrase dialog
//
BOOL CALLBACK PassphraseDlgProc(
	HWND hwndDlg,  // handle to dialog box
	UINT uMsg,     // message
	WPARAM wParam, // first message parameter
	LPARAM lParam  // second message parameter
) {
    SDlgInfo *pDlgInfo;
	SSafeEdit* pSafeEdit;
	switch (uMsg) {
	case WM_INITDIALOG:
        pDlgInfo = (SDlgInfo *)lParam;
        (void)SetWindowLong(hwndDlg, GWL_USERDATA, lParam);

		pSafeEdit = new SSafeEdit;
		pSafeEdit->uiLen = MAX_PASSPHRASE_LEN;
		(pSafeEdit->szPassphrase = new TCHAR[pSafeEdit->uiLen+1])[0] = (TCHAR)0;
		pSafeEdit->lpfnOldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndDlg, IDC_PASSPHRASE), GWL_WNDPROC, (LONG)SafeEdit);
		pSafeEdit->fIsWindowUnicode = IsWindowUnicode(hwndDlg);
		(void)SetWindowLong(GetDlgItem(hwndDlg, IDC_PASSPHRASE), GWL_USERDATA, (LONG)pSafeEdit);

		SetDlgItemText(hwndDlg, IDC_ENTER_PASS , CMessage().AppMsg(((SDlgInfo *)lParam)->IDDMainPrompt).GetMsg());
		SetDlgItemText(hwndDlg, IDOK, CMessage().AppMsg(INF_IDOK).GetMsg());
		SetDlgItemText(hwndDlg, IDCANCEL, CMessage().AppMsg(INF_IDCANCEL).GetMsg());

        SetDlgItemText(hwndDlg, IDC_CHECKCACHE, CMessage().AppMsg(INF_SAVE_DECKEY).GetMsg());
        CheckDlgButton(hwndDlg, IDC_CHECKCACHE, pDlgInfo->fSaveInCache ? BST_CHECKED : BST_UNCHECKED);

        SetWindowLong(GetDlgItem(hwndDlg, IDS_FILENAME), GWL_STYLE, GetWindowLong(GetDlgItem(hwndDlg, IDS_FILENAME), GWL_STYLE) | SS_PATHELLIPSIS);
        if (pDlgInfo->szFileName && pDlgInfo->szFileName[0]) {
            SetDlgItemText(hwndDlg, IDS_FILENAME, pDlgInfo->szFileName);
        } else {
            ShowWindow(GetDlgItem(hwndDlg, IDS_FILENAME), SW_HIDE);
        }
		
		SetWindowText(hwndDlg, CVersion().String());

        // Ensure that we are not obscured by parent, if possible.
        SetWindowPos(GetParent(hwndDlg), hwndDlg, 0, 0, 0, 0, SWP_NOSIZE|SWP_NOMOVE);
		SetFocus(GetDlgItem(hwndDlg, IDC_PASSPHRASE));
        return FALSE;

	case WM_COMMAND:
	{
        pDlgInfo = (SDlgInfo *)GetWindowLong(hwndDlg, GWL_USERDATA);
		LPTSTR szPhrase = ((SSafeEdit*)GetWindowLong(GetDlgItem(hwndDlg, IDC_PASSPHRASE), GWL_USERDATA))->szPassphrase;
		switch (wParam) {
        case IDC_CHECKCACHE:
            pDlgInfo->fSaveInCache = IsDlgButtonChecked(hwndDlg, IDC_CHECKCACHE);
            break;
		case IDOK:
            pDlgInfo->szPassphrase = szPhrase;
			EndDialog(hwndDlg, TRUE);
			break;
		case IDCANCEL:
			delete szPhrase;
			EndDialog(hwndDlg, FALSE);
			break;
		}
		return TRUE;
	}
	default:
		return FALSE;
	}
}
//
//	Return empty string/NULL if user cancelled
//	Return the passphrase otherwise.
//
CStrPtr
GetNewPassphrase() {
    if (!CRegistry(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValServerMode).GetDword(FALSE)) {
	    SDlgInfo dlgInfo;
        CRegistry utRegSaveFlag(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValSaveEncKey);

        dlgInfo.szPassphrase = NULL;
        dlgInfo.fSaveInCache = utRegSaveFlag.GetDword();

        switch (DialogBoxParam(
		    ghInstance,
		    MAKEINTRESOURCE(IDD_NEWPASSPHRASE),
		    GetForegroundWindow(),
		    NewPassphraseDlgProc,
		    (LPARAM)&dlgInfo)) {
	    case TRUE: {
            utRegSaveFlag.SetDword(dlgInfo.fSaveInCache);
            if (dlgInfo.szPassphrase != NULL) {
		        CStrPtr szReturn(dlgInfo.szPassphrase);
		        delete dlgInfo.szPassphrase;
		        return szReturn;
            } else return NULL;
	    }
	    case FALSE:
		    return NULL;
	    default:
		    CMessage().AppMsg(MSG_INTERNAL_ERROR, _T("GetNewPassphrase()")).ShowError();
		    return NULL;
	    }
    } else {
        CMessage().Wrap(0).AppMsg(ERR_KEYPROMPT_SERVER_MODE).LogEvent(0);
        return NULL;
    }
}
//
//	Get a passphrase for an existing password. No verification here.
//
//	Return empty string/NULL if user cancels
//	Return passphrase otherwise
//
CStrPtr
GetPassphrase(int iPromptID, LPTSTR szFileName) {
    if (!CRegistry(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValServerMode).GetDword(FALSE)) {
	    SDlgInfo dlgInfo;
        CRegistry utRegSaveFlag(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValSaveDecKey);

	    dlgInfo.IDDMainPrompt = iPromptID;
        dlgInfo.szPassphrase = NULL;
        dlgInfo.fSaveInCache = utRegSaveFlag.GetDword();
        dlgInfo.szFileName = szFileName;

	    switch (DialogBoxParam(ghInstance,
		    MAKEINTRESOURCE(IDD_PASSPHRASE),
		    GetForegroundWindow(),
		    PassphraseDlgProc,
		    (LPARAM)&dlgInfo)) {
	    case TRUE: {
            utRegSaveFlag.SetDword(dlgInfo.fSaveInCache);
            if (dlgInfo.szPassphrase != NULL) {
		        CStrPtr szReturn(dlgInfo.szPassphrase);
		        delete dlgInfo.szPassphrase;
    		    return szReturn;
            } else return NULL;
	    }
	    case FALSE:
		    return NULL;
	    default:
		    CMessage().AppMsg(MSG_INTERNAL_ERROR, _T("GetPassphrase()")).ShowError();
		    return NULL;
	    }
    } else {
        CMessage().Wrap(0).AppMsg(ERR_KEYPROMPT_SERVER_MODE).LogEvent(0);
        return NULL;
    }
}
//
//  Parameter package for the warning dialogue.
//
class CWarnDlg {
public:
    LPTSTR szMsg;
    LPTSTR szNotAgainMsg;
    BOOL fNotAgain;

    CWarnDlg() { szMsg = szNotAgainMsg = NULL; }
    ~CWarnDlg() {
        if (szMsg != NULL) delete szMsg;
        if (szNotAgainMsg != NULL) delete szMsg;
    }
};

//
//
//
BOOL CALLBACK
WarningDlgProc(
	HWND hwndDlg,  // handle to dialog box
	UINT uMsg,     // message
	WPARAM wParam, // first message parameter
	LPARAM lParam  // second message parameter
) {
    CWarnDlg *pDlgWarn;
	switch (uMsg) {
	case WM_INITDIALOG:
        pDlgWarn = (CWarnDlg *)lParam;
        (void)SetWindowLong(hwndDlg, GWL_USERDATA, lParam);

        SetDlgItemText(hwndDlg, IDC_MSG, pDlgWarn->szMsg);
        SetDlgItemText(hwndDlg, IDC_CHECKMSG, pDlgWarn->szNotAgainMsg);
        CheckDlgButton(hwndDlg, IDC_CHECKMSG, BST_UNCHECKED);
				
		SetWindowText(hwndDlg, CVersion().String());
		SetFocus(GetDlgItem(hwndDlg, IDOK));
		return FALSE;

	case WM_COMMAND:
	{
        pDlgWarn = (CWarnDlg *)GetWindowLong(hwndDlg, GWL_USERDATA);
		switch (wParam) {
        case IDC_CHECKMSG:
            pDlgWarn->fNotAgain = IsDlgButtonChecked(hwndDlg, IDC_CHECKMSG);
            break;
		case IDOK:
        case IDCANCEL:
			EndDialog(hwndDlg, TRUE);
			break;
		}
		return TRUE;
	}
	default:
		return FALSE;
	}
}

//
//  Show a warning dialogue with message, an 'ok' button and a
//  'don't show this warning again' checkbox. Return the state
//  of the checkbox.
//
BOOL
WarningDlg(LPTSTR szFileName, DWORD dwMsg, DWORD dwNotAgainMsg) {
    if (!CRegistry(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValServerMode).GetDword(FALSE)) {
	    CWarnDlg dlgWarn;

	    dlgWarn.szMsg = CopySz(CMessage().Wrap(0).AppMsg(dwMsg, NULL, szFileName).GetMsg());
        dlgWarn.szNotAgainMsg = CopySz(CMessage().AppMsg(dwNotAgainMsg).GetMsg());
        dlgWarn.fNotAgain = FALSE;

	    switch (DialogBoxParam(ghInstance,
                               MAKEINTRESOURCE(IDD_WARNING),
		                       GetForegroundWindow(),
		                       WarningDlgProc,
		                       (LPARAM)&dlgWarn)) {
	    case TRUE:
        case FALSE: {
            return dlgWarn.fNotAgain;
        }
	    default:
		    CAssert(FALSE).App(MSG_INTERNAL_ERROR, _T("WarningDlg()")).Throw();
	    }
        return FALSE;
    } else {
        CMessage().Wrap(0).AppMsg(dwMsg, NULL, szFileName).LogEvent(0);
        return FALSE;
    }
}
