/*
    @(#) $Id: FileCmd.cpp,v 1.22 2002/12/12 14:14:48 svante Exp $

	AxCrypt - Compressing and Encrypting Wrapper and Application Launcher for Secure Local,
	Server or Web Storage of Document Files.

	Copyright (C) 2001 Svante Seleborg/Axon Data, All rights reserved.

	This program is free software; you can redistribute it and/or modify it under the terms
	of the GNU General Public License as published by the Free Software Foundation;
	either version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
	Boston, MA 02111-1307 USA

	The author may be reached at mailto:axcrypt@axondata.se and http://axcrypt.sourceforge.net
----
	FileCmd.cpp						Implementation of file operation commands from main code.

	E-mail							YYYY-MM-DD				Reason
	axcrypt@axondata.se 			2001-12-02				Initial for 0.0b4
                                    2002-02-12              Use IsValid() for CHandle vars.
                                    2002-07-25              Rev 1.2
                                    2002-08-22              Rel 1.2.2
*/
#include	"StdAfx.h"
//
#include	"shellapi.h"
#include	"shlobj.h"
#include	"CWrapper.h"
#include	"CFileTemp.h"
#include	"CChildProc.h"
#include	"CWinTrix.h"
#include	"Dialog.h"
#include	"CVersion.h"
#include    "FileCmd.h"
#include	"Utility.h"
#include    "CRegistry.h"
#include	<io.h>
#include	<fcntl.h>
//
//  Encrypt and compress a file, but leave the original.
//
//  Return 0 on success, error code otherwise
//
//  "Primary Execute Request Thread"
//
DWORD
CmdEncryptZCFile(struct SCmdParam *pCmdParam, BOOL fWipeTemps) {
	if (pCmdParam->szParam1 == NULL) {
		return 0;
	}

	DWORD dwReturn = 0;
	try {
        CFileName fnPlain(pCmdParam->szParam1);
        fnPlain.SetCurDir(pCmdParam->szCurDir);

        // If we should igore already encrypted content...
        if (pCmdParam->fIgnoreEncrypted) {
            CFileIO fioPlain;
            BOOL fIsEncrypted = TRUE;
            fioPlain.Open(fnPlain.Get(), FALSE, GENERIC_READ, FILE_SHARE_READ);
            try {
                CHeaders Headers;
                Headers.VerifyStructure(fioPlain);
            } catch (TAssert utErr) {
                fIsEncrypted = FALSE;
            }
            if (fIsEncrypted) {
                return WRN_IGNORED;   // Is encrypted, do nothing.
            }
        }

		CKeyPrompt utKeyPrompt; // we need this for the duration
		CCryptoKey *pCryptoKey = pgKeyList->FindEncKey(pCmdParam->dwBatch);
        TKey *pKey = NULL;
        if (pCryptoKey != NULL) {
            pKey = pCryptoKey->Key();
        }
        // If no default encryption key
        if (pKey == NULL) {
		    utKeyPrompt.New();
		    // If the user entered a key, and did not cancel
		    if (utKeyPrompt.Get() != NULL) {
                // Test if we really should save the key here.
                BOOL fSaveEncKey = CRegistry(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValSaveEncKey).GetDword();
                if (fSaveEncKey || pCmdParam->dwBatch != 0) {
    				// This is may be a permanent allocation here.
				    HEAP_CHECK_BEGIN(_T("CmdEncryptZCFile(a)"), TRUE);
                    pKey = pgKeyList->AddEncKey(utKeyPrompt.Get(), fSaveEncKey ? 0 : pCmdParam->dwBatch)->Key();
				    HEAP_CHECK_END
                } else {
                    pKey = utKeyPrompt.Get();
                }
		    } else {
                dwReturn = WRN_CANCEL;
            }
        }
		HEAP_CHECK_BEGIN(_T("CmdEncryptZCFile(b)"), 0)
		if (pKey != NULL) {

            // Either use the given output name, or derive one.
            CFileName fnCipher;
            DWORD dwOpenMode = CREATE_NEW;
            fnCipher.Set(pCmdParam->szParam1);
    		fnCipher.AddExt((LPTSTR)gszAxCryptFileExt);
            if (pCmdParam->szParam2 && pCmdParam->szParam2[0]) {
                fnCipher.Override(pCmdParam->szParam2);
                dwOpenMode = CREATE_ALWAYS;
            }

			CFileIO utFilePlain, utFileCipher;
			utFilePlain.Open(fnPlain.Get(),
                             FALSE,
                             GENERIC_READ,
                             FILE_SHARE_READ);

            fnCipher.SetCurDir(pCmdParam->szCurDir);
			utFileCipher.Create(fnCipher.Get(), FALSE, GENERIC_READ|GENERIC_WRITE, dwOpenMode);

			HEAP_CHECK_BEGIN(_T("CmdEncryptZCFile(c)"), 0)
			CHeaders utHeaders;
			utHeaders.SetDataEncKey(pKey);
			utHeaders.SetFileName(CFileName(pCmdParam->szParam1).GetTitle());
			utHeaders.SetFileTimes(utFilePlain.GetFileTimes());

			// Store a given id-tag into the headers, in the clear.
			if (pCmdParam->szIdTag && *pCmdParam->szIdTag) {
				utHeaders.SetIdTag(pCmdParam->szIdTag);
			}

			HEAP_CHECK_BEGIN(_T("CmdEncryptZCFile(d)"), 0)
			CWrapper utWrap(&utHeaders, pCmdParam->hProgressWnd);
			utWrap.Wrap(utFilePlain, utFileCipher, fWipeTemps);
			HEAP_CHECK_END
			HEAP_CHECK_END

			utFileCipher.Close(TRUE);			// Close and force keep
			utFilePlain.Close();
            SHChangeNotify(SHCNE_CREATE, SHCNF_PATH, fnCipher.Get(), NULL);
        } else {
            dwReturn = WRN_CANCEL;
        }
		HEAP_CHECK_END
	} catch (TAssert utErr) {
        // Hide progress window, if any.
        if (pCmdParam->hProgressWnd) {
            SendMessage(GetParent(pCmdParam->hProgressWnd), WM_APP, 0, 0);
        }

        if (utErr.LastError() == WRN_CANCEL) {
            utErr.App(WRN_CANCEL).Show();
        } else {
    		utErr.File(MSG_WRAP_ERROR, pCmdParam->szParam1).Show();
        }
		return utErr.LastError();
	}
	return dwReturn;
}
//
// Encrypt the file to name concatenated with the extension,
// Wipe and delete the orignal.
//
//	Return 0 on success, error code otherwise.
//
// "Primary Execute Request Thread"
//
DWORD
CmdEncryptZFile(struct SCmdParam *pCmdParam) {
	if (pCmdParam->szParam1 == NULL) {
		return 0;
	}

	DWORD dwReturn = CmdEncryptZCFile(pCmdParam);
    if (dwReturn || dwReturn == WRN_IGNORED) return dwReturn;
	try {
        dwReturn = CmdWipeSilent(pCmdParam);
	} catch (TAssert utErr) {
        // Hide progress window, if any.
        if (pCmdParam->hProgressWnd) {
            SendMessage(GetParent(pCmdParam->hProgressWnd), WM_APP, 0, 0);
        }
        if (utErr.LastError() == WRN_CANCEL) {
            utErr.App(WRN_CANCEL).Show();
        } else {
			utErr.File(MSG_WRAP_ERROR, pCmdParam->szParam1).Show();
        }
		return utErr.LastError();
	}
	return dwReturn;
}
//
//	Helper to first look in key cache, and then prompt for key if that
//	did not work.
//
//	Return TRUE on opened headers, FALSE on user cancel.
//	Throws CAssert exception error.
//
//  The Key-encrypting-key is returned in *ppKeyEncKey, and must be deleted
//  by caller.
//
static BOOL
TryCacheAndPromptOpen(CHeaders *pHeaders, TKey **ppKeyEncKey, DWORD dwBatch, LPTSTR szFileName) {

    if (pgKeyList->TryOpen(pHeaders, ppKeyEncKey, dwBatch)) {
	    return TRUE;
    }

	CKeyPrompt utPromptKey;
	utPromptKey.Old(INF_ENTER_PASS, szFileName);
	while (utPromptKey.Get() != NULL) {
		if (pHeaders->Open(utPromptKey.Get())) {
            BOOL fSaveDecKey = CRegistry(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValSaveDecKey).GetDword(FALSE);
            if (fSaveDecKey || dwBatch != 0) {
                pgKeyList->AddKey(utPromptKey.Get(), FALSE, fSaveDecKey ? 0 : dwBatch);
            }
            *ppKeyEncKey = new TKey(*utPromptKey.Get());
			return TRUE;
		}
		MessageBeep(MB_OK);
		utPromptKey.Old(INF_REENTER_PASS, szFileName);
	}
	return FALSE;
}
//
//  Decrypt a file, and wipe the original
//
DWORD
CmdDecryptFile(struct SCmdParam *pCmdParam) {
	if (pCmdParam->szParam1 == NULL) {
		return 0;
	}

	DWORD dwReturn = CmdDecryptCFile(pCmdParam, TRUE);
    if (dwReturn) return dwReturn;
	try {
        CFileName utCipherName(pCmdParam->szParam1);
        utCipherName.SetCurDir(pCmdParam->szCurDir);

		CFileIO utFileCipher;
		utFileCipher.Open(utCipherName.Get(), TRUE,
                          GENERIC_READ|GENERIC_WRITE);
		utFileCipher.WipeShort();			// Just wipe the first 512 bytes of the cipher text file.
		utFileCipher.Close();				// Close (and delete)
        SHChangeNotify(SHCNE_DELETE, SHCNF_PATH, utCipherName.Get(), NULL);
	} catch (TAssert utErr) {
        // Hide progress window, if any.
        if (pCmdParam->hProgressWnd) {
            SendMessage(GetParent(pCmdParam->hProgressWnd), WM_APP, 0, 0);
        }
        if (utErr.LastError() == WRN_CANCEL) {
            utErr.App(WRN_CANCEL).Show();
        } else {
			utErr.File(MSG_WRAP_ERROR, pCmdParam->szParam1).Show();
        }
		return utErr.LastError();
	}
	return dwReturn;
}
//
// Decrypt a file. Retain the original.
//
DWORD
CmdDecryptCFile(struct SCmdParam *pCmdParam, BOOL fWipeTemps) {
	if (pCmdParam->szParam1 == NULL) {
		return 0;
	}

	HEAP_CHECK_BEGIN(_T("CmdDecryptFile"), 0);
	try {
		CFileIO utFileCipher, utFilePlain;
		CHeaders utHeaders;
		CWrapper utWrap(&utHeaders, pCmdParam->hProgressWnd);
		utFileCipher.Open(CFileName(pCmdParam->szParam1).SetCurDir(pCmdParam->szCurDir).Get(),
                          FALSE,
                          GENERIC_READ|GENERIC_WRITE);

		utHeaders.Load(utFileCipher);

		CPtrTo<TKey> pKeyEncKey;
		if (TryCacheAndPromptOpen(&utHeaders, &pKeyEncKey, pCmdParam->dwBatch, pCmdParam->szParam1)) {
            // Either use the given output name, or
			// Get the real original name from the headers
            CFileName szOriginalName;
            DWORD dwOpenMode = CREATE_NEW;
            szOriginalName.Set(pCmdParam->szParam1);
    		szOriginalName.SetTitle(utHeaders.GetFileName());
            if (pCmdParam->szParam2 && pCmdParam->szParam2[0]) {
                szOriginalName.Override(pCmdParam->szParam2);
                dwOpenMode = CREATE_ALWAYS;
            }

			utFilePlain.Create(szOriginalName.SetCurDir(pCmdParam->szCurDir).Get(), FALSE, GENERIC_READ|GENERIC_WRITE, dwOpenMode);

			HEAP_CHECK_BEGIN(_T("CmdDecryptFile(a)"), 0);
			// Now we have open headers to work with!
			CWrapper utUnwrap(&utHeaders, pCmdParam->hProgressWnd);
			utUnwrap.Unwrap(utFileCipher, utFilePlain, fWipeTemps);
			HEAP_CHECK_END

			// Restore the old file times.
			utFilePlain.SetFileTimes(utHeaders.GetFileTimes());
			utFilePlain.Close(TRUE);			// Override delete-on-close
			
			utFileCipher.Close();				// Close (and retain)
            SHChangeNotify(SHCNE_CREATE, SHCNF_PATH, szOriginalName.Get(), NULL);
        } else {
            if (CRegistry(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValServerMode).GetDword(FALSE)) {
                return ERR_NO_PASSPHRASE;
            } else {
                return WRN_CANCEL;
            }
        }
	} catch (TAssert utErr) {
        // Hide progress window, if any.
        if (pCmdParam->hProgressWnd) {
            SendMessage(GetParent(pCmdParam->hProgressWnd), WM_APP, 0, 0);
        }
        if (utErr.LastError() == WRN_CANCEL) {
            utErr.App(WRN_CANCEL).Show();
        } else {
		    utErr.App(MSG_UNWRAP_ERROR, CFileName(pCmdParam->szParam1).GetTitle()).Show();
        }
		return utErr.LastError();
	}
	return 0;	// Success!
	HEAP_CHECK_END
}
//
//	Test to see if a file is locked. Non-existence is treated as locked, we assume
//	the reason is that the application is busy doing things to it, and that it will
//	reappear soon... The most likely reason is a save by the application.
//
BOOL
IsFileLocked(LPTSTR szFileName) {
	// Now attempt to open the file - Lets try to get ahead of the competition to avoid conflict
	// by increasing the priority for a short while.
	int iThreadPrio = GetThreadPriority(GetCurrentThread);
	(void)SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST);
	HANDLE hFile = CreateFile(szFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
	DWORD dwLastError = GetLastError();
	if (hFile != INVALID_HANDLE_VALUE) (void)CloseHandle(hFile);
	(void)SetThreadPriority(GetCurrentThread(), iThreadPrio);

	if (hFile == INVALID_HANDLE_VALUE) {
		switch (dwLastError) {
		case ERROR_FILE_NOT_FOUND:
		case ERROR_SHARING_VIOLATION:
			return TRUE;
		default:
			SetLastError(dwLastError);
			CAssert(FALSE).Sys(MSG_SYSTEM_CALL, _T("CreateFile() [IsFileLocked()]")).Throw();
		}
	}
	return FALSE;
}

//
//	Wait for a file to become closed, assuming it will be opened in a non-shareable
//	way.
//
//	First Wait for it to become locked a given maximum milliseconds,
//	in increments of given milliseconds.
//
//	Return TRUE if we have determined that the file really was locked and is now released.
//
BOOL
WaitForLockThenUnlock(LPTSTR szFileName, DWORD dwMaxWaitForLock, DWORD dwIncrement) {
	CHChange hChangeNotification;		// Auto-close on exception etc.
	dwMaxWaitForLock -= dwMaxWaitForLock % dwIncrement;		// Ensure even multiple of increments
	// First see if it actually will become locked...
	do {
		// If it has become locked for the first time...
		if (IsFileLocked(szFileName)) {
			hChangeNotification = FindFirstChangeNotification(CFileName(szFileName).GetDir(), FALSE, FILE_NOTIFY_CHANGE_FILE_NAME | FILE_NOTIFY_CHANGE_ATTRIBUTES | FILE_NOTIFY_CHANGE_LAST_WRITE);
			CAssert(hChangeNotification.IsValid()).Sys(MSG_SYSTEM_CALL, _T("FindFirstChangeNotification() [WaitForLockThenUnlock()]")).Throw();
			int i;
			do {
				do {
					// We trust that we will see change notifications, but in some cases we will will not,
					// thus we test every 1/10th of a second.
					DWORD dwWaitResult = WaitForSingleObjectEx(hChangeNotification, 100, FALSE);
					// If a change notification was received...
					if (dwWaitResult == WAIT_OBJECT_0) {
						CAssert(FindNextChangeNotification(hChangeNotification)).Sys(MSG_SYSTEM_CALL, _T("FindNextChangeNotification() [WaitForLockThenUnlock()]")).Throw();
					} else {
						// ... otherwise assert that it was a timeout, not an error.
						CAssert(dwWaitResult == WAIT_TIMEOUT).Sys(MSG_SYSTEM_CALL, _T("WaitForSingleObjectEx() [WaitForLockThenUnlock()]")).Throw();
					}
					// Notification or timeout - check if file still locked.
				} while (IsFileLocked(szFileName));
				// Wait for up to 1/2 second for the file to become re-locked.
				for (i = 500; i > 0; i -= 100) {
					// First we give the other app a chance to re-lock, if it is just a save. This is really far from perfect...
					Sleep(100);	// 1/10th of a second should be enough in most cases, actually the important thing is to yield control
					if (IsFileLocked(szFileName)) break;
				}
				// re-start wait unless we have waited a bit and the file is still free.
			} while (i > 0);
			// Now it has remained unlocked long enough
			CAssert(hChangeNotification.Close()).Sys(MSG_SYSTEM_CALL, _T("CloseHandle(hChangeNotification) [WaitForLockThenUnlock()]")).Throw();
			return TRUE;
			// ... still not locked for the first time - if we should wait more, sleep and decrement.
		} else if (dwMaxWaitForLock) {
			Sleep(dwIncrement);
			dwMaxWaitForLock -= dwIncrement;
		}
	} while (dwMaxWaitForLock);
	return FALSE;
}
//
//	Launch an app and wait until it is done with the file
//
//	There is too much ad hoc code here - there is no really
//	proper way to do it it seems, but better would be to hook
//	file open/close.
//
//	The logic is somewhat convoluted since there is no really one good way to figure out
//	if a launched app is done with a file. An app may, or may not, result in a new process
//	that can be waited for. An app may, or may not, keep the file opened with no sharing
//	as long as the app is open. We try to handle all these cases, sometimes resulting in
//	slightly non-intuitive results for the user unfortunately. Basically the rule will be:
//	close the whole app, then AxCrypt will definitely become aware of the fact. In an MDI-type
//	of app it may not be enough to just close the actual document.
//
//	We have two things to look for: A process to wait for and/or Release of the file. The
//	logic is basically:
//
//	Launch the app - possibly via 'Open With...' dialogue.
//
//	if (file-is-locked-by-app) {
//		wait for file, then clean up. Disregard process.
//	} else if (there is an app to wait for) {
//		wait for app to exit.
//		Give a possible older instance a little bit of time, yield up to 1/10th sec...
//		if (file-is-locked-by-app) {
//			wait for file. (Sometimes an app will move control to an older instance...)
//		}
//		clean up.
//	} else {
//		Give message, and let the user press 'Ok' or 'Wait some more'.
//		when 'Ok' - clean up.
//	}
//
void
LaunchApp(LPTSTR szFileName, HWND hForegroundWnd) {
    BringWindowToTop(hForegroundWnd);

	SHELLEXECUTEINFO ShellInfo;
	ZeroMemory(&ShellInfo, sizeof ShellInfo);
	ShellInfo.hwnd = hForegroundWnd;
	ShellInfo.cbSize = sizeof ShellInfo;
	ShellInfo.fMask = SEE_MASK_FLAG_DDEWAIT | SEE_MASK_FLAG_NO_UI;
	ShellInfo.nShow = SW_NORMAL;
	ShellInfo.lpFile = szFileName;

	//
	// The code below is mostly to compensate for deficiencies in Win32 API
	// concerning tracking of process id's etc, especially finding children.
	//
	
	// Child snapshot object
	CChildProc utAppProc;
	
	// Ensure that no other thread launches an app until we have determined
	// the child process id
	BOOL fLaunchAppOk;
	DWORD dwLaunchErrorCode;
	{
		CCriticalSection utLaunchAppCritical(&gLaunchAppCritical);
		utLaunchAppCritical.Enter();
		utAppProc.TakeSnapshot(GetCurrentProcessId());
		fLaunchAppOk = ShellExecuteEx(&ShellInfo);
		dwLaunchErrorCode = GetLastError();
		utAppProc.TakeSnapshot(GetCurrentProcessId());
		utLaunchAppCritical.Leave();
	}
	DWORD dwAppProcId = utAppProc.GetNewChild();

	// If not launch ok - try open with dialog
	if (!fLaunchAppOk) {
		CAssert(dwLaunchErrorCode == ERROR_NO_ASSOCIATION).File(MSG_SHELL_EXECUTE, szFileName).Throw();

		PROCESS_INFORMATION ProcessInfo;
		ZeroMemory(&ProcessInfo, sizeof ProcessInfo);
		
		STARTUPINFO StartupInfo;
		ZeroMemory(&StartupInfo, sizeof StartupInfo);
		StartupInfo.cb = sizeof StartupInfo;

		CStrPtr szCmdLine = CStrPtr(_T("rundll32.exe shell32.dll,OpenAs_RunDLL ")) + CStrPtr(szFileName);

		CChildProc utOpenWithProc;
		// Ensure that no other thread launches an app until we have determined
		// the child process id
		{
			CCriticalSection utLaunchAppCritical(&gLaunchAppCritical);
			utLaunchAppCritical.Enter();
			utOpenWithProc.TakeSnapshot(GetCurrentProcessId());
			CAssert(CreateProcess(
				NULL,
				szCmdLine,
				NULL,
				NULL,
				FALSE,
				0,
				NULL,
				NULL,
				&StartupInfo,
				&ProcessInfo)).File(MSG_SHELL_EXECUTE, szFileName).Throw();

			// Find the Proc ID of the "Open With..." dialogue.
			utOpenWithProc.TakeSnapshot(GetCurrentProcessId());
			utLaunchAppCritical.Leave();
		}
		
		DWORD dwOpenWithProcId = utOpenWithProc.GetNewChild();
		CAssert(dwOpenWithProcId != 0).File(MSG_OPEN_WITH, szFileName).Throw();
		// Here we don't need a critical sections, since the open with
		// process can not launch multiple children.
		utAppProc.TakeSnapshot(dwOpenWithProcId);

		// Wait for the user to pick something to run.
		(void)MessageWaitForSingleObject(ProcessInfo.hProcess);
		CAssert(CloseHandle(ProcessInfo.hThread)).Sys(MSG_SYSTEM_CALL, _T("CloseHandle(ProcessInfo.hThread) [LaunchApp()]")).Throw();
		CAssert(CloseHandle(ProcessInfo.hProcess)).Sys(MSG_SYSTEM_CALL, _T("CloseHandle(ProcessInfo.hProcess) [LaunchApp()]")).Throw();

		utAppProc.TakeSnapshot(dwOpenWithProcId);
		dwAppProcId = utAppProc.GetNewChild();
	}
	CHandle hAppProc;		// Self-closing if need be
	if (dwAppProcId) {
		hAppProc = OpenProcess(SYNCHRONIZE | PROCESS_QUERY_INFORMATION, FALSE, dwAppProcId);
		if (hAppProc.IsValid()) {
			// Give the process up to 10 seconds to load etc.
			DWORD dwWaitReturn = WaitForInputIdle(hAppProc, 10000);
			if (dwWaitReturn == WAIT_TIMEOUT) {
				CMessage().AppMsg(WRN_INPUT_IDLE_TIMEOUT, CFileName(szFileName).GetTitle()).ShowWarning(MB_OK);
			} else {
				CAssert(dwWaitReturn == 0).Sys().Throw();
				// The wait can terminate successfully because:
				//
				//	1 - The process terminated
				//	2 - The process really is InputIdle
				//
				//	We determine this by closing and reopening the process handle.
				CAssert(hAppProc.Close()).Sys(MSG_SYSTEM_CALL, _T("CloseHandle(hAppProc) [LaunchApp(1)]")).Throw();
				hAppProc = OpenProcess(SYNCHRONIZE | PROCESS_QUERY_INFORMATION, FALSE, dwAppProcId);
			}
		}
		if (!hAppProc.IsValid()) {
			// Check that it was just an old proc id/no known child that was the problem.
			CAssert(GetLastError() == ERROR_INVALID_PARAMETER).Sys().Throw();
		}
	}
	//
	//	Now we either don't know which app was started, or we do know that it is ready for input
	//
	//	Give it up to 1/2 second to lock the file - it turns out that not even waiting for InputIdle
	//	is enough for some apps, notably Acrobat Reader for example, and then wait for it to become
	//	free again.
	if (!WaitForLockThenUnlock(szFileName, 500, 100)) {
		// After all this, we still have not been able to detect a lock on the file! Let's fall back on
		// the process if we have one. Notepad is such an example...
		if (hAppProc.IsValid()) {
			// If we at least think we started a process, let's see if we can wait for it...
			// This might fail, if the process is terminated before we get here.
	
//			CWinTrix().SetProcForegroundWindow(dwChildProcId);
			(void)MessageWaitForSingleObject(hAppProc);
			CMessage().Wrap(0).AppMsg(INF_DEBUG2, _T("End Waiting for process in OpenLaunch"), szFileName).LogEvent();
		} else {
			// if there was no process to wait for, and the file was not immediately locked
			// This is a problem, but let's wait up to 5 seconds to let the app find and lock the file.
			if (!WaitForLockThenUnlock(szFileName, 5000, 100)) {
				// If after 5 seconds the app still did not lock it
				// Display dialog and wait etc.
				CMessage().AppMsg(WRN_CANT_TELL_DONE, CFileName(szFileName).GetTitle()).ShowDialog(MB_ICONWARNING | MB_OK);
			}
		}
	}
	if (hAppProc.IsValid()) {
		CAssert(hAppProc.Close()).Sys(MSG_SYSTEM_CALL, _T("CloseHandle(hAppProc) [LaunchApp(2)]")).Throw();
	}
}
//
//	Deccrypt the named file, launch the application, and wait for exit and free file.
//
// "Primary Execute Request Thread"
//
//	Return status code of the thread.
//
DWORD
CmdDecryptOpenLaunch(struct SCmdParam *pCmdParam) {
	if (pCmdParam->szParam1 == NULL) {
		return 0;
	}

    if (!CRegistry(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValServerMode).GetDword(FALSE)) {

	    HEAP_CHECK_BEGIN(_T("CmdDecryptOpenLaunch()"), 0)
	    // Do not attempt to remove the directory until we exit the function, otherwise there may
	    // be strange effects with left-over files if the destructors are called in the wrong order.
	    CTempDir utTempDir;
	    utTempDir.New();
	    try {
		    // These will auto-close upon destruction.
		    CFileIO utFileCipher, utFilePlain;
		    CHeaders utHeaders;

            utFileCipher.Open(CFileName(pCmdParam->szParam1).SetCurDir(pCmdParam->szCurDir).Get(),
                              FALSE,
                              GENERIC_READ|GENERIC_WRITE);

		    utHeaders.Load(utFileCipher);

		    CPtrTo<TKey> pKeyEncKey;
		    if (TryCacheAndPromptOpen(&utHeaders, &pKeyEncKey, pCmdParam->dwBatch, pCmdParam->szParam1)) {
			    // Get the real original name from the headers and combine it with a unique
			    // temp directory.

			    CFileName szOriginalName(pCmdParam->szParam1);
			    szOriginalName.SetDir(utTempDir.Get());
			    szOriginalName.SetTitle(utHeaders.GetFileName());

			    // Allow some 'leak' here due to file-name caching by CFileIO
			    HEAP_CHECK_BEGIN(_T("CmdDecryptOpenLaunch(a.a)"), TRUE)
			    utFilePlain.Create(szOriginalName.Get(), FALSE, GENERIC_READ|GENERIC_WRITE);	// create always w/delete-on-close
			    HEAP_CHECK_END
	    
                CWrapper utWrap(&utHeaders, pCmdParam->hProgressWnd);

			    HEAP_CHECK_BEGIN(_T("CmdDecryptOpenLaunch(a.b)"), 0)
			    utWrap.Unwrap(utFileCipher, utFilePlain);
			    HEAP_CHECK_END
			    
			    // Restore the old file times.
			    utFilePlain.SetFileTimes(utHeaders.GetFileTimes());
			    utFilePlain.Close(TRUE);				// Keep the plain-file.
			    
			    // Launch appropriate app
			    HEAP_CHECK_BEGIN(_T("CmdDecryptOpenLaunch(a.c)"), 0)
                if (pCmdParam->hProgressWnd != NULL) {
                    // Hide it.
                    SendMessage(GetParent(pCmdParam->hProgressWnd), WM_APP, 0, 0);
                }

			    LaunchApp(szOriginalName.Get(), pCmdParam->hForegroundWnd);

			    HEAP_CHECK_END
				    
			    // Rewrap if modified
			    utFilePlain.Open(szOriginalName.Get(), TRUE, GENERIC_READ|GENERIC_WRITE);
			    if (utHeaders.CompareFileTime(&utFilePlain.GetFileTimes()->LastWriteTime) < 0) {
				    utHeaders.SetFileTimes(utFilePlain.GetFileTimes());
				    utHeaders.WrapKeyData(pKeyEncKey);
				    // Re-wrap it
				    utWrap.Wrap(utFilePlain, utFileCipher);
			    }

			    utFileCipher.Close();					// Close and keep

			    utFilePlain.WipeTemp(pCmdParam->hProgressWnd); 		// Implies delete-on-close
			    utFilePlain.Close();					// Close and delete.
		    }
	    } catch (TAssert utErr) {
            // Hide progress window, if any.
            if (pCmdParam->hProgressWnd) {
                SendMessage(GetParent(pCmdParam->hProgressWnd), WM_APP, 0, 0);
            }
		    utErr.File(MSG_OPEN_LAUNCH, pCmdParam->szParam1).Show();
	    }
	    BringWindowToTop(pCmdParam->hForegroundWnd);
	    return 0;
	    HEAP_CHECK_END
    } else {
        CMessage().Wrap(0).AppMsg(ERR_LOG_OPEN_IN_SERVER_MODE, (LPTSTR)pCmdParam->szParam1).LogEvent(0);
        return WRN_CANCEL;
    }
}
//
//	Securely wipe the file and then delete it. No confirmation question.
//
DWORD
CmdWipeSilent(struct SCmdParam *pCmdParam) {
	if (pCmdParam->szParam1 == NULL) {
		return 0;
	}

	HEAP_CHECK_BEGIN(_T("CmdWipeSilent()"), 0)

    try {    
        // If we should show a possible warning,
        // Check the attributes for those we cannot wipe properly.
        CRegistry utRegWarn(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValNoUnsafeWipeWarn);
        if (!utRegWarn.GetDword()) {
            DWORD dwAttrib = GetFileAttributes(CFileName(pCmdParam->szParam1).SetCurDir(pCmdParam->szCurDir).Get());
            CAssert(dwAttrib != 0xFFFFFFFF).Sys(MSG_SYSTEM_CALL, pCmdParam->szParam1).Throw();
            if (dwAttrib & (FILE_ATTRIBUTE_COMPRESSED|FILE_ATTRIBUTE_ENCRYPTED|FILE_ATTRIBUTE_SPARSE_FILE)) {
                if (WarningDlg(pCmdParam->szParam1, WRN_INSECURE_WIPE, INF_DONTREPEAT)) {
                    utRegWarn.SetDword(TRUE);
                }
            }
        }

	    CFileIO utFile2Wipe;
	    utFile2Wipe.Open(CFileName(pCmdParam->szParam1).SetCurDir(pCmdParam->szCurDir).Get(),
                         TRUE,
                         GENERIC_READ|GENERIC_WRITE);
	    utFile2Wipe.WipeData(pCmdParam->hProgressWnd);		// Also sets delete on close
	    utFile2Wipe.Close();

        SHChangeNotify(SHCNE_DELETE, SHCNF_PATH, pCmdParam->szParam1, NULL);
	} catch (TAssert utErr) {
        // Hide progress window, if any.
        if (pCmdParam->hProgressWnd) {
            SendMessage(GetParent(pCmdParam->hProgressWnd), WM_APP, 0, 0);
        }
		utErr.File(MSG_WIPE_ERROR, pCmdParam->szParam1).Show();
	}
	HEAP_CHECK_END
	return 0;
}
//
//	Securely wipe the file and the delete it. First ask 'are you sure...'.
//  If there is a batch-id, we ask 'yes, no, cancel', otherwise just
//  yes - no.
//
DWORD
CmdWipe(struct SCmdParam *pCmdParam) {
	if (pCmdParam->szParam1 == NULL) {
		return 0;
	}

	HEAP_CHECK_BEGIN(_T("CmdWipe()"), 0)

    BringWindowToTop(pCmdParam->hForegroundWnd);

    switch (MessageBox(pCmdParam->hForegroundWnd,
		               CMessage().AppMsg(WRN_REALLY_WIPE, (TCHAR *)(pCmdParam->szParam1)).GetMsg(),
		               CVersion().String(),
                       (pCmdParam->dwBatch ? MB_YESNOCANCEL : MB_YESNO) | MB_ICONERROR | MB_DEFBUTTON2 | MB_TOPMOST)) {
    case IDYES:
        return CmdWipeSilent(pCmdParam);
        break;

    case IDCANCEL:
        // This will cause batched operations to stop.
        return WRN_CANCEL;
        break;
    
    case IDNO:
    default:
        break;
	}
	HEAP_CHECK_END
    return 0;
}
//
//  Clear keys in memory, belonging to a specific batch if given,
//  otherwise all.
//
DWORD
CmdClearKeys(struct SCmdParam *pCmdParam) {
    pgKeyList->ClearKeys(pCmdParam->dwBatch);
    return 0;
}
//
//  Add keys to our memory. Handle especially if it is the default
//  encryption key.
//
//  Careful... Here we should be working in ANSI to maintain key
//  compatibility...
//
DWORD CmdAddKey(struct SCmdParam *pCmdParam, BOOL fIsEncKey) {
	
	// Just ignore empty key requests.
	if (pCmdParam->szParam1 == NULL) {
		return 0;
	}
    
	//
    //  Filter the key using the same criteria as the the Password entry
    //  dialog. Remember about Unicode and Ansi...
    //
    char *pNxtInChar = pCmdParam->szParam1;
    
    // Ensure that szFilterKey is deallocated on exit of this function.
    CPtrTo<char> szFilterKey;
    szFilterKey = new char[strlen(pNxtInChar) + 1];
    CAssert(szFilterKey != NULL).Sys(MSG_MEMORY_ALLOC, _T("CmdAddKey [new szFilterKey]")).Throw();
    
    char *pNxtOutChar = szFilterKey;

    while (*pNxtInChar) {
        if (strchr((const char *)szPassphraseChars, *pNxtInChar) != NULL) {
            *pNxtOutChar++ = *pNxtInChar;
        }
        pNxtInChar++;
    }
    *pNxtOutChar = '\0';

    // We are responsible for ensuring the deletion of the returned key hash pointer
    CPtrTo<TKey> utKey = CSha1().GetKeyHash((BYTE *)(char *)szFilterKey, strlen(szFilterKey));

    if (fIsEncKey) {
        // AddEncKey checks with find key first...
        (void)pgKeyList->AddEncKey(utKey, pCmdParam->dwBatch);
    } else {
        // Check if we already know the key...
        if (!pgKeyList->FindKey(utKey, pCmdParam->dwBatch)) {
            // ...if not - add it to the list of known keys.
            pgKeyList->AddKey(utKey, FALSE, pCmdParam->dwBatch);
        }
    }
    return 0;
}
//
//  Ask for keys using the regular prompts, thus ensuring a safe way
//  to enter keys into the cache.
//
DWORD
CmdPromptKey(struct SCmdParam *pCmdParam, BOOL fIsEncKey) {
    CKeyPrompt utKeyPrompt;
    DWORD dwReturn = 0;

    if (fIsEncKey) {
        (void)utKeyPrompt.New();
		// If the user entered a key, and did not cancel
		if (utKeyPrompt.Get() != NULL) {
            // Test if we really should save the key here.
            BOOL fSaveEncKey = CRegistry(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValSaveEncKey).GetDword();
            if (fSaveEncKey || pCmdParam->dwBatch != 0) {
    			// This is may be a permanent allocation here.
				HEAP_CHECK_BEGIN(_T("CmdPromptKey(a)"), TRUE);
                pgKeyList->AddEncKey(utKeyPrompt.Get(), fSaveEncKey ? 0 : pCmdParam->dwBatch)->Key();
				HEAP_CHECK_END
            }
		} else {
            dwReturn = WRN_CANCEL;
        }
    } else {
        (void)utKeyPrompt.Old(INF_ENTER_PASS, _T(""));
    	if (utKeyPrompt.Get() != NULL) {
            BOOL fSaveDecKey = CRegistry(HKEY_CURRENT_USER, gszAxCryptRegKey, szRegValSaveDecKey).GetDword(FALSE);
            if (fSaveDecKey || pCmdParam->dwBatch != 0) {
    			// This is may be a permanent allocation here.
				HEAP_CHECK_BEGIN(_T("CmdPromptKey(b)"), TRUE);
                pgKeyList->AddKey(utKeyPrompt.Get(), FALSE, fSaveDecKey ? 0 : pCmdParam->dwBatch);
                HEAP_CHECK_END
            }
        } else {
            dwReturn = WRN_CANCEL;
        }
    }
    return dwReturn;
}
//
//  Rename the input file to a unique, anonymous name.
//
DWORD
CmdAnonRename(struct SCmdParam *pCmdParam) {
	if (pCmdParam->szParam1 == NULL) {
		return 0;
	}

	HEAP_CHECK_BEGIN(_T("CmdWipeSilent()"), 0)
    try {    
		CFileIO utFileCipher;
		CHeaders utHeaders;
        CFileName utCipherName(pCmdParam->szParam1);
		utFileCipher.Open(utCipherName.SetCurDir(pCmdParam->szCurDir).Get(),
                          FALSE,
                          GENERIC_READ);

		utHeaders.Load(utFileCipher);
        
        // Get the low 32 bits of the HMAC.
        DWORD dwHmacLow = (DWORD)((DQWORD *)utHeaders.GetHMAC()->Lsb64());
        utFileCipher.Close();

        BOOL fOk;
        CFileName utAnonName = utCipherName;
        int nMaxTry = 25;   // Maximize retries to avoid possibility of looping.
        do {
            utAnonName.SetName(CPtrTo<TCHAR>(FormatSz("%1!08X!", dwHmacLow++)));
        } while (!(fOk = MoveFile(utCipherName.Get(), utAnonName.Get())) && GetLastError() == ERROR_ALREADY_EXISTS && --nMaxTry);
        CAssert(fOk).Sys(MSG_SYSTEM_CALL, _T("CmdAnonRename() [MoveFile()]")).Throw();
        SHChangeNotify(SHCNE_RENAMEITEM, SHCNF_PATH, utCipherName.Get(), utAnonName.Get());
    } catch (TAssert utErr) {
		utErr.Show();
		return utErr.LastError();
    }
    HEAP_CHECK_END
    return 0;
}

DWORD
CmdTestHaveKey(struct SCmdParam *pCmdParam) {
	if (pCmdParam->szParam1 == NULL) {
		return 0;
	}

	// These will auto-close upon destruction.
	CFileIO fioCipher;
	CHeaders Headers;

	fioCipher.Open(CFileName(pCmdParam->szParam1).SetCurDir(pCmdParam->szCurDir).Get(), FALSE, GENERIC_READ);

	Headers.Load(fioCipher);

	CPtrTo<TKey> pKeyEncKey;
    if (pgKeyList->TryOpen(&Headers, &pKeyEncKey, pCmdParam->dwBatch)) {
	    return 0;
    }
    return WRN_NO_HAVE_KEY;
}
//
//	Output the id-tag, if any on standard output.
//
DWORD
CmdShowIdTag(struct SCmdParam *pCmdParam) {
	if (pCmdParam->szParam1 == NULL) {
		return 0;
	}

	try {
		CHeaders utHeaders;
		CFileIO utFileCipher;
		utFileCipher.Open(CFileName(pCmdParam->szParam1).SetCurDir(pCmdParam->szCurDir).Get(),
                          FALSE,
                          GENERIC_READ|GENERIC_WRITE);

		utHeaders.Load(utFileCipher);
		CPtrTo<TCHAR> szIdTag = utHeaders.GetIdTag();
		if (szIdTag) {
			if (pCmdParam->hStdOut == NULL || pCmdParam->hStdOut == INVALID_HANDLE_VALUE) {
				CAssert(FALSE, ERR_NO_STDOUT).App(ERR_NO_STDOUT).Throw();
			}
			int fd = _open_osfhandle((long)(HANDLE)pCmdParam->hStdOut, _O_TEXT|_O_APPEND);
			FILE *fp = fdopen(fd, "w+t");
			(void)fprintf(fp, "%s\n", (LPTSTR)szIdTag);
			(void)fflush(fp);
			// I don't think we should do any close here...
			//(void)fclose(fp); 
			//(void)close(fd);
		} else {
			return ERR_NO_IDTAG;
		}

		utFileCipher.Close();				// Close (and delete)
	} catch (TAssert utErr) {
        // Hide progress window, if any.
        if (pCmdParam->hProgressWnd) {
            SendMessage(GetParent(pCmdParam->hProgressWnd), WM_APP, 0, 0);
        }
		utErr.File(ERR_FILE, pCmdParam->szParam1).Show();
		return utErr.LastError();
	}
	return 0;

}

//
// There are a number of registry settings that need to be set. The extension
// ".EXT" shall be associated with the "open" action.
// We must also establish a "universial" ContextMenuHandler for right-clicking.
// Assumption is that the dll and exe resides in the same directory and have the
// same name, except for extension .exe and .dll. We also assume we are running
// from the proper directory, i.e. we are installed in place.
//
// So... For the ContextMenuHandler Shell Extension we need:
//
// [HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\Shell Extensions\Approved]
// gszAxCryptCLSID = szAxCryptProgDesc
//
// [HKEY_CLASSES_ROOT\*\shellex\ContextMenuHandlers\szAxCryptProgID]
// @ = gszAxCryptCLSID
//
// [HKEY_CLASSES_ROOT\CLSID\gszAxCryptCLSID]
// @ = szAxCryptProgID
//
// [HKEY_CLASSES_ROOT\CLSID\gszAxCryptCLSID\InprocServer32]
// @ = path-to-dll
// "ThreadingModel" = "Apartment"
//
// For the "open" action association with the ".EXT" extension (szAxCryptFileExtension)
//
// [HKEY_CLASSES_ROOT\szAxCryptFileExtension]
// @ = szAxCryptProgID
//
// [HKEY_CLASSES_ROOT\szAxCryptProgID]
// @ = szAxCryptProgDesc
//
// [HKEY_CLASSES_ROOT\szAxCryptProgID\DefaultIcon]
// @ = \"path-to-exe\", 0
// 
// [HKEY_CLASSES_ROOT\szAxCryptProgID\CLSID]
// @ = gszAxCryptCLSID
//
// [HKEY_CLASSES_ROOT\szAxCryptProgID\PropertySheetHandlers\gszAxCryptCLSID]
//
// [HKEY_CLASSES_ROOT\szAxCryptProgID\shell\open\command]
// @ = \"path-to-exe\" \"%1\"
//
//  For global application options and state
//
//  [HKEY_CURRENT_USER\Software\Axon Data\AxCrypt] aka szAxCryptRegKey, szRegValServerMode
//  "ServerMode" = 0        ;!= 0 means we enter quiet server mode
//
// For the application state:
//
// [HKEY_CURRENT_USER\Software\Axon Data\AxCrypt] aka szAxCryptRegKey, szRegValEventLogLevel
// "EventLogLevel" = 0			; >0 means we log events, depending on the level of detail.
//
// [HKEY_CURRENT_USER\Software\Axon Data\AxCrypt] aka szAxCryptRegKey, szRegValNoUnsafeWipeWarn
// "NoUnsafeWipeWarn" = 0			; TRUE => Don't warn about unsafe wipes any more.
//
// [HKEY_CURRENT_USER\Software\Axon Data\AxCrypt] aka szAxCryptRegKey, szRegValSaveEncKey
// "SaveEncKey" = 0			; TRUE => Do save enc passphrases in the cache, and use as default.
//
// [HKEY_CURRENT_USER\Software\Axon Data\AxCrypt] aka szAxCryptRegKey, szRegValSaveDecKey
// "SaveDecKey" = 1			; TRUE => Do save dec passphrases in the cache.
//
void CmdInstallInRegistry(struct SCmdParam *pCmdParam) { 
	CFileName szPath2Exe; szPath2Exe.SetPath2ExeName();
	CFileName szPath2Dll(szPath2Exe); szPath2Dll.SetExt(_T(".dll"));
    CFileName szPath2Ico(szPath2Exe); szPath2Ico.SetExt(_T(".ico"));

	try {
        CRegistry utRegKey;

        // First of all - let's make a CLSID for this installation instance.
        utRegKey.Root(HKEY_LOCAL_MACHINE).Key(gszAxCryptRegKey).Value(szRegValCLSID);
        // Check that we are not already installed with GUID.
        if (CPtrTo<TCHAR>(utRegKey.GetSz(NULL)) == NULL) {
            GUID guid;
            CAssertEq(CoCreateGuid(&guid), S_OK).Sys().Throw();
    
            LPOLESTR oleszCLSID;
            CAssertEq(StringFromCLSID(guid, &oleszCLSID), S_OK).Sys().Throw();
            gszAxCryptCLSID = CopyWzSz(oleszCLSID);
            CoTaskMemFree(oleszCLSID);
        } else {
            CAssert(FALSE, ERROR_FILE_EXISTS).Sys().Throw();
        }

        // Then, Add the Approved key
        // [HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\Shell Extensions\Approved]
        // gszAxCryptCLSID = szAxCryptProgDesc
        LONG lRes;
        HKEY hRegKey;
		switch(lRes = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                                   _T("Software\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved"),
                                   0,
                                   KEY_WRITE,
                                   &hRegKey)) {
		case ERROR_ACCESS_DENIED:
			// 
			// The user does not have permission to add a new value to this key. In this 
			// case, a reasonable action would be to warn the user that some 
			// application features will not be available unless an administrator 
			// installs the application. If the shell extension is central to the 
			// functioning of the application, tell the user that the install 
			// can only be performed by an administrator, and stop the install. 
			//
			CAssert(FALSE, lRes).Sys();
			break;
		case ERROR_FILE_NOT_FOUND:
			// 
			// The key does not exist. This should only happen if setup is running 
			// on Windows 95 instead of Windows NT, or if you are installing on an older 
			// version of either operating system that does not have the new shell. 
			// We ignore this.
			break;
		default:
			// Now let's actually do the work.
            CRegistry().HKey(hRegKey).Value(gszAxCryptCLSID).SetSz(gszAxCryptProgDesc);
		}

        // Next order of the day - fix the extension to use
        utRegKey.Root(HKEY_LOCAL_MACHINE).CreateKey(gszAxCryptRegKey);
        utRegKey.Value(szRegValFileExt);
        if (pCmdParam->szParam1 == NULL) {
            gszAxCryptFileExt = CopySz(szAxCryptDefFileExt);
        } else {
            gszAxCryptFileExt = CopySz(CFileName(pCmdParam->szParam1).GetExt());
        }
        utRegKey.SetSz(gszAxCryptFileExt);


        //  [HKEY_CLASSES_ROOT] section
        utRegKey.Root(HKEY_CLASSES_ROOT);

        //  [HKEY_CLASSES_ROOT\*\shellex\ContextMenuHandlers\szAxCryptProgID]
		//  @ = szAxCryptCLSID
        utRegKey.CreateKey(_T("*\\shellex\\ContextMenuHandlers\\%1"), gszAxCryptProgID);
        utRegKey.Value(_T("")).SetSz(gszAxCryptCLSID);

        //  [HKEY_CLASSES_ROOT\Folder\shellex\ContextMenuHandlers\szAxCryptProgID]
		//  @ = gszAxCryptCLSID
        utRegKey.CreateKey(_T("Folder\\shellex\\ContextMenuHandlers\\%1"), gszAxCryptProgID);
        utRegKey.Value(_T("")).SetSz(gszAxCryptCLSID);

        //  [HKEY_CLASSES_ROOT\CLSID\gszAxCryptCLSID]
		//  @ = szAxCryptProgID
        utRegKey.CreateKey(_T("CLSID\\%1"), gszAxCryptCLSID);
        utRegKey.Value(_T("")).SetSz(gszAxCryptProgID);

		//  [HKEY_CLASSES_ROOT\CLSID\gszAxCryptCLSID\InprocServer32]
		//  @ = path-to-dll
		//  "ThreadingModel" = "Apartment"
        utRegKey.CreateKey(_T("CLSID\\%1\\InprocServer32"), gszAxCryptCLSID);
        utRegKey.Value(_T("")).SetSz(szPath2Dll.Get());
        utRegKey.Value(_T("ThreadingModel")).SetSz(_T("Apartment"));

		//
		// For the "open" action association with the ".EXT" extension (szAxCryptFileExtension)
		//
		//  [HKEY_CLASSES_ROOT\szAxCryptFileExtension]
		//  @ = szAxCryptProgID
        utRegKey.CreateKey(gszAxCryptFileExt);
        utRegKey.Value(_T("")).SetSz(gszAxCryptProgID);

		//  [HKEY_CLASSES_ROOT\szAxCryptProgID]
		//  @ = INF_FILE_TYPE_NAME	(Language dependant!)
        utRegKey.CreateKey(gszAxCryptProgID);
        utRegKey.Value(_T("")).SetSz(CMessage().AppMsg(INF_FILE_TYPE_NAME).GetMsg());

		//  [HKEY_CLASSES_ROOT\szAxCryptProgID\CLSID]
		//  @ = gszAxCryptCLSID
        utRegKey.CreateKey(_T("%1\\CLSID"), gszAxCryptProgID);
        utRegKey.Value(_T("")).SetSz(gszAxCryptCLSID);

        //  [HKEY_CLASSES_ROOT\szAxCryptProgID\DefaultIcon]
		//  @ = \"path-to-ico\", 0
        utRegKey.CreateKey(_T("%1\\DefaultIcon"), gszAxCryptProgID);
        utRegKey.Value(_T("")).SetSz(_T("%1,0"), szPath2Ico.Get());

		//  [HKEY_CLASSES_ROOT\szAxCryptProgID\shellex\PropertySheetHandlers\gszAxCryptCLSID]
        utRegKey.CreateKey(_T("%1\\shellex\\PropertySheetHandlers"), gszAxCryptProgID);
        CRegistry(utRegKey.GetHKey()).CreateKey(gszAxCryptCLSID);

        //  [HKEY_CLASSES_ROOT\szAxCryptProgID\shell\open\command]
		//  @ = \"path-to-exe\" \"%1\"
        utRegKey.CreateKey(_T("%1\\shell\\open\\command"), gszAxCryptProgID);
        utRegKey.Value(_T("")).SetSz(_T("\"%1\" \"%%1\""), szPath2Exe.Get());

        //  [HKEY_LOCAL_MACHINE]
        utRegKey.Root(HKEY_LOCAL_MACHINE).Key(gszAxCryptRegKey);

        utRegKey.Value(szRegValProductName).SetSz(CVersion(ghInstance).ExtProductName());
        utRegKey.Value(szRegValCLSID).SetSz(gszAxCryptCLSID);

        //  [HKEY_CURRENT_USER] section
        utRegKey.Root(HKEY_CURRENT_USER);

        //  [HKEY_CURRENT_USER\Software\Axon Data\AxCrypt] aka szAxCryptRegKey, szRegValServerMode
        utRegKey.CreateKey(gszAxCryptRegKey);

        //  "ServerMode" = 0        ;!= 0 means we enter quiet server mode
        utRegKey.Value(szRegValServerMode).SetDword(0);

        // "ServerErrorShellCmd" = "" -> means nothing happens here by default.
        utRegKey.Value(szRegValServerErrorShell).SetSz(_T(""));

		// "EventLogLevel" = 0			; >0 means we log events, depending on the level of detail.
		utRegKey.Value(szRegValEventLogLevel).SetDword(0);

		// "KeyWrapIterations" = 10000	; Current default.
		utRegKey.Value(szRegValKeyWrapIterations).SetDword(KEY_WRAP_ITERATIONS);

		// "NoUnsafeWipeWarn" = 0			; TRUE => Don't warn about unsafe wipes any more.
		utRegKey.Value(szRegValNoUnsafeWipeWarn).SetDword(FALSE);

		// "SaveEncKey" = 0			; TRUE => Do save enc passphrases in the cache, and use as default.
		utRegKey.Value(szRegValSaveEncKey).SetDword(FALSE);

		// "SaveDecKey" = 1			; TRUE => Do save dec passphrases in the cache.
		utRegKey.Value(szRegValSaveDecKey).SetDword(TRUE);

		// "NoDecryptMenu" = 0		; TRUE => Do not show the decrypt menu in the shell extension.
		utRegKey.Value(szRegValNoDecryptMode).SetDword(FALSE);

        // "CompressThreshold = COMPRESS_THRESHOLD. All expected compression ratios
        // higher or equal to this value leads to compression, otherwise it's skipped.
        utRegKey.Value(szRegValCompressLevel).SetDword(COMPRESS_THRESHOLD);

        // Enable rename menu by default.
        utRegKey.Value(szRegValNoRenameMenu).SetDword(FALSE);
    } catch (TAssert utErr) {
		utErr.App(MSG_INSTALL_ERROR).Show();
	}
}

// Remove all that was installed in "Install in Registry". We do not duplicate that list here
// to make it easier to update. Please check above.
void CmdRemoveFromRegistry(void) {
	try {
		// [HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\Shell Extensions\Approved]
		// gszAxCryptCLSID = szAxCryptProgDesc
        CRegistry utReg;
        utReg.Root(HKEY_LOCAL_MACHINE);
        utReg.Key(_T("Software\\Microsoft\\Windows\\CurrentVersion\\Shell Extensions\\Approved"));
        utReg.Value(gszAxCryptCLSID).DelValue();

        // [HKEY_CLASSES_ROOT\*\shellex\ContextMenuHandlers\szAxCryptProgID]
        utReg.Root(HKEY_CLASSES_ROOT);
        utReg.Key(_T("*\\shellex\\ContextMenuHandlers"));
        utReg.DelSubKey(gszAxCryptProgID);

		// [HKEY_CLASSES_ROOT\Folder\shellex\ContextMenuHandlers\szAxCryptProgID]
        utReg.Key(_T("Folder\\shellex\\ContextMenuHandlers"));
        utReg.DelSubKey(gszAxCryptProgID);

        // [HKEY_CLASSES_ROOT\CLSID\gszAxCryptCLSID\InprocServer32]
		// [HKEY_CLASSES_ROOT\CLSID\gszAxCryptCLSID]
        if (gszAxCryptCLSID && *gszAxCryptCLSID) {
            utReg.Key(_T("CLSID"));
            {
                CRegistry utReg2;
                utReg2.Root(utReg.GetHKey()).Key(gszAxCryptCLSID);
                utReg2.DelSubKey(_T("InprocServer32"));
            }
            utReg.DelSubKey(gszAxCryptCLSID);
        }

        // [HKEY_CLASSES_ROOT\szAxCryptFileExtension]
        utReg.DelSubHKey(HKEY_CLASSES_ROOT, gszAxCryptFileExt);

		// [HKEY_CLASSES_ROOT\szAxCryptProgID\shell\open\command]
		// [HKEY_CLASSES_ROOT\szAxCryptProgID\CLSID]
		// [HKEY_CLASSES_ROOT\szAxCryptProgID\DefaultIcon]
		// [HKEY_CLASSES_ROOT\szAxCryptProgID]

        if (utReg.Key(gszAxCryptProgID).GetHKey()) {
			if (gszAxCryptCLSID && *gszAxCryptCLSID) {
				// [HKEY_CLASSES_ROOT\szAxCryptProgID\shellex\PropertySheetHandlers\gszAxCryptCLSID]
				CRegistry utReg2;
				utReg2.Root(utReg.GetHKey());
				utReg2.Key(_T("shellex\\PropertySheetHandlers"));
				utReg2.DelSubKey(gszAxCryptCLSID);
			}
			utReg.DelSubKey(_T("shellex\\PropertySheetHandlers"));
			utReg.DelSubKey(_T("shellex"));
			utReg.DelSubKey(_T("shell\\open\\command"));
			utReg.DelSubKey(_T("shell\\open"));
			utReg.DelSubKey(_T("shell"));
			utReg.DelSubKey(_T("CLSID"));
			utReg.DelSubKey(_T("DefaultIcon"));
		}

        utReg.DelSubHKey(HKEY_CLASSES_ROOT, gszAxCryptProgID);

		// [HKEY_CURRENT_USER\Software\Axon Data\AxCrypt] aka szAxCryptRegKey
        utReg.DelSubHKey(HKEY_CURRENT_USER, gszAxCryptRegKey);

        // HKEY_LOCAL_MACHINE\gszAxCryptRegKey
        utReg.Root(HKEY_LOCAL_MACHINE).Key(gszAxCryptRegKey);
        utReg.Value(szRegValProductName).DelValue();
        utReg.Value(szRegValCLSID).DelValue();
        utReg.Value(szRegValFileExt).DelValue();

	} catch (TAssert utErr) {
		utErr.App(MSG_UNINSTALL_ERROR).Show();
	}
}
