/*
    @(#) $Id: GetOption.cpp,v 1.3 2002/08/11 16:47:09 svante Exp $

	AxCrypt - Compressing and Encrypting Wrapper and Application Launcher for Secure Local,
	Server or Web Storage of Document Files.

	Copyright (C) 2001 Svante Seleborg/Axon Data, All rights reserved.

	This program is free software; you can redistribute it and/or modify it under the terms
	of the GNU General Public License as published by the Free Software Foundation;
	either version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
	Boston, MA 02111-1307 USA

	The author may be reached at mailto:axcrypt@axondata.se and http://axcrypt.sourceforge.net
----
	GetOption.cpp					Parse the command line, standard stuff. This is taken
									from somewhere, can't remember, but it should be free
									to use.

	E-mail							YYYY-MM-DD				Reason
	axcrypt@axondata.se 			2001					Initial

*/
/////////////////////////////////////////////////////////////////////////////// 
// 
//  FILE: getopt.c 
// 
//      GetOption function 
// 
//  FUNCTIONS: 
// 
//      GetOption() - Get next command line option and parameter 
// 
//  COMMENTS: 
// 
/////////////////////////////////////////////////////////////////////////////// 
 
#include "StdAfx.h"
#include <stddef.h> 
#include <ctype.h> 
#include <string.h> 
 
/////////////////////////////////////////////////////////////////////////////// 
// 
//  FUNCTION: GetOption() 
// 
//      Get next command line option and parameter 
// 
//  PARAMETERS: 
// 
//      argc - count of command line arguments 
//      argv - array of command line argument strings 
//      pszValidOpts - string of valid, case-sensitive option characters, 
//                     a colon ':' following a given character means that 
//                     option can take a parameter 
//      ppszParam - pointer to a pointer to a string for output 
// 
//  RETURNS: 
// 
//      If valid option is found, the character value of that option 
//          is returned, and *ppszParam points to the parameter if given, 
//          or is NULL if no param 
//      If standalone parameter (with no option) is found, 1 is returned, 
//          and *ppszParam points to the standalone parameter 
//      If option is found, but it is not in the list of valid options, 
//          -1 is returned, and *ppszParam points to the invalid argument 
//      When end of argument list is reached, 0 is returned, and 
//          *ppszParam is NULL 
// 
//  COMMENTS: 
// 
/////////////////////////////////////////////////////////////////////////////// 
 
int GetOption (int argc, 
			   char** argv, 
			   char* pszValidOpts, 
			   char** ppszParam) { 
    static int iArg = 1; 
    char chOpt; 
    char* psz = NULL; 
    char* pszParam = NULL; 
 
    if (iArg < argc) { 
        psz = &(argv[iArg][0]); 
        if (*psz == '-' || *psz == '/') { 
            // we have an option specifier 
            chOpt = argv[iArg][1]; 
            if (isalnum(chOpt) || ispunct(chOpt)) { 
                // we have an option character 
                psz = strchr(pszValidOpts, chOpt); 
                if (psz != NULL) { 
                    // option is valid, we want to return chOpt 
                    if (psz[1] == ':') { 
                        // option can have a parameter 
                        psz = &(argv[iArg][2]); 
                        if (*psz == '\0') { 
                            // must look at next argv for param 
                            if (iArg+1 < argc) { 
                                psz = &(argv[iArg+1][0]); 
                                if (*psz == '-' || *psz == '/') { 
                                    // next argv is a new option, so param 
                                    // not given for current option 
                                } else { 
                                    // next argv is the param 
                                    iArg++; 
                                    pszParam = psz; 
                                } 
                            } else { 
                                // reached end of args looking for param 
                            } 
                         } else { 
                            // param is attached to option 
                            pszParam = psz; 
                        } 
                    } else { 
                        // option is alone, has no parameter 
                    } 
                } else { 
                    // option specified is not in list of valid options 
                    chOpt = -1; 
                    pszParam = &(argv[iArg][0]); 
                } 
            } else { 
                // though option specifier was given, option character 
                // is not alpha or was was not specified 
                chOpt = -1; 
                pszParam = &(argv[iArg][0]); 
            } 
        } else { 
            // standalone arg given with no option specifier 
            chOpt = 1; 
            pszParam = &(argv[iArg][0]); 
        } 
    } else { 
        // end of argument list 
        chOpt = 0; 
    } 
    iArg++; 
    *ppszParam = pszParam; 
    return (chOpt); 
}
