/*
    @(#) $Id: Utility.cpp,v 1.4 2002/08/23 16:42:03 svante Exp $

	AxCrypt - Compressing and Encrypting Wrapper and Application Launcher for Secure Local,
	Server or Web Storage of Document Files.

	Copyright (C) 2001 Svante Seleborg/Axon Data, All rights reserved.

	This program is free software; you can redistribute it and/or modify it under the terms
	of the GNU General Public License as published by the Free Software Foundation;
	either version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
	Boston, MA 02111-1307 USA

	The author may be reached at mailto:axcrypt@axondata.se and http://axcrypt.sourceforge.net
----
	Utility.cpp						Various context-free utility functions.
	
	E-mail							YYYY-MM-DD				Reason
	axcrypt@axondata.se 			2002-08-05				Rel 1.2	Initial
                                    2002-08-16              RegCloseKey instead of CloseHandle
                                                            Additional functions in CRegistry

*/
#include	"StdAfx.h"
//#include    <stdio.h>
#include    <stdarg.h>
#include    "Utility.h"

LPTSTR CopySz(LPCTSTR szSrc) {
    if (szSrc == NULL) return NULL;
    LPTSTR szDst = new TCHAR[_tcslen(szSrc) + 1];
    CAssert(szDst != NULL).App(MSG_MEMORY_ALLOC, _T("CopySz [new]")).Throw();
    return _tcscpy(szDst, szSrc);
}
//
//  Return an allocated formatted string, please remember to delete.
//
LPTSTR FormatSz(LPCTSTR szFormat, ...) {
    va_list argPtr;
    va_start(argPtr, szFormat);
    LPTSTR szFormattedValue;

    CAssert(FormatMessage(FORMAT_MESSAGE_FROM_STRING | FORMAT_MESSAGE_ALLOCATE_BUFFER,
                          szFormat,
                          0,
                          0,
                          (LPTSTR)&szFormattedValue,
                          0,
                          &argPtr)).Sys(MSG_SYSTEM_CALL, _T("FormatSz() [FormatMessage()]")).Throw();
    va_end(argPtr);
    LPTSTR szReturn = CopySz(szFormattedValue);
    // First free the buffer, then assert...
    LocalFree(szFormattedValue);

    return szReturn;
}

static UINT GetSizePidl(LPCITEMIDLIST pidl) {
    UINT cbTotal = 0;
    if (pidl != NULL) {
        while (pidl->mkid.cb) {
            cbTotal += pidl->mkid.cb;
            pidl = (LPCITEMIDLIST)((BYTE *)pidl + pidl->mkid.cb);
        }
        cbTotal += sizeof pidl->mkid.cb;    // Room for zero count too.
    }
    return cbTotal;
}

LPITEMIDLIST CopyPidl(IMalloc *pMalloc, LPCITEMIDLIST pidl) {
    UINT cb = 0;
	
    // Calculate size of list.
    cb = GetSizePidl(pidl);

    LPITEMIDLIST pidlRet = (LPITEMIDLIST)pMalloc->Alloc(cb);
    CAssert(pidlRet != NULL).App(MSG_MEMORY_ALLOC, _T("CopyPidl [Alloc()]")).Throw();
    CopyMemory(pidlRet, pidl, cb);
    return pidlRet;
}

WCHAR *
CopySzWz(LPCSTR szIn) {
    if (szIn == NULL) return NULL;

    int iLen = MultiByteToWideChar(CP_ACP, 0, szIn, -1, NULL, 0);
	WCHAR *wzOut = new WCHAR[iLen];
    CAssert(wzOut != NULL).App(MSG_MEMORY_ALLOC, _T("CopySzWz [new]")).Throw();
	MultiByteToWideChar(CP_ACP, 0, szIn, -1, wzOut, iLen);
    return wzOut;
}

LPSTR
CopyWzSz(WCHAR *wzIn) {
    if (wzIn == NULL) return NULL;

    int iLen = WideCharToMultiByte(CP_ACP, 0, wzIn, -1, NULL, 0, NULL, NULL);
	char *szOut = new char[iLen];
    CAssert(szOut != NULL).App(MSG_MEMORY_ALLOC, _T("CopyWzSz [new]")).Throw();
	WideCharToMultiByte(CP_ACP, 0, wzIn, -1, szOut, iLen, NULL, NULL);
    return szOut;
}

void DebugBox(LPTSTR szMsg) {
    (void)MessageBox(NULL, szMsg, _T("AxCrypt Debug Message"), MB_OK);
}

