/*
    @(#) $Id: CtxMenu.cpp,v 1.17 2003/01/06 20:28:03 svante Exp $

	AxCrypt - Compressing and Encrypting Wrapper and Application Launcher for Secure Local,
	Server or Web Storage of Document Files.

	Copyright (C) 2001 Svante Seleborg/Axon Data, All rights reserved.

	This program is free software; you can redistribute it and/or modify it under the terms
	of the GNU General Public License as published by the Free Software Foundation;
	either version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
	Boston, MA 02111-1307 USA

	The author may be reached at mailto:axcrypt@axondata.se and http://axcrypt.sourceforge.net
----
	ContextMenu.cpp					IContextMenu implementatino

	E-mail							YYYY-MM-DD				Reason
	axcrypt@axondata.se 			2001					Initial
                                    2002-07-26              Ver 1.2

*/
#include	"StdAfx.h"
#include	"CFileName.h"
#include	"process.h" 
#include	"AxCryptM.h"
#include	"CVersion.h"
#include	"Utility.h"
#include	"stdio.h"
#include    "CRegistry.h"
//
#define	VERB_ENCRYPTZ	"encryptz"
#define	VERB_ENCRYPT	"encrypt"
#define	VERB_DECRYPT	"decrypt"
#define	VERB_OPEN		"open"
#define	VERB_WIPE		"wipe"
#define VERB_DEBUG      "debug"
#define VERB_CLEARKEYS  "clearkeys"
#define VERB_RENAME     "renameanon"
//
struct CShellExt::SVerbs CShellExt::m_Verbs[] = {
	{VERB_OPEN, INF_MENU_OPEN, HLP_MENU_OPEN, 0, CShellExt::DoOpen},
	{VERB_ENCRYPTZ, INF_MENU_WRAPZ, HLP_MENU_WRAPZ, 0, DoEncryptCompress},
    {VERB_ENCRYPT, INF_MENU_WRAP, HLP_MENU_WRAP, 0, CShellExt::DoEncryptOnly},
	{VERB_DECRYPT, INF_MENU_UNWRAP, HLP_MENU_UNWRAP, 0, CShellExt::DoDecrypt},
	{VERB_WIPE, INF_MENU_WIPE, HLP_MENU_WIPE, 0, CShellExt::DoWipe},
	{VERB_DEBUG, INF_MENU_DEBUG, HLP_MENU_DEBUG, 0, CShellExt::DoDebug},
    {VERB_CLEARKEYS, INF_MENU_CLEARKEYS, HLP_MENU_CLEARKEYS, 0, CShellExt::DoClearKeys},
    {VERB_RENAME, INF_MENU_RENAME, HLP_MENU_RENAME, 0, CShellExt::DoRename},
    {"", 0, 0, -1, CShellExt::DoNothing}
};
//
//  Returns a static buffer that is overwritten for each call
//
static LPTSTR
GetPath(IShellFolder *pShellFolder, LPCITEMIDLIST pidl) {
    static STRRET strret;
    static char szFile[MAX_PATH];
    HRESULT hRes;

    //
    // Getting the display name of certain objects is not supported on Win98, and probably
    // WinNT, notably this occurs when right-clicking a computer in the network neighborhood.
    // Thus we try to be tolerant, but do log the event if it happens.
    //
    hRes = pShellFolder->GetDisplayNameOf(pidl, SHGDN_NORMAL | SHGDN_FORPARSING, &strret);
    if (hRes != NOERROR) {
        CMessage().Wrap(0).AppMsg(MSG_SYSTEM_CALL, CMessage().SysMsg(hRes).GetMsg(), _T("CFileObjectList::GetPath [GetDisplayNameOf]")).LogEvent(1);
        return _T("");
    }

    // Here we do not really know what the format returned will be... Let's assume CSTR for now
    switch (strret.uType) {
    case STRRET_CSTR:
        return strret.cStr;
        break;
    case STRRET_OFFSET:
        CAssert(FALSE).App(MSG_INTERNAL_ERROR, _T("CFileObjectList::GetPath [STRRET_OFFSET]")).Throw();
        break;
    case STRRET_WSTR:
        CAssert(WideCharToMultiByte(CP_ACP, 0, strret.pOleStr, -1, szFile, sizeof szFile, NULL, NULL)).Sys(MSG_SYSTEM_CALL, _T("CFileObjectList::GetPath [WideCharToMultiByte]")).Throw();
        
        gpMalloc->Free(strret.pOleStr);
        return szFile;
        break;
    default:
        CAssert(FALSE).App(MSG_INTERNAL_ERROR, _T("CFileObjectList::GetPath [Unknown]")).Throw();
        break;
    }
    return _T("");
}
//
//  Simple helper to test if a file is possible to open for write,
//  this is to make sure we only refresh the Windows Explorer
//  if necessary.
//
static BOOL
CanOpenForWrite(LPTSTR szFileName) {
    HANDLE hFile = CreateFile(
        szFileName,
        GENERIC_READ | GENERIC_WRITE,
        0,
        NULL,
        OPEN_EXISTING,
        FILE_ATTRIBUTE_NORMAL,
        NULL);
    if (hFile != INVALID_HANDLE_VALUE) CloseHandle(hFile);
    return hFile != INVALID_HANDLE_VALUE;
}
//
//  Return true if it is a regular encrypteable/decryptable file
//
static BOOL
IsFile(IShellFolder *pShellFolder, LPCITEMIDLIST pidl) {
    HRESULT hRes;
    ULONG rgf = SFGAO_FOLDER|SFGAO_FILESYSTEM ;
    hRes = pShellFolder->GetAttributesOf(1, &pidl, &rgf);
    CAssert(hRes == NOERROR, hRes).Sys(MSG_SYSTEM_CALL, _T("CFileObjectList::IsFile [GetAttributesOf]")).Throw();
    // Some tricks are necessary to handle .zip files and other such-like extensions in XP. They are viewed
    // as folders by the shell, but not by the file system. In XP, they do not have the SFGAO_FILESYSANCESTOR
    // set, but then neither does regular folders in Win9x, so we can't use that. Instead we check the
    // actual file attributes too, as we can't be sure just 
    DWORD dwAttrib = GetFileAttributes(GetPath(pShellFolder, pidl));
    if (dwAttrib != 0xFFFFFFFF) {
        // The net effect here is to disregard the shells opinion, if the file system insists that the
        // object is a file. We could probably skip asking the shell completely, but it feels good having
        // this here for the future when this mess maybe gets cleared up and we don't have to support Win9x
        // anymore.
        if ((rgf & SFGAO_FOLDER) == 0 || (dwAttrib & FILE_ATTRIBUTE_DIRECTORY) == 0) {
            if ((dwAttrib & (FILE_ATTRIBUTE_DIRECTORY | 
                             FILE_ATTRIBUTE_HIDDEN |
                             FILE_ATTRIBUTE_OFFLINE |
                             FILE_ATTRIBUTE_SYSTEM)) == 0) {
                return TRUE;
            }
        }
    }

    return FALSE;
}
//
//  Determine if a file is an axcrypt encrypted file.
//
//  This is done by ensuring that it's not a folder, and that the
//  name ends with .axx. In the future, one might go further and actually
//  examine the file itself.
//
static BOOL
IsEncrypted(IShellFolder *pShellFolder, LPCITEMIDLIST pidl) {
    if (IsFile(pShellFolder, pidl)) {
        CFileName fn(GetPath(pShellFolder, pidl));
        return _tcscmp(fn.GetExt(), gszAxCryptFileExt) == 0;
    }
    return FALSE;
}

static BOOL
IsFolder(IShellFolder *pShellFolder, LPCITEMIDLIST pidl) {
    HRESULT hRes;
    ULONG rgf;
    hRes = pShellFolder->GetAttributesOf(1, &pidl, &rgf);
    CAssert(hRes == NOERROR, hRes).Sys(MSG_SYSTEM_CALL, _T("CFileObjectList::IsFolder [GetAttributesOf]")).Throw();
    // Some tricks are necessary to handle .zip files and other such-like extensions in XP. They are viewed
    // as folders by the shell, but not by the file system. In XP, they do not have the SFGAO_FILESYSANCESTOR
    // set, but then neither does regular folders in Win9x, so we can't use that. Instead we do a double
    // test, and also check the actual file attributes.
    if ((rgf & SFGAO_FOLDER) != 0 && (rgf & SFGAO_FILESYSTEM) != 0) {
        DWORD dwAttrib = GetFileAttributes(GetPath(pShellFolder, pidl));
        if (dwAttrib != 0xFFFFFFFF) {
            if ((dwAttrib & FILE_ATTRIBUTE_DIRECTORY) &&
                (dwAttrib & (FILE_ATTRIBUTE_HIDDEN |
                             FILE_ATTRIBUTE_OFFLINE |
                             FILE_ATTRIBUTE_SYSTEM)) == 0) {
                return TRUE;
            }
        }
    }
    return FALSE;
}

CFileObjectList::CFileObjectList() {
    try {
        m_pDataObj = NULL;
        m_pidlShellFolder = NULL;
        m_pShellFolder = NULL;
        //m_pShellView = NULL; // No use any more - explorer gets confused when
        // trying to do refresh from the worker thread, presumably the actual view
        // has changed into a new one (which was the whole point of the worker thread...)
        m_pShellBrowser = NULL;
        m_pDesktopFolder = NULL;

        m_hShellFolderWnd = NULL;
        m_hProgressWnd = NULL;
        m_pCIDA = NULL;
        m_fShellViewHasBeenReset = FALSE;

        HRESULT hRes = SHGetDesktopFolder(&m_pDesktopFolder);
        CAssert(hRes == NOERROR, hRes).Sys(MSG_SYSTEM_CALL, _T("CFileObjectList::CFileObjectList [SHGetDesktopFolder]")).Throw();

        //InitIteration();
    } catch (TAssert utErr) {
        utErr.Show();
    }
}

CFileObjectList::~CFileObjectList() {
    try {
        EndIteration();
        if (m_pDataObj) m_pDataObj->Release();
        if (m_pShellFolder) m_pShellFolder->Release();
        if (m_pidlShellFolder) gpMalloc->Free(m_pidlShellFolder);
        //if (m_pShellView) m_pShellView->Release();
        if (m_pShellBrowser) m_pShellBrowser->Release();
        if (m_pDesktopFolder) m_pDesktopFolder->Release();
    } catch (TAssert utErr) {
        utErr.Show();
    }
}
//
//  Save a refeference pointer to the selection object, and
//  initialize the iteration.
//
void
CFileObjectList::SetObject(IDataObject *pdObj) {
	// We may be called more than once 
	if (m_pDataObj) m_pDataObj->Release(); 

    // Save the object
	if (pdObj) {
		m_pDataObj = pdObj;
		pdObj->AddRef();
        InitIteration();
    } else {
        m_pDataObj = NULL;
    }
}

void
CFileObjectList::SetActiveShellView(HWND hWnd) {
    HRESULT hRes;
    if (hWnd && m_hShellFolderWnd == NULL) {
        m_hShellFolderWnd = hWnd;
	    m_pShellBrowser = (IShellBrowser *)SendMessage(hWnd, WM_USER + 7, 0, 0L);
        if (m_pShellBrowser != NULL) {
            m_pShellBrowser->AddRef();
	        if ((hRes = m_pShellBrowser->QueryActiveShellView(&m_pShellView)) != NOERROR) {
                m_pShellView = NULL;
            }
        }
    }
}
//
//  Return number of items selected
//
int
CFileObjectList::ItemsSelected() {
    return m_pCIDA != NULL ? m_pCIDA->cidl : 0;
}
//
//  TRUE if exactly one _file_ is selected
//
BOOL
CFileObjectList::SelectionIsOneFile() {
    return (ItemsSelected() == 1 &&
            IsFile(m_pShellFolder, (LPCITEMIDLIST)&((char *)m_pCIDA)[m_pCIDA->aoffset[1]]));
}
//
//  TRUE if it is our opinion that the property sheet is appropriate for
//  the selection.
//
BOOL
CFileObjectList::ShowPropertySheet() {
    if (ItemsSelected() == 1) {
        const ITEMIDLIST *pidl = (LPCITEMIDLIST)&((char *)m_pCIDA)[m_pCIDA->aoffset[1]];
        return IsEncrypted(m_pShellFolder, pidl);
    } else {
        return FALSE;
    }
}
//
//  TRUE if it is our opinion that the 'open' menu is appropriate for
//  the selection.
//
BOOL
CFileObjectList::ShowOpenMenu() {
    // If we have a selection and if only one file is selected.
    if (ItemsSelected() == 1) {
        return IsEncrypted(m_pShellFolder, (LPCITEMIDLIST)&((char *)m_pCIDA)[m_pCIDA->aoffset[1]]);
    } else {
        return FALSE;
    }
}
//
//  TRUE if it is our opinion that the 'encrypt' menu is appropriate for
//  the selection.
//
BOOL
CFileObjectList::ShowEncryptMenu() {
    if (ItemsSelected() == 1) {
        const ITEMIDLIST *pidl = (LPCITEMIDLIST)&((char *)m_pCIDA)[m_pCIDA->aoffset[1]];
        if (SelectionIsOneFile()) {
            return !IsEncrypted(m_pShellFolder, pidl);
        } else {
            return IsFolder(m_pShellFolder, pidl);
        }
    } else {
        return ItemsSelected() > 0;
    }
}
//
//  TRUE if it is our opinion that the 'decrypt' menu is appropriate for
//  the selection.
//
BOOL
CFileObjectList::ShowDecryptMenu() {
	if (gfNoDecryptMode) {
		return FALSE;
	}
    if (ItemsSelected() == 1) {
        const ITEMIDLIST *pidl = (LPCITEMIDLIST)&((char *)m_pCIDA)[m_pCIDA->aoffset[1]];
        return IsFolder(m_pShellFolder, pidl) || IsEncrypted(m_pShellFolder, pidl);
    } else {
        return ItemsSelected() > 0;
    }
}
//
//  TRUE if it is our opinion that the 'wipe' menu is appropriate for
//  the selection.
//
BOOL
CFileObjectList::ShowWipeMenu() {
    if (ItemsSelected() == 1) {
        const ITEMIDLIST *pidl = (LPCITEMIDLIST)&((char *)m_pCIDA)[m_pCIDA->aoffset[1]];
        return IsFolder(m_pShellFolder, pidl) || IsFile(m_pShellFolder, pidl);
    } else {
        return ItemsSelected() > 0;
    }
}
//
// TRUE if it is our opinion that the 'rename' menu is appropriate.
//
BOOL
CFileObjectList::ShowRenameMenu() {
	CRegistry utRegKey(HKEY_CURRENT_USER, gszAxCryptRegKey);
    if (utRegKey.Value(szRegValNoRenameMenu).GetDword(FALSE)) {
        return FALSE;
    }

    if (ItemsSelected() == 1) {
        const ITEMIDLIST *pidl = (LPCITEMIDLIST)&((char *)m_pCIDA)[m_pCIDA->aoffset[1]];
        return IsFolder(m_pShellFolder, pidl) || IsEncrypted(m_pShellFolder, pidl);
    } else {
        return ItemsSelected() > 0;
    }
}
//
//  Iterate over the selection, calling the call back command handler, first
//  once to do possible one-time things, and then for each item in the selection,
//  expanding folders into files as appropriate.
//
void
CFileObjectList::IterateAll(CShellExt *pShellExt, pfCmdHandlerT pfCmdHandler) {
    // First let the command handler init or perform once-only operations. We will
    // only continue and iterate if the return value is zero.
    DWORD dwRet = (pShellExt->*pfCmdHandler)(TRUE, NULL, m_pDesktopFolder, m_pidlShellFolder);
    if (dwRet == 0) {
        CProgressDialog dlgProgress;
        m_hProgressWnd = dlgProgress.Create(ghInstance, IDD_PROGRESS, m_hShellFolderWnd);

        LPCITEMIDLIST pidl;
        BOOL fFileSeen = FALSE;
        while (GetWindowLong(GetParent(m_hProgressWnd), GWL_USERDATA) && dwRet == 0 && (pidl = Iterate()) != NULL) {
            fFileSeen = fFileSeen || IsFile(m_pShellFolder, pidl);
            dwRet = IteratePidl(m_pShellFolder, pidl, pShellExt, pfCmdHandler);
        }
        if (fFileSeen) {
            ShellNotify(m_pDesktopFolder, m_pidlShellFolder, SHCNE_UPDATEDIR);
        }
    }
}
//
//  Setup the memory objects for the iteration over the selection. Use the given
//  data object, and get it in the format of a SHELLIDLIST, i.e. an array of
//  pidls.
//
void
CFileObjectList::InitIteration() {
    HRESULT hRes;
    EndIteration();

    FORMATETC fmtetc = {0, (DVTARGETDEVICE FAR *)NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL};
	fmtetc.cfFormat = RegisterClipboardFormat(CFSTR_SHELLIDLIST);

    hRes = m_pDataObj->GetData(&fmtetc, &m_StgMedium);
    CAssert(hRes == S_OK, hRes).Sys(MSG_SYSTEM_CALL, _T("CFileObjectList::InitIteration [GetData]")).Throw();

    CAssert((m_pCIDA = (CIDA *)GlobalLock(m_StgMedium.hGlobal)) != NULL).Sys(MSG_SYSTEM_CALL, _T("CFileObjectList.:InitIteration [GlobalLock]")).Throw();
    // if there is any selection...
	if (m_pCIDA->cidl > 0) {
	    ITEMIDLIST *pidlFolder = (ITEMIDLIST *)&((char *)m_pCIDA)[m_pCIDA->aoffset[0]];
        // If a folder is given...
        if (pidlFolder->mkid.cb) {
            // ... we get a new ShellFolder object for this one.
            IShellFolder *psf;
            hRes = m_pDesktopFolder->BindToObject(pidlFolder, NULL, IID_IShellFolder, (LPVOID *)&psf);
            CAssert(hRes == NOERROR, hRes).Sys(MSG_SYSTEM_CALL, _T("CFileObjectList.:InitIteration [BindToObject]")).Throw();
            m_pShellFolder = psf;
            m_pidlShellFolder = CopyPidl(gpMalloc, pidlFolder);
        }
        m_iItemIndex = 1;
        m_fShellViewHasBeenReset = FALSE;
    }
    if (m_pShellFolder == NULL) {
        m_pShellFolder = m_pDesktopFolder;
        m_pShellFolder->AddRef();
        hRes = SHGetSpecialFolderLocation(m_hShellFolderWnd, CSIDL_DESKTOP, &m_pidlShellFolder);
        CAssert(hRes == NOERROR, hRes).Sys(MSG_SYSTEM_CALL, _T("CFileObjectList::InitIteration [SHGetSpecialFolderLocation]")).Throw();
    }
}
//
//  Get next item from the selection
//
LPCITEMIDLIST
CFileObjectList::Iterate() {
    if (m_pCIDA && m_iItemIndex <= m_pCIDA->cidl) {
		return (LPCITEMIDLIST)&((char *)m_pCIDA)[m_pCIDA->aoffset[m_iItemIndex++]];
    } else {
        return NULL;
    }
}
//
//  Call the call back for an item, expanding recursively into folders as appropriate.
//
DWORD
CFileObjectList::IteratePidl(IShellFolder *pShellFolder, LPCITEMIDLIST pidl, CShellExt *pShellExt, pfCmdHandlerT pfCmdHandler) {
    // If it's a folder, we recurse.
    DWORD dwRet = 0;
    IShellFolder *pShellSubFolder = NULL;
    IEnumIDList *pEnumIDList = NULL;
    try {
        if (IsFolder(pShellFolder, pidl)) {
            HRESULT hRes;
            hRes = pShellFolder->BindToObject(pidl, NULL, IID_IShellFolder, (LPVOID *)&pShellSubFolder);
            CAssert(hRes == NOERROR, hRes).Sys(MSG_SYSTEM_CALL, _T("CFileObjectList::IteratePidl [BindToObject]")).Throw();
        
            hRes = pShellSubFolder->EnumObjects(NULL, SHCONTF_FOLDERS | SHCONTF_NONFOLDERS, &pEnumIDList);
            CAssert(hRes == NOERROR, hRes).Sys(MSG_SYSTEM_CALL, _T("CFileObjectList::IteratePidl [EnumbObjects]")).Throw();

            ITEMIDLIST *pidlEnum;
            while (dwRet == 0 && (hRes = pEnumIDList->Next(1, &pidlEnum, NULL)) == NOERROR) {
                dwRet = IteratePidl(pShellSubFolder, pidlEnum, pShellExt, pfCmdHandler);
                gpMalloc->Free(pidlEnum);
            }
            if (dwRet == 0) {
                CAssert(hRes == S_FALSE, hRes).Sys(MSG_SYSTEM_CALL, _T("CFileObjectList::IteratePidl [Next]")).Throw();
                ShellNotify(pShellFolder, pidl, SHCNE_UPDATEDIR);
            }
            pEnumIDList->Release(); pEnumIDList = NULL;
            pShellSubFolder->Release(); pShellSubFolder = NULL;
            // If not abort due to server error, check that the enumeration ended properly.
        } else if (IsFile(pShellFolder, pidl)) {
            LPSTR szFile = GetPath(pShellFolder, pidl);
            // Try to ensure source writeability by forcing a refresh of the view
            // if there is a problem opening the file for writing.
            if (!m_fShellViewHasBeenReset) {
                if (!CanOpenForWrite(szFile)) {
                    SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST | SHCNF_FLUSH, NULL, NULL);
                    m_fShellViewHasBeenReset = TRUE;
                }
            }
            // Set the progress window file name properly
            SetDlgItemText(GetParent(m_hProgressWnd), IDS_FILE, szFile);
            dwRet = (pShellExt->*pfCmdHandler)(FALSE, m_hProgressWnd, pShellFolder, pidl);
        }
    } catch (TAssert utErr) {
        if (pEnumIDList != NULL) pEnumIDList->Release();
        if (pShellSubFolder != NULL) pShellSubFolder->Release();
        utErr.Throw();
    }
    return dwRet;
}
//
//  Release the memory objects used to iterate over the selection.
//
void
CFileObjectList::EndIteration() {
    if (m_pCIDA) {
        GlobalUnlock(m_pCIDA);
        m_pCIDA = NULL;
        ReleaseStgMedium(&m_StgMedium);
        ZeroMemory(&m_StgMedium, sizeof m_StgMedium);
    }
}
//
//  Notify the shell of a change. Take the pidl, re-interpret it relative to the
//  desktop, and notify.
//
void
CFileObjectList::ShellNotify(IShellFolder *pShellFolder, LPCITEMIDLIST pidl, LONG lEvent) {
    //
    // Due to differences in OS's, we can't reinterpret a pidl that's already desktop relative,
    // as GetDisplayNameOf() will then return 'Desktop' on Win 9x - which is not parseable by ParseDisplayName().
    // It's anyway silly to re-interpret something that's ok to start with...
    //
    if (pShellFolder != m_pDesktopFolder) {
        STRRET strret;
        HRESULT hRes;
        ITEMIDLIST *pidlDesktopRelative = NULL;

        hRes = pShellFolder->GetDisplayNameOf(pidl, SHGDN_NORMAL | SHGDN_FORPARSING, &strret);
        CAssert(hRes == NOERROR, hRes).Sys(MSG_SYSTEM_CALL, _T("CFileObjectList::ShellNotify [GetDisplayNameOf]")).Throw();

        // SHGDN_FORPARSING is a lie! It does not necessarily return a useful string, it's just a hint
        // according to updated information, so we need to be tolerant to this.
        ULONG lEaten;
        switch (strret.uType) {
        case STRRET_WSTR:
            hRes = m_pDesktopFolder->ParseDisplayName(m_hShellFolderWnd, NULL, strret.pOleStr, &lEaten, &pidlDesktopRelative, NULL);
            gpMalloc->Free(strret.pOleStr);
            break;
        case STRRET_CSTR: {
            WCHAR *wzName = CopySzWz(strret.cStr);
            hRes = m_pDesktopFolder->ParseDisplayName(m_hShellFolderWnd, NULL, wzName, &lEaten, &pidlDesktopRelative, NULL);
            delete wzName;
            break;
        }
        default:
            CAssert(FALSE).App(MSG_INTERNAL_ERROR, _T("CFileObjectList::ShellNotify [GetDisplayNameOf]")).Throw();
        }
        // This is not a good situation, but we just log the problem at verbosity level 1, and silently
        // continue.
        if (hRes != NOERROR) {
            CMessage().Wrap(0).AppMsg(MSG_SYSTEM_CALL, CMessage().SysMsg(hRes).GetMsg(), _T("CFileObjectList::ShellNotify [ParseDisplayName]")).LogEvent(1);
            // CAssert(hRes == NOERROR, hRes).Sys(MSG_SYSTEM_CALL, _T("CFileObjectList::ShellNotify [ParseDisplayName]")).Throw();
        } else {
            SHChangeNotify(lEvent, SHCNF_IDLIST | SHCNF_FLUSH, pidlDesktopRelative, NULL);
            gpMalloc->Free(pidlDesktopRelative);
        }
    } else {
        SHChangeNotify(lEvent, SHCNF_IDLIST | SHCNF_FLUSH, pidl, NULL);
    }
    return;
}
//
//	Adds menu items to the specified menu. The menu items should be inserted
//	in the menu at the position specified by indexMenu, and their menu item
//	identifiers must be between the idCmdFirst and idCmdLast parameter values. 
//
//	Returns anHRESULT structure in which, if the method is successful,
//	the code member contains the menu identifier of the last menu item added plus one. 
//	hMenu 
//		Handle to the menu. The handler should specify this handle when
//		adding menu items. 
//	indexMenu
//		Zero-based position at which to insert the first menu item. 
//	idCmdFirst 
//		Minimum value that the handler can specify for a menu item identifier. 
//	idCmdLast 
//		Maximum value that the handler can specify for a menu item identifier. 
//	uFlags 
//		Optional flags specifying how the context menu can be changed.
//		The remaining bits of the low-order word are reserved by the system.
//		The high-order word may be used for context-specific communications.
//
//	The actual identifier of each menu item should be idCmdFirst plus a menu identifier
//	offset in the range zero through (idCmdLast - idCmdFirst).
//
STDMETHODIMP
CShellExt::QueryContextMenu(HMENU hMenu, UINT indexMenu, UINT idCmdFirst, UINT idCmdLast, UINT uFlags) {
    UINT idCmd = idCmdFirst;
    try {
	    if (!(uFlags & CMF_DEFAULTONLY)) {
            try {
                BOOL fDefaultSet = FALSE, fAnyFileOp = FALSE;
	            m_hMenu = CreatePopupMenu();
	            InitVerbs(m_Verbs);

	            InsertMenu(hMenu, indexMenu++, MF_SEPARATOR | MF_BYPOSITION, 0, NULL);
	            InsertMenu(hMenu, indexMenu++, MF_STRING | MF_BYPOSITION | MF_POPUP, (UINT_PTR)m_hMenu, CVersion(ghInstance).ExtProductName());
	            InsertMenu(hMenu, indexMenu++, MF_SEPARATOR | MF_BYPOSITION, 0, NULL);
	            idCmd += 3;

                if (m_pSelection->ShowEncryptMenu()) {
		            SetVerb(m_Verbs, VERB_ENCRYPTZ, idCmd - idCmdFirst);
		            InsertMenu(m_hMenu, -1, MF_STRING | MF_BYPOSITION, idCmd, CMessage().AppMsg(GetMenuMsgId(m_Verbs, idCmd - idCmdFirst)).GetMsg());
			        if (!fDefaultSet && m_pSelection->SelectionIsOneFile()) fDefaultSet = SetMenuDefaultItem(m_hMenu, idCmd, FALSE);
		            idCmd++;
                    fAnyFileOp = TRUE;
	            }

                if (m_pSelection->ShowDecryptMenu()) {
		            SetVerb(m_Verbs, VERB_DECRYPT, idCmd - idCmdFirst);
		            InsertMenu(m_hMenu, -1, MF_STRING | MF_BYPOSITION, idCmd, CMessage().AppMsg(GetMenuMsgId(m_Verbs, idCmd - idCmdFirst)).GetMsg());
			        if (!fDefaultSet && m_pSelection->SelectionIsOneFile()) fDefaultSet = SetMenuDefaultItem(m_hMenu, idCmd, FALSE);
                    idCmd++;
                    fAnyFileOp = TRUE;
	            }

                if (m_pSelection->ShowOpenMenu()) {
		            SetVerb(m_Verbs, VERB_OPEN, idCmd - idCmdFirst);
		            InsertMenu(m_hMenu, -1, MF_STRING | MF_BYPOSITION, idCmd, CMessage().AppMsg(GetMenuMsgId(m_Verbs, idCmd - idCmdFirst)).GetMsg());
			        if (!fDefaultSet && m_pSelection->SelectionIsOneFile()) fDefaultSet = SetMenuDefaultItem(m_hMenu, idCmd, FALSE);
		            idCmd++;
                    fAnyFileOp = TRUE;
                }

                if (m_pSelection->ShowRenameMenu()) {
		            SetVerb(m_Verbs, VERB_RENAME, idCmd - idCmdFirst);
		            InsertMenu(m_hMenu, -1, MF_STRING | MF_BYPOSITION, idCmd, CMessage().AppMsg(GetMenuMsgId(m_Verbs, idCmd - idCmdFirst)).GetMsg());
		            idCmd++;
                    fAnyFileOp = TRUE;
                }

                // Insert a separator if necessary after file operations before key clear
                if (fAnyFileOp) {
                    InsertMenu(m_hMenu, -1, MF_SEPARATOR | MF_BYPOSITION, 0, NULL);
                }
                SetVerb(m_Verbs, VERB_CLEARKEYS, idCmd - idCmdFirst);
	            InsertMenu(m_hMenu, -1, MF_STRING | MF_BYPOSITION, idCmd, CMessage().AppMsg(GetMenuMsgId(m_Verbs, idCmd - idCmdFirst)).GetMsg());
	            idCmd++;

                if (m_pSelection->ShowWipeMenu()) {
                    InsertMenu(m_hMenu, -1, MF_SEPARATOR | MF_BYPOSITION, 0, NULL);
	                
	                SetVerb(m_Verbs, VERB_WIPE, idCmd - idCmdFirst);
	                InsertMenu(m_hMenu, -1, MF_STRING | MF_BYPOSITION, idCmd, CMessage().AppMsg(GetMenuMsgId(m_Verbs, idCmd - idCmdFirst)).GetMsg());
	                idCmd++;
                }
#ifdef _DEBUG
                SetVerb(m_Verbs, VERB_DEBUG, idCmd - idCmdFirst);
	            InsertMenu(m_hMenu, -1, MF_STRING | MF_BYPOSITION, idCmd, CMessage().AppMsg(GetMenuMsgId(m_Verbs, idCmd - idCmdFirst)).GetMsg());
	            idCmd++;
#endif
            } catch (TAssert utErr) {
	            utErr.Show();
	            return (HRESULT)(MAKE_SCODE(SEVERITY_SUCCESS, 0, 0));
            }
        }
    } catch (TAssert utErr) {
        utErr.Show();
    }
	// Make the next index in the menu the success resultcode from this method.
	return (HRESULT)(MAKE_SCODE(SEVERITY_SUCCESS, 0, (WORD)(idCmd - idCmdFirst)));
}
//
//	Carries out the command associated with a context menu item. 
//
//	Returns NOERROR if successful, or an OLE-defined error code otherwise. 
//	lpici 
//		Address of a CMINVOKECOMMANDINFO structure containing information
//		about the command.
//
//	The shell calls this method when the user chooses a command that the
//	handler added to a context menu. This method may also be called by an
//	application without any corresponding user action
// 
STDMETHODIMP
CShellExt::InvokeCommand(LPCMINVOKECOMMANDINFO lpici) { 
    HRESULT hRes = E_INVALIDARG;
	// if we are called by the shell, invoke the right action depending
	// on which menu item that was selected.
    char *szVerb = NULL;

    try {
        pfCmdHandlerT pfCmdHandler = NULL;
        if (lpici->hwnd) m_pSelection->SetActiveShellView(lpici->hwnd);

	    // If extended structure (why make this so hard???)...
	    if (lpici->cbSize == sizeof CMINVOKECOMMANDINFOEX &&
		    // ...and if Unicode...
		    (lpici->fMask & CMIC_MASK_UNICODE) &&
			    // ...and a verb, not a cmd...
			HIWORD(((LPCMINVOKECOMMANDINFOEX)lpici)->lpVerbW)) {

		    // ... then Do Uni-code based verb look-up
		    int iLen = WideCharToMultiByte(CP_ACP, 0, ((LPCMINVOKECOMMANDINFOEX)lpici)->lpVerbW, -1, NULL, 0, NULL, NULL);
		    // Not generic here!
		    
		    szVerb = new char[iLen];
		    (void)WideCharToMultiByte(CP_ACP, 0, ((LPCMINVOKECOMMANDINFOEX)lpici)->lpVerbW, -1, szVerb, iLen, NULL, NULL);
            pfCmdHandler = DoVerbByVerb(m_Verbs, szVerb);
        } else if (HIWORD(lpici->lpVerb)) {
		    // Do Ansi based verb look-up
            pfCmdHandler = DoVerbByVerb(m_Verbs, (LPTSTR)lpici->lpVerb);
	    } else {
            pfCmdHandler = DoVerbByCmd(m_Verbs, LOWORD(lpici->lpVerb));
	    }
        if (pfCmdHandler != NULL) {
            DWORD dwThreadId;
            HANDLE hThread;
            AddRef();
            m_pfCmdHandler = pfCmdHandler;
            CAssert((hThread = CreateThread(NULL, 0, CommandThread, this, 0, &dwThreadId)) != NULL).Sys(MSG_SYSTEM_CALL, _T("CShellExt::InvokeCommand [CreateThread]")).Throw();
        }
    } catch (TAssert utErr) {
        utErr.Show();
        hRes = E_INVALIDARG;
    }
    if (szVerb != NULL) delete szVerb;
	return hRes; 
} 
//
//	Retrieves the language-independent command string or the
//	Help text for a context menu item. 
//
//	Returns NOERROR if successful, or an OLE-defined error code otherwise.
//
//	idCmd 
//		Menu item identifier offset. 
//	uFlags 
//		Flags specifying the information to retrieve. This can be one of
//		the following values:
//			GCS_HELPTEXT	Returns the Help text for the menu item.  
//			GCS_VALIDATE	Validates that the menu item exists.  
//			GCS_VERB		Returns the language-independent command
//							name for the menu item.  
//	pwReserved 
//		Reserved. Applications must specify NULL when calling this method,
//		and handlers must ignore this parameter when called. 
//	pszName 
//		Address of the buffer to receive the null-terminated string
//		being retrieved. 
//	cchMax 
//		Size of the buffer to receive the null-terminated string.
//
//	The language-independent command name can be passed to the
//	IContextMenu::InvokeCommand method to activate a command by an application.
//
//	The Help text is a description that Windows Explorer displays in its status bar;
//	it should be reasonably short (under 40 characters). 
// 
STDMETHODIMP
CShellExt::GetCommandString(UINT idCmd,	UINT uFlags, UINT *pwReserved, LPSTR pszName, UINT cchMax) {
        try {
	        if (!IsValidCmd(m_Verbs, idCmd)) {
		        return NOERROR;
	        }
	        switch (uFlags) {
	        case GCS_HELPTEXTA:
		        lstrcpynA(pszName, CMessage().AppMsg(GetVerbByCmd(m_Verbs, idCmd)->dwHlp).GetMsg(), cchMax);
		        break;
	        case GCS_HELPTEXTW: {
			        CMessage utMsg;
			        utMsg.AppMsg(GetVerbByCmd(m_Verbs, idCmd)->dwHlp);
			        WCHAR *wszMsg = CopySzWz(utMsg.GetMsg());
			        lstrcpynW((LPWSTR)pszName, wszMsg, cchMax);
			        delete wszMsg;
		        }
		        break;
	        case GCS_VERBA:
		        lstrcpynA(pszName, GetVerbByCmd(m_Verbs, idCmd)->szVerb, cchMax);
		        break;
	        case GCS_VERBW: {
                    WCHAR *wszMsg = CopySzWz(GetVerbByCmd(m_Verbs, idCmd)->szVerb);
		            lstrcpynW((LPWSTR)pszName, wszMsg, cchMax);
		            delete wszMsg;
	            }
            }
        } catch (TAssert utErr) {
            utErr.Show();
        }
	return NOERROR; 
} 
//
//	Init verb-command id association
//
void
CShellExt::InitVerbs(SVerbs *pVerbs) {
	while (pVerbs->szVerb[0]) {
		pVerbs->iCmd = -1;
		pVerbs++;
	}
}
//
//	Set verb-command id association
//
void
CShellExt::SetVerb(SVerbs *pVerbs, char *szVerb, int iCmd) {
	while (pVerbs->szVerb[0]) {
		if (strcmp(pVerbs->szVerb, szVerb) ==0) {
			pVerbs->iCmd = iCmd;
			return;
		}
		pVerbs++;
	}
}

BOOL
CShellExt::IsValidCmd(SVerbs *pVerbs, int iCmd) {
	while (pVerbs->szVerb[0]) {
		if (pVerbs->iCmd == iCmd) {
			return TRUE;
		}
		pVerbs++;
	}
	return FALSE;
}

DWORD
CShellExt::GetMenuMsgId(SVerbs *pVerbs, int iCmd) {
	while (pVerbs->szVerb[0]) {
		if (pVerbs->iCmd == iCmd) {
			return pVerbs->dwCmd;
		}
		pVerbs++;
	}
	return INF_MENU_ERROR;
}

CShellExt::SVerbs *
CShellExt::GetVerbByCmd(SVerbs *pVerbs, int iCmd) {
	while (pVerbs->szVerb[0]) {
		if (pVerbs->iCmd == iCmd) {
			return pVerbs;
		}
		pVerbs++;
	}
	return pVerbs;
}
//
//	Execute by command id association
//
pfCmdHandlerT
CShellExt::DoVerbByCmd(SVerbs *pVerbs, int iCmd) {
	while (pVerbs->szVerb[0]) {
		if (pVerbs->iCmd == iCmd) {
			return pVerbs->pfCmdHandler;
		}
		pVerbs++;
	}
	return NULL;
}
//
//	Execute by verb association
//
pfCmdHandlerT
CShellExt::DoVerbByVerb(SVerbs *pVerbs, char *szVerb) {
	while (pVerbs->szVerb[0]) {
		if (strcmp(pVerbs->szVerb, szVerb) == 0) {
			return pVerbs->pfCmdHandler;
		}
		pVerbs++;
	}
	return NULL;
}
//
//  Called from the worker thread, this is the function that actually does
//  the work. We need the iteration, as the CreateThread cannot call a member
//  function directly.
//
void
CShellExt::IterateSelection() {
    // Ensure that OLE does not unload our DLL prematurely!
    //InterlockedIncrement(&glRefThisDLL); 

    // Assign a batch-id to this operation
    SetBatch();

    // Iterate over the selection, giving call-back function and 'this'
    m_pSelection->IterateAll(this, m_pfCmdHandler);

    // Clear batch keys.
    CallAxCrypt(NULL, _T(" -t"));
    
    // Decrement our own reference counter.
    Release();
    // Also release the hold on the DLL it-self.
    //InterlockedDecrement(&glRefThisDLL); 
}
//
//  Just an intermediary for the worker thread.
//
/*static*/ DWORD WINAPI
CShellExt::CommandThread(LPVOID lParam) {
    try {
        ((CShellExt *)lParam)->IterateSelection();
    } catch (TAssert utErr) {
        utErr.Show();
    }
    return 0;
}

DWORD
CShellExt::DoEncryptCompress(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile) {
    if (!fInit) {
        if (!IsEncrypted(pShellFolder, pidlFile)) {
            return DoAxCrypt(hProgressWnd, _T(" -z "), pShellFolder, pidlFile);
        }
    }
	return 0;
}

DWORD
CShellExt::DoDecrypt(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile) {
    if (!fInit) {
        if (IsEncrypted(pShellFolder, pidlFile)) {
	        return DoAxCrypt(hProgressWnd, _T(" -d "), pShellFolder, pidlFile);
        }
    }
	return 0;
}

DWORD
CShellExt::DoWipe(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile) {
    if (!fInit) {
        if (IsFile(pShellFolder, pidlFile)) {
            return DoAxCrypt(hProgressWnd, _T(" -w "), pShellFolder, pidlFile);
        }
    }
	return 0;
}

DWORD
CShellExt::DoOpen(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile) {
    if (!fInit) {
        if (IsEncrypted(pShellFolder, pidlFile)) {
	        return DoAxCrypt(hProgressWnd, _T(" "), pShellFolder, pidlFile);
        }
    }
    return 0;
}

DWORD
CShellExt::DoDebug(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile) {
    if (fInit) {
        if (MessageBox(NULL, _T("Initalizing iteration, continue?"), _T("AxCrypt Shell Extension Debug"), MB_YESNO) == IDYES) {
            return 0;
        } else {
            return WRN_CANCEL;
        }
    } else {
        MessageBox(NULL, GetPath(pShellFolder, pidlFile), _T("AxCrypt Shell Extension Debug"), MB_OK);
    }
    return 0;
}

DWORD
CShellExt::DoClearKeys(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile) {
    if (fInit) {
        // Reset the batch-number to zero, then abort. This causes nothing to happen in
        // the iteration, but the end-batch will still be executed, now with value '0'...
        SetBatch(0);
    }
    return WRN_CANCEL;
}

DWORD
CShellExt::DoRename(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile) {
    if (!fInit) {
        if (IsEncrypted(pShellFolder, pidlFile)) {
	        return DoAxCrypt(hProgressWnd, _T(" -h "), pShellFolder, pidlFile);
        }
    }
	return 0;
}

DWORD
CShellExt::DoNothing(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile) {
	return 0;
}

DWORD
CShellExt::DoEncryptOnly(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile) {
    if (!fInit) {
        if (!IsEncrypted(pShellFolder, pidlFile)) {
	        return DoAxCrypt(hProgressWnd, _T(" -e "), pShellFolder, pidlFile);
        }
    }
	return 0;
}
//
//	Actually perform  an action by calling the main process, adding the
//  batch identifier in the process...
// 
DWORD
CShellExt::CallAxCrypt(HWND hProgressWnd, LPTSTR szParams) {
    CFileName szPath2Exe;

	szPath2Exe.SetPath2ExeName(ghInstance).DelExt().AddExt(_T(".exe"));
	STARTUPINFO StartupInfo;
	PROCESS_INFORMATION ProcessInformation;
    DWORD dwReturn;
    CStrPtr szCmd = CStrPtr(szPath2Exe.GetQuoted()) +
                    CStrPtr(_T("-b ")) +
                    CStrPtr(GetBatchStr()) +
                    CStrPtr(szParams);
    ZeroMemory(&StartupInfo, sizeof StartupInfo);
	ZeroMemory(&ProcessInformation, sizeof ProcessInformation);
	StartupInfo.cb = sizeof StartupInfo;

    if (!CreateProcess(
		NULL,
		(LPTSTR)szCmd,
		NULL,
		NULL,
		0,
		CREATE_SUSPENDED,
		NULL,
		NULL,
		&StartupInfo,
		&ProcessInformation)) {
        CAssert(FALSE).Sys().Show();
        DWORD dwLastError = GetLastError();			// Just for debug, ignore right now.
        return dwLastError;
	}
    // Mark the progress-window with the correct process-ID.
    if (hProgressWnd != NULL) {
        SetWindowLong(hProgressWnd, GWL_USERDATA, ProcessInformation.dwProcessId);
    }
    ResumeThread(ProcessInformation.hThread);
    DlgMessageWaitForSingleObject(GetParent(hProgressWnd), ProcessInformation.hProcess, INFINITE);

    if (!GetExitCodeProcess(ProcessInformation.hProcess, &dwReturn)) {
        MessageBox(NULL, _T("Could not get exit code"), _T("AxCrypt Shell Extension"), MB_OK);
        CMessage().SysMsg(GetLastError()).ShowError();
        return GetLastError();
    }
    CloseHandle(ProcessInformation.hProcess);
    CloseHandle(ProcessInformation.hThread);
    return dwReturn;
}

DWORD
CShellExt::DoAxCrypt(HWND hProgressWnd, LPTSTR szOption, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile) {
    CStrPtr szCmd = CStrPtr(szOption) + CStrPtr(CFileName(GetPath(pShellFolder, pidlFile)).GetQuoted());
    return CallAxCrypt(hProgressWnd, szCmd);	
} 

//
//  Derive a reasonably unique batch id for this operation.
//  Zero is a reserved value meaning all, thus we avoid
//  that one.
//
void
CShellExt::SetBatch(int iBatch) {
    if (iBatch == -1) {
        m_iBatch = (GetTickCount() & 0x7fffffff) | 1;
    } else {
        m_iBatch = iBatch;
    }

    if (m_szBatch != NULL) delete m_szBatch;
    
    // Calculate the length of the resulting string, in chars, including null.
    int iStrLen = 1, j = m_iBatch;
    do {
        j /= 10;
        iStrLen++;
    } while (j);

    m_szBatch = new TCHAR[iStrLen];
    CAssert(m_szBatch != NULL).App(MSG_MEMORY_ALLOC, _T("CShellExt::GetBatchStr() [m_szBatch]")).Throw();
    (void)itoa(m_iBatch, m_szBatch, 10);
}
//
//  Get the current batch id into a dynamically allocated
//  string, and return the pointer. Please do not delete
//  it...
//
LPTSTR
CShellExt::GetBatchStr() {
    return m_szBatch;
}
