#if			!defined(_SHELLEXTENSION)
#define		_SHELLEXTENSION
/*
    @(#) $Id: ShellExt.h,v 1.9 2002/08/23 16:42:07 svante Exp $

	AxCrypt - Compressing and Encrypting Wrapper and Application Launcher for Secure Local,
	Server or Web Storage of Document Files.

	Copyright (C) 2001 Svante Seleborg/Axon Data, All rights reserved.

	This program is free software; you can redistribute it and/or modify it under the terms
	of the GNU General Public License as published by the Free Software Foundation;
	either version 2 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along with this program;
	if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
	Boston, MA 02111-1307 USA

	The author may be reached at mailto:axcrypt@axondata.se and http://axcrypt.sourceforge.net
----
	ShellExt.h      				Definition of Class Factory and Object for the AxCrypt
									Shell Extension.

	E-mail							YYYY-MM-DD				Reason
	axcrypt@axondata.se	    		2001					Initial
                                    2002-07-25              Ver 1.2
*/
#ifndef		STRICT 
#define		STRICT 
#endif 
 
#define		INC_OLE2
#include	<windows.h> 

#include	<windowsx.h> 
#include	<shlobj.h>
#include	<tchar.h>
//
#include    "resource.h"
#include	"Utility.h"
#include	"AxCommon.h"
#include	"CAssert.h"
#include	"AxCryptM.h"
// 
// The class ID of the AxCrypt Shell Extension class. 
// 
// class id:  1918AECB-5BDB-43D9-83B4-FDD4BEC67E4F 
// 
// 
extern long glRefThisDLL;					// Reference count of this DLL. 
extern HINSTANCE ghInstance;				// Handle to this DLL itself.
extern IMalloc *gpMalloc;
//
//  Helpers
//
extern void DecrementDllReference();
extern void IncrementDllReference();
//
//	Our very own Class Factory - used and required by the shell
//
class CShellExtClassFactory : public IClassFactory { 
protected: 
	ULONG m_cRef;
public: 
	CShellExtClassFactory(); 
	~CShellExtClassFactory(); 

	// IUnknown
	STDMETHODIMP QueryInterface(REFIID iid, void **ppvObject); 
	STDMETHODIMP_(ULONG) AddRef(); 
	STDMETHODIMP_(ULONG) Release(); 

	// IClassFactory
	STDMETHODIMP CreateInstance(IUnknown * pUnkOuter, REFIID riid, void ** ppvObject);
	STDMETHODIMP LockServer(BOOL fLock);
};
typedef CShellExtClassFactory *LPCSHELLEXTCLASSFACTORY;
//
//	Forward declaration: Cmd Handler function type
//
class CShellExt;
typedef DWORD (CShellExt::*pfCmdHandlerT)(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile);
//
//  Support class to handle all interaction with the list of files
//  we get from the Windows Explorer.
//
class CFileObjectList {
public:
    CFileObjectList();
    ~CFileObjectList();

    void SetObject(IDataObject *pdobj);
    void SetActiveShellView(HWND hWnd);
    
    int ItemsSelected();
    BOOL SelectionIsOneFile();
    BOOL ShowPropertySheet();
    BOOL ShowOpenMenu();
    BOOL ShowEncryptMenu();
    BOOL ShowDecryptMenu();
    BOOL ShowWipeMenu();
    BOOL ShowRenameMenu();
    BOOL ShowReEncryptMenu() { return FALSE;}
    
    void IterateAll(CShellExt *pShellExt, pfCmdHandlerT pfCmdHandler);

private:
    void InitIteration();
    LPCITEMIDLIST Iterate();
    DWORD IteratePidl(IShellFolder *pShellFolder, LPCITEMIDLIST pidl, CShellExt *pShellExt, pfCmdHandlerT pfCmdHandler);
    void EndIteration();
    void ShellNotify(IShellFolder *pShellFolder, LPCITEMIDLIST pidl, LONG lEvent);

    IShellFolder *m_pDesktopFolder;
    IShellBrowser *m_pShellBrowser;
    IDataObject *m_pDataObj;
    IShellFolder *m_pShellFolder;
    ITEMIDLIST *m_pidlShellFolder;
    IShellView *m_pShellView;

    CIDA *m_pCIDA;
    STGMEDIUM m_StgMedium;
    
    HWND m_hShellFolderWnd;
    UINT m_iItemIndex;
    BOOL m_fShellViewHasBeenReset;
    HWND m_hProgressWnd;
};
//
// The shell extension object as such 
//
class CShellExt : public IShellExtInit, IContextMenu, IShellPropSheetExt {
public:
	CShellExt(); 
	~CShellExt(); 
 
	// IUnknown
	STDMETHODIMP QueryInterface(REFIID iid, void **ppvObject);
	STDMETHODIMP_(ULONG) AddRef();
	STDMETHODIMP_(ULONG) Release();
 
	// IShellExtInit
	STDMETHODIMP Initialize(LPCITEMIDLIST pidlFolder, IDataObject *pdobj, HKEY hkeyProgID);	

	// IContextMenu
	STDMETHODIMP QueryContextMenu(HMENU hMenu, UINT indexMenu, UINT idCmdFirst, UINT idCmdLast, UINT uFlags);
 	STDMETHODIMP InvokeCommand(LPCMINVOKECOMMANDINFO lpici); 
 	STDMETHODIMP GetCommandString(UINT idCmd, UINT uFlags, UINT *pwReserved, LPSTR pszName, UINT cchMax);
 
	// IShellPropSheetExt
	STDMETHODIMP AddPages(LPFNADDPROPSHEETPAGE lpfnAddPage, LPARAM lParam); 
	STDMETHODIMP ReplacePage(UINT uPageID, LPFNADDPROPSHEETPAGE lpfnReplaceWith, LPARAM lParam); 

private:
    int m_iBatch;               // The batch-id assigned to this operation.
    LPTSTR m_szBatch;           // The string representation of the batch id
	ULONG m_cRef;               // The object reference counter
	HMENU m_hMenu;              // Handle to the context menu, when relevant.
public:
    CFileObjectList *m_pSelection;// The list of selected object,and iteration context etc
    pfCmdHandlerT m_pfCmdHandler;
private:
    //
    //	Verbs list w/commands. Notable: Use explicit ASCII-strings here, we
    //	convert to Unicode when necessary.
    //
    static struct SVerbs {
	    char *szVerb;
	    DWORD dwCmd;
	    DWORD dwHlp;
	    int iCmd;
        pfCmdHandlerT pfCmdHandler;
    } m_Verbs[];

    void InitVerbs(SVerbs *pVerbs);
    void SetVerb(SVerbs *pVerbs, char *szVerb, int iCmd);
    BOOL IsValidCmd(SVerbs *pVerbs, int iCmd);
    DWORD GetMenuMsgId(SVerbs *pVerbs, int iCmd);
    SVerbs *GetVerbByCmd(SVerbs *pVerbs, int iCmd);
    pfCmdHandlerT DoVerbByCmd(SVerbs *pVerbs, int iCmd);
    pfCmdHandlerT DoVerbByVerb(SVerbs *pVerbs, char *szVerb);

    static DWORD WINAPI CommandThread(LPVOID lParam);
    void IterateSelection();

    DWORD DoEncryptCompress(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile);
    DWORD DoDecrypt(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile);
    DWORD DoWipe(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile);
    DWORD DoOpen(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile);
    DWORD DoDebug(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile);
    DWORD DoClearKeys(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile);
    DWORD DoRename(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile);
    DWORD DoNothing(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile);
    DWORD DoEncryptOnly(BOOL fInit, HWND hProgressWnd, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile);

public:
    DWORD CallAxCrypt(HWND hProgressWnd, LPTSTR szParams);
private:
	DWORD DoAxCrypt(HWND hProgressWnd, LPTSTR szOption, IShellFolder *pShellFolder, LPCITEMIDLIST pidlFile);
public:
    void SetBatch(int iBatch = -1);
private:
    LPTSTR GetBatchStr();
}; 
typedef CShellExt *LPCSHELLEXT; 
//
//	Do not include heap check-points in the shell extension.
//
#define	HEAP_CHECK_BEGIN(Where, AllowedLeak)
#define	HEAP_CHECK_END
#endif		_SHELLEXTENSION
